package cologne.eck.all_peas.vm_specific;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2020  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

import java.awt.Window;
import java.net.MalformedURLException;
import java.util.Locale;

import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.tools.TestLog;
import cologne.eck.tools.UnexpectedValueException;

public class JREProperties extends PeaProperties {
	

	// special panel for PswDialogView, currently only used in PeaProperties.getFileType():
	private static Object typePanel;

	private static Window mainWindow = null;
	
	/**
	 * size and location
	 */
	//private static Dimension frameDim = new Dimension(550,450);
	private static int[] frameDim = {550, 450};// = new Dimension(550,450)
	//private static Point frameLocation = new Point(100, 30);
	private static int[] frameLocation = {100, 30};// = new Point(100, 30);
	
	//private static ResourceBundle languagesBundle;
	private static Object languagesBundle;	
	
	/**
	 * Get the current main window: PswDialogView
	 * or current instance of PeaLockFrame
	 * 
	 * @return	current main window
	 */
	public static Window getMainWindow() {
		if (mainWindow == null) {
			if (PeaControl.isPlainTextMode() == false) {
				mainWindow = PswDialogView.getView();
			} else {
				if ( PeaControl.getDialog().getLockFrame() instanceof Window) {
					mainWindow = (Window) PeaControl.getDialog().getLockFrame();
				}
			}
		}
		return mainWindow;
	}
	/**
	 * Set the current main window: PswDialogView
	 * or current instance of PeaLockFrame
	 * 
	 * @param win	the current main window
	 */
	public static void setMainWindow(Window win) {
		JREProperties.mainWindow = win;
	}
	
	/**
	 * @return the typePanel
	 */
	public static Object getTypePanel() {
		return typePanel;
	}
	/**
	 * @param typePanel the typePanel to set
	 */
	public static void setTypePanel(Object typePanel) {
		JREProperties.typePanel = typePanel;
	}
	
	/**
	 * Get the current language bundle: Cast to ResourceBundle.
	 * If no bundle was set before, it is set with default language
	 * 
	 * @return		the current language bundle
	 */
	public final static Object getBundle(){
		if (languagesBundle == null) {
			try {
				PeaProperties.getVmBridge().setLanguagesBundle(
						new Locale( System.getProperty("user.language")).getLanguage());// default language
			} catch (MalformedURLException e) {
				TestLog.e(JREProperties.class, e.toString() + " - " + e.getLocalizedMessage());
			}
		}
		return languagesBundle;
	}
	/**
	 * Set the languageBundle
	 * @param _bundle
	 */
	protected final static void setBundle(Object _bundle) {
		languagesBundle = _bundle;
	}

	
	/**
	 * Get frame dimension as width and height values
	 * 
	 * @return	width and height values in int[]
	 */
	public static int[] getFrameDim() {
		return frameDim;
	}
	/**
	 * int[] with width and height
	 * 
	 * @param whValues width and height in int[]
	 */
	public static void setFrameDim(int[] whValues) {
		if (whValues.length != 2) {
			new UnexpectedValueException("whValue", "int[]", "is invalid").printDescription();
		}
		JREProperties.frameDim = whValues;
	}
	/**
	 * Get location of frame as x/y
	 * 
	 * @return	x,y value of Point
	 */
	public static int[] getFrameLocation() {
		return frameLocation;
	}
	/**
	 * Set Location as x,y values
	 * 
	 * @param xyValues	must be instance of awt.Point
	 */
	public static void setFrameLocation(int[] xyValues) {
		if (xyValues.length != 2) {
			new UnexpectedValueException("xyValue", "int[]", "is invalid").printDescription();
		}
		JREProperties.frameLocation = xyValues;
	}


}
