package cologne.eck.calendar_pea.gui;

/*
 * PeaFactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.ResourceBundle;

import javax.accessibility.AccessibleContext;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.plaf.FontUIResource;

import cologne.eck.all_peas.cloud.CloudControl;
import cologne.eck.all_peas.cloud.CloudPropertyHandler;
import cologne.eck.all_peas.cloud.StorageProvider;
import cologne.eck.all_peas.cloud.gui.CloudMenu;
import cologne.eck.all_peas.cloud.gui.UploadWorker;
import cologne.eck.all_peas.control.PathFileManager;
import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.files.FileTypePanel;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.gui.PeaDialog;
import cologne.eck.all_peas.gui.PeaFileChooser;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.all_peas.gui.menu.HelpMenu;
import cologne.eck.all_peas.gui.menu.PeaMenuBar;
import cologne.eck.all_peas.gui.menu.SettingMenu;
import cologne.eck.all_peas.vm_specific.JREProperties;
import cologne.eck.calendar_pea.control.CalendarConstants;
import cologne.eck.calendar_pea.control.CalendarControl;
import cologne.eck.calendar_pea.control.CalendarPEAControl;
import cologne.eck.calendar_pea.control.CalendarSetting;
import cologne.eck.calendar_pea.data.PCalendar;
import cologne.eck.calendar_pea.gui.calendar_view.MonthPanel;
import cologne.eck.calendar_pea.gui.calendar_view.WeekPanel;
import cologne.eck.calendar_pea.gui.event_display.EventButton;
import cologne.eck.calendar_pea.gui.event_setting.DatePicker;
import cologne.eck.calendar_pea.util.ReadICal;
import cologne.eck.calendar_pea.util.WriteICal;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.peafactory.crypto.kdf.KeyDerivation;
import cologne.eck.tools.Converter;
import cologne.eck.tools.FileTools;
import cologne.eck.tools.ReadResources;
import cologne.eck.tools.TestLog;
import cologne.eck.tools.UnexpectedValueException;
import cologne.eck.tools.WriteResources;


@SuppressWarnings("serial")
public
class CalendarMenuBar  extends PeaMenuBar implements ActionListener {

	/*
	 * Item to perform saving the all calendars
	 */
	private JMenuItem saveItem = null;
	private JMenuItem saveAsItem = null;
	private JMenuItem defaultFileItem = null;
	//private boolean askPathFile = false;// ask to remember file, when previously stored in cloud
	
	private JSpinner yearSpinner; 
	
	private CloudMenu cloudMenu;// to fire save action programmatically
	
	// dialog with list of calendars to hide or show
	private JDialog calendarListDialog;
	
	
	private ResourceBundle calLanguagesBundle = CalendarSetting.getResourceBundle();
	private ResourceBundle peaLanguagesBundle = (ResourceBundle) JREProperties.getBundle();
	
	public CalendarMenuBar() {
		

		Font font = new Font(Font.SANS_SERIF, Font.PLAIN, PeaProperties.getFontSize());

		JMenu fileMenu = new JMenu(calLanguagesBundle.getString("file"));// File");//\u00FC");
		fileMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a menu for basic calendar operations like new, save, import, export.");
		fileMenu.setMnemonic(calLanguagesBundle.getString("file").charAt(0));
		fileMenu.setFont(font);
//		fileMenu.setMnemonic(KeyStroke.getKeyStroke(PeaProperties.getOsBridge().translate("file_mnemonic")).getKeyCode());
		this.add(fileMenu);
		
		JMenuItem newItem = new JMenuItem(calLanguagesBundle.getString("new"));//languagesBundle.getString("instruction"));
		newItem.getAccessibleContext().
	    setAccessibleDescription(
	    "Clicking this component opens a dialog to create a new calendar with name and color.");
		newItem.setActionCommand("new");
		newItem.setFont(font);
		//colorItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("instruction_mnemonic")).getKeyCode());
		newItem.addActionListener(this);
		fileMenu.add(newItem);
		
/*		JMenuItem openItem = new JMenuItem(languagesBundle.getString("open"));//languagesBundle.getString("instruction"));
		openItem.setActionCommand("open");
		openItem.getAccessibleContext().
    		setAccessibleDescription(
    		"Clicking this component opens a calendar.");
		openItem.setFont(font);
		//colorItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("instruction_mnemonic")).getKeyCode());
		openItem.addActionListener(this);
		fileMenu.add(openItem);*/
		
		saveItem = new JMenuItem(calLanguagesBundle.getString("save"));//languagesBundle.getString("instruction"));
		saveItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component saves all open calendars in the default file.");
		saveItem.setActionCommand("save");
		saveItem.setFont(font);
		//colorItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("instruction_mnemonic")).getKeyCode());
		saveItem.addActionListener(this);
		fileMenu.add(saveItem);		
		
		saveAsItem = new JMenuItem(calLanguagesBundle.getString("save_as"));//languagesBundle.getString("instruction"));
		saveAsItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component starts a dialog to save all open calendars in a specific file.");
		saveAsItem.setActionCommand("saveAs");
		saveAsItem.setFont(font);
		//colorItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("instruction_mnemonic")).getKeyCode());
		saveAsItem.addActionListener(this);
		fileMenu.add(saveAsItem);		
		
		JMenuItem importItem = new JMenuItem(calLanguagesBundle.getString("import"));//languagesBundle.getString("instruction"));
		importItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a file chooser to select a calendar file to import.");
		importItem.setActionCommand("import");
		importItem.setFont(font);
		//colorItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("instruction_mnemonic")).getKeyCode());
		importItem.addActionListener(this);
		fileMenu.add(importItem);		
		
		JMenuItem exportItem = new JMenuItem(calLanguagesBundle.getString("export"));//languagesBundle.getString("instruction"));
		exportItem.getAccessibleContext().
	    setAccessibleDescription(
	    		"Clicking this component opens a file chooser to select a calendar file to export one calendar unencrypted.");
		exportItem.setActionCommand("export");
		exportItem.setFont(font);
		//colorItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("instruction_mnemonic")).getKeyCode());
		exportItem.addActionListener(this);
		fileMenu.add(exportItem);
		
		
		defaultFileItem = new JMenuItem(calLanguagesBundle.getString("default_folder"));//languagesBundle.getString("instruction"));
		defaultFileItem.setToolTipText(calLanguagesBundle.getString("choose_default_location"));
		defaultFileItem.getAccessibleContext().
	    setAccessibleDescription(
	    		"Clicking this component opens a file chooser to select the default folder to save calendars.");
		defaultFileItem.setActionCommand("default-folder");
		defaultFileItem.setFont(font);
		//colorItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("instruction_mnemonic")).getKeyCode());
		defaultFileItem.addActionListener(this);
		fileMenu.add(defaultFileItem);
		
		JMenuItem quitItem = new JMenuItem(calLanguagesBundle.getString("quit"));//languagesBundle.getString("instruction"));
		quitItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component saves all open calendars and exits the program.");
		quitItem.setActionCommand("quit");
		quitItem.setFont(font);
		//colorItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("instruction_mnemonic")).getKeyCode());
		quitItem.addActionListener(this);
		fileMenu.add(quitItem);
		

		JMenu navigationMenu  = new JMenu(calLanguagesBundle.getString("navigation"));//PeaProperties.getOsBridge().translate("file"));// File");//\u00FC");
		navigationMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens possibilities to navigate to a specific year, month or date.");
		navigationMenu.setMnemonic(calLanguagesBundle.getString("navigation").charAt(0));
		navigationMenu.setFont(font);
	//	editMenu.setMnemonic(KeyStroke.getKeyStroke(PeaProperties.getOsBridge().translate("edit_mnemonic")).getKeyCode());
		this.add(navigationMenu);
		
		JMenu monthMenu = new JMenu(calLanguagesBundle.getString("month"));
		monthMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a month spinner to navigate to a specific month.");
		monthMenu.setFont(font);
		String[] monthStrings = DatePicker.generateMonthStrings();
		for (int i = 0; i < 12; i++) {
			JMenuItem item = new JMenuItem(monthStrings[i]);
			item.addActionListener(this);
			item.setActionCommand("setMonth" + monthStrings[i]);
			monthMenu.add(item);
		}
		navigationMenu.add(monthMenu);	
		
		JMenu yearItem = new JMenu(calLanguagesBundle.getString("year"));//languagesBundle.getString("instruction"));
		yearItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a year spinner to navigate to a specific year.");
		yearItem.setFont(font);
		//yearItem.setActionCommand("setYear");
		//colorItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("instruction_mnemonic")).getKeyCode());
		//yearItem.addActionListener(this);
		JPanel yearPanel= new JPanel();
        SpinnerModel yearModel = new SpinnerNumberModel(LocalDate.now().getYear(), //initial value
                0, //min
                5000, //max
                1);                //step
        JLabel yearLabel = new JLabel(CalendarSetting.getResourceBundle().getString("year"));
        yearLabel.setFont(font);
		yearSpinner = new JSpinner( yearModel);
		yearSpinner.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, PeaProperties.getFontSize() + 2));
		// remove the dot after thousand and add whitespace to right
		JSpinner.NumberEditor ne = new JSpinner.NumberEditor(yearSpinner,"#");
		yearSpinner.setEditor(ne);
		if (ne.getTextField() instanceof JFormattedTextField) {
			((JFormattedTextField)ne.getTextField()).setHorizontalAlignment(JTextField.CENTER);//.setAlignmentX(CENTER_ALIGNMENT);
		}
		yearSpinner.setPreferredSize(new Dimension(120, 30));
		JButton setYearButton = new JButton("ok");
		setYearButton.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component cases a navigation to the specified date.");
		setYearButton.setFont(font);
		setYearButton.addActionListener(this);
		setYearButton.setActionCommand("setYear");
		yearPanel.add(yearLabel);
		yearPanel.add(yearSpinner);
		yearPanel.add(setYearButton);
		yearItem.add(yearPanel);
		navigationMenu.add(yearItem);	
				
		JMenuItem datePickerItem = new JMenuItem(calLanguagesBundle.getString("go_to_date"));//languagesBundle.getString("instruction"));
		datePickerItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a date picker to select a specific date to navigate to.");
		datePickerItem.setActionCommand("setBeginDate");
		datePickerItem.setFont(font);
		//colorItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("instruction_mnemonic")).getKeyCode());
		datePickerItem.addActionListener(this);
		navigationMenu.add(datePickerItem);	
		
		
JMenuItem searchItem = new JMenuItem(peaLanguagesBundle.getString("search"));//languagesBundle.getString("instruction"));
searchItem.getAccessibleContext().
setAccessibleDescription(
	    "Clicking this component opens a dialog to search for key words.");
searchItem.setActionCommand("search");
searchItem.setFont(font);
//colorItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("instruction_mnemonic")).getKeyCode());
searchItem.addActionListener(this);
navigationMenu.add(searchItem);	
		

		JMenu editMenu = new JMenu(calLanguagesBundle.getString("edit"));//PeaProperties.getOsBridge().translate("file"));// File");//\u00FC");
		editMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens items to edit calendars like removing, renaming, color and password.");
		editMenu.setMnemonic(calLanguagesBundle.getString("edit").charAt(0));
		editMenu.setFont(font);
	//	editMenu.setMnemonic(KeyStroke.getKeyStroke(PeaProperties.getOsBridge().translate("edit_mnemonic")).getKeyCode());
		this.add(editMenu);
		
		JMenuItem removeCalendarItem = new JMenuItem(calLanguagesBundle.getString("remove_calendar"));//languagesBundle.getString("instruction"));
		removeCalendarItem.getAccessibleContext().
	    setAccessibleDescription(
	    "Clicking this component opens a dialog to select and to delete a calendar.");
		removeCalendarItem.setActionCommand("removeCalendar");
		removeCalendarItem.setFont(font);
		//colorItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("instruction_mnemonic")).getKeyCode());
		removeCalendarItem.addActionListener(this);
		editMenu.add(removeCalendarItem);	
		
		JMenuItem calendarColorItem = new JMenuItem(calLanguagesBundle.getString("change_color"));//languagesBundle.getString("instruction"));
		calendarColorItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a dialog to select a calendar to change its color.");
		calendarColorItem.setActionCommand("calendarColor");
		calendarColorItem.setFont(font);
		//colorItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("instruction_mnemonic")).getKeyCode());
		calendarColorItem.addActionListener(this);
		editMenu.add(calendarColorItem);		
		
		JMenuItem renameItem = new JMenuItem(calLanguagesBundle.getString("rename_calendar"));//languagesBundle.getString("instruction"));
		renameItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a dialog to select a calendar to change its name.");
		renameItem.setActionCommand("rename");
		renameItem.setFont(font);
		//colorItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("instruction_mnemonic")).getKeyCode());
		renameItem.addActionListener(this);
		editMenu.add(renameItem);
		
		
		JMenuItem changePswItem = new JMenuItem(PeaProperties.getVmBridge().translate("change_password"));//languagesBundle.getString("instruction"));
		changePswItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a dialog to set a new password to encrypt the calendars.");
		changePswItem.setActionCommand("changePsw");
		changePswItem.setFont(font);
		//colorItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("instruction_mnemonic")).getKeyCode());
		changePswItem.addActionListener(this);
		editMenu.add(changePswItem);

		
		JMenu viewMenu = new JMenu(calLanguagesBundle.getString("view"));//PeaProperties.getOsBridge().translate("file"));// File");//\u00FC");
		viewMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens items to modify the view of the calendar like color, font size, start time.");
		//appearanceMenu.setMnemonic(KeyStroke.getKeyStroke(PeaProperties.getOsBridge().translate("file_mnemonic")).getKeyCode());
		viewMenu.setMnemonic(calLanguagesBundle.getString("view").charAt(0));
		viewMenu.setFont(font);
		this.add(viewMenu);
		
		JMenuItem manageCalendarItem = new JMenuItem(calLanguagesBundle.getString("hide_show_calendars"));//languagesBundle.getString("instruction"));
		manageCalendarItem.getAccessibleContext().
	    setAccessibleDescription(
	    "Clicking this component opens a dialog with calendar to hide or show.");
		manageCalendarItem.setActionCommand("manageCalendars");
		manageCalendarItem.setFont(font);
		//colorItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("instruction_mnemonic")).getKeyCode());
		manageCalendarItem.addActionListener(this);
		viewMenu.add(manageCalendarItem);	
		
		
		JMenuItem calendarFilesItem = new JMenuItem(calLanguagesBundle.getString("calendars_and_files"));//languagesBundle.getString("instruction"));
		calendarFilesItem.getAccessibleContext().
	    setAccessibleDescription(
	    "Clicking this component shows the files of a calendar.");
		calendarFilesItem.setActionCommand("calendarFiles");
		calendarFilesItem.setFont(font);
		//colorItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("instruction_mnemonic")).getKeyCode());
		calendarFilesItem.addActionListener(this);
		viewMenu.add(calendarFilesItem);	
		
		
		JMenu viewSettingMenu = new JMenu(calLanguagesBundle.getString("view_settings"));//PeaProperties.getOsBridge().translate("file"));// File");//\u00FC");
		viewSettingMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens items to modify the view");
		//appearanceMenu.setMnemonic(KeyStroke.getKeyStroke(PeaProperties.getOsBridge().translate("file_mnemonic")).getKeyCode());
		viewSettingMenu.setMnemonic(calLanguagesBundle.getString("view_settings").charAt(0));
		viewSettingMenu.setFont(font);
		viewMenu.add(viewSettingMenu);
		
		JMenuItem dayColorItem = new JMenuItem(calLanguagesBundle.getString("background_days"));//languagesBundle.getString("instruction"));
		dayColorItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a dialog to choose a color for the background of the calendar days.");
		dayColorItem.setActionCommand("daysColor");
		dayColorItem.setFont(font);
		//colorItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("instruction_mnemonic")).getKeyCode());
		dayColorItem.addActionListener(this);
		viewSettingMenu.add(dayColorItem);		
		
		JMenuItem buttonFontSizeItem = new JMenuItem(calLanguagesBundle.getString("font_size_event"));//languagesBundle.getString("instruction"));
		buttonFontSizeItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a dialog to set the font size of the event buttons.");
		buttonFontSizeItem.setActionCommand("eventButtonFont");
		buttonFontSizeItem.setFont(font);
		//colorItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("instruction_mnemonic")).getKeyCode());
		buttonFontSizeItem.addActionListener(this);
		viewSettingMenu.add(buttonFontSizeItem);		
		
		JMenuItem startTimeItem = new JMenuItem(calLanguagesBundle.getString("start_time_day"));//languagesBundle.getString("instruction"));
		startTimeItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a dialog to select the time, the calendar view starts with.");
		startTimeItem.setActionCommand("startTimeDay");
		startTimeItem.setFont(font);
		//colorItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("instruction_mnemonic")).getKeyCode());
		startTimeItem.addActionListener(this);
		viewSettingMenu.add(startTimeItem); 
		
		JMenuItem numberOfLinesItem = new JMenuItem(calLanguagesBundle.getString("number_weeks"));//languagesBundle.getString("instruction"));
		numberOfLinesItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a dialog to choose the number of lines of days, a month view should show.");
		numberOfLinesItem.setActionCommand("weekNumber");
		numberOfLinesItem.setFont(font);
		//colorItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("instruction_mnemonic")).getKeyCode());
		numberOfLinesItem.addActionListener(this);
		viewSettingMenu.add(numberOfLinesItem); 
		
		// General PA setting:
		
		JMenuItem increaseFontSizeItem = new JMenuItem(peaLanguagesBundle.getString("enlarge_font"));//languagesBundle.getString("cleanup"));
		increaseFontSizeItem.setActionCommand("increaseFontSize");
		increaseFontSizeItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component increases the font size for this program.");
		increaseFontSizeItem.setFont(font);
//		increaseFontSizeItem.setToolTipText(languagesBundle.getString("enlarge_font"));
		//cleanupItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("about_mnemonic")).getKeyCode());
		increaseFontSizeItem.addActionListener(this);
		viewSettingMenu.add(increaseFontSizeItem);
		
		JMenuItem decreaseFontSizeItem = new JMenuItem(peaLanguagesBundle.getString("reduce_font"));//languagesBundle.getString("cleanup"));
		decreaseFontSizeItem.setActionCommand("decreaseFontSize");
		decreaseFontSizeItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component decreases the font size for this program.");
		decreaseFontSizeItem.setFont(font);
//		increaseFontSizeItem.setToolTipText(languagesBundle.getString("tooltip_cleanup"));
		//cleanupItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("about_mnemonic")).getKeyCode());
		decreaseFontSizeItem.addActionListener(this);
		viewSettingMenu.add(decreaseFontSizeItem);
		
		JMenuItem colorItem = new JMenuItem(peaLanguagesBundle.getString("background_color"));//languagesBundle.getString("cleanup"));
		colorItem.setActionCommand("setColor");
		colorItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a color chooser to set the background color for this program.");
		colorItem.setFont(font);
//		increaseFontSizeItem.setToolTipText(languagesBundle.getString("tooltip_cleanup"));
		//cleanupItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("about_mnemonic")).getKeyCode());
		colorItem.addActionListener(this);
		viewSettingMenu.add(colorItem);
				
		
		//this.add(Box.createHorizontalGlue());
//		this.setPreferredSize(new Dimension((int)this.getPreferredSize().getWidth(), 35));
//		this.setMaximumSize(new Dimension((int)this.getPreferredSize().getWidth() + 30, 35));
//		this.setMinimumSize(new Dimension(300, 30));

//		this.setBorder(PeaBorderFactory.createBorder(true));
		
		cloudMenu = new CloudMenu(LockFrameCalendar.getFrame());
		this.add(cloudMenu);
		
		SettingMenu settingMenu = new SettingMenu(LockFrameCalendar.getFrame());
		this.add(settingMenu);
		
		HelpMenu helpMenu = new HelpMenu(LockFrameCalendar.getFrame());
		this.add(helpMenu);
	}
	@Override
	public void actionPerformed(ActionEvent ae) {
		
		ResourceBundle languagesBundle = CalendarSetting.getResourceBundle();
		String command = ae.getActionCommand();
		// reset message label:
		((LockFrameCalendar)PeaControl.getDialog().getLockFrame() ).setMessage(null,  false);
		if (command.equals("import")) {

			try {

				int previousEventNumber = CalendarControl.getNumberOfAllShownEvents();
				PeaFileChooser pfc = new PeaFileChooser(this);
				pfc.customizeFileChooser(languagesBundle.getString("import"), languagesBundle.getString("select_calendar_file"), true);
				FileFilter filter = new FileNameExtensionFilter("iCalendar", "ics");
				pfc.setFileFilter(filter);
				pfc.showOpenDialog(this);
				File file = pfc.getSelectedFile();
				if (file != null && file.canRead() && file.isFile()) {
					// Read calendars from file
					// get content of file: 
					byte[] content = ReadResources.readExternFile(file.getAbsolutePath());
					TestLog.v(getClass(), "Read imported file " + file.getAbsoluteFile());
					PCalendar[] cals = new ReadICal().readCalendarsFromBytes(content, null, false);
					if (cals == null) {
						((LockFrameCalendar)PeaControl.getDialog().getLockFrame() ).setMessage(languagesBundle.getString("no_valid_calendar_found"), true);
						return;
					}
					// add all calendars
					int len = cals.length;
					for (int i = 0; i < len; i++) {
						// add to models 1. associated file name for calendar, 2. file-calendar map
						CalendarPEAControl.setFileNameforCalendar(cals[i], this);
						// display calendars
						CalendarControl.showCalendar(cals[i], file.getAbsolutePath());
					}
				}
				if (ReadICal.isNoCalendarAdded() == true) {
					((LockFrameCalendar)PeaControl.getDialog().getLockFrame() ).setMessage(CalendarSetting.getResourceBundle().getString("no_valid_calendar_found"), true);
				}
				else if (ReadICal.isInvalidFormatFound()) {
					if (ReadICal.isUnsupportedPropertiesFound() == true) {
						((LockFrameCalendar)PeaControl.getDialog().getLockFrame() ).setMessage(CalendarSetting.getResourceBundle().getString("errors_occurred") 
								+ ": " +  CalendarSetting.getResourceBundle().getString("unsupported_properties")
								+ ", " + CalendarSetting.getResourceBundle().getString("invalid_format"), true);
					} else {
						((LockFrameCalendar)PeaControl.getDialog().getLockFrame() ).setMessage(CalendarSetting.getResourceBundle().getString("errors_occurred") 
								+ ": " + CalendarSetting.getResourceBundle().getString("invalid_format"), true);
					}
				} else if (ReadICal.isUnsupportedPropertiesFound()) {
					((LockFrameCalendar)PeaControl.getDialog().getLockFrame() ).setMessage(CalendarSetting.getResourceBundle().getString("errors_occurred") 
							+ ": " +  CalendarSetting.getResourceBundle().getString("unsupported_properties"), true);

				} else {
					int addedEvents = CalendarControl.getNumberOfAllShownEvents() - previousEventNumber;
					((LockFrameCalendar)PeaControl.getDialog().getLockFrame() ).setMessage(CalendarSetting.getResourceBundle().getString("events_added") + ": " + addedEvents, false);
				}
				//LockFrameCalendar.setDocChangeUnsaved(true);
				if (LockFrameCalendar.getFrame() != null) {
					LockFrameCalendar.getFrame().markUnsavedContentChanges();
				}
			} catch (Exception e) {
				PeaDialog.showMessage(this, CalendarSetting.getResourceBundle().getString("import") +"\n" 
						+ CalendarSetting.getResourceBundle().getString("operation_failed") + "\n"
						+ e.getLocalizedMessage());
				TestLog.e(getClass(), e.getLocalizedMessage() + ", " + e.toString());
				e.printStackTrace();
			}
		} else if (command.equals("new")) {	
			
			PeaControl.getDialog().initializeNewFiles();
			CalendarControl.refreshView();

			((LockFrameCalendar)PeaControl.getDialog().getLockFrame() ).setMessage(CalendarSetting.getResourceBundle().getString("new"), false);// + ": " + newCalendarName, false);
			//LockFrameCalendar.setDocChangeUnsaved(true);
			if (LockFrameCalendar.getFrame() != null) {
				LockFrameCalendar.getFrame().markUnsavedContentChanges();
			}
			
/*		} else if (command.equals("open")) {
			
			JFileChooser pfc = new JFileChooser();
			FileFilter filter = new FileNameExtensionFilter("iCalendar", "ics");
			pfc.setFileFilter(filter);
			pfc.showOpenDialog(this);
			File file = pfc.getSelectedFile();
			
			if (file != null) {
				TestLog.o(getClass(), "opened file: " + file.getAbsolutePath());

				PCalendar cal = new ReadICal().readEncryptedCalendarFile(file);

				if (cal != null) {				
					CalendarControl.showCalendar(cal);
				} else {
					TestLog.e(getClass(), "Open Calendar failed");
				}
			} */
		} else if (command.equals("export")) {
			// get names of all shown calendars
			String[] calendarNames = CalendarControl.getShownCalendarNameList();		
			String selectedCalendarName = null;
			// check if exists:
			if (calendarNames == null || calendarNames.length == 0) {
				PeaDialog.showMessage(this, languagesBundle.getString("calendar_missing")
						+ "\n → " + languagesBundle.getString("file")
						+ "\n    → " + languagesBundle.getString("new")
						);
				return;
			}
			// select one calendar
			if (calendarNames != null && calendarNames.length > 0) {
				selectedCalendarName = (String) JOptionPane.showInputDialog(LockFrameCalendar.getFrame(), 
						languagesBundle.getString("export_calendar"),
						languagesBundle.getString("export_calendar"),
						JOptionPane.QUESTION_MESSAGE, 
						null, 
						calendarNames, 
						calendarNames[0]);
				if (selectedCalendarName == null || selectedCalendarName.equals("") ) {
					TestLog.e(getClass(), "No calendar to export selected...");
					return;
				}
			}

			// open file chooser to select a directory to store files
			PeaFileChooser pfc = new PeaFileChooser(this);
			pfc.setFileSelectionMode(PeaFileChooser.DIRECTORIES_ONLY);
			pfc.setApproveButtonText(languagesBundle.getString("select"));
			pfc.customizeFileChooser(languagesBundle.getString("export"), languagesBundle.getString("select_folder_for_calendar_file"), false);
			pfc.showOpenDialog(this);
			// get selected directory
			File directory = pfc.getSelectedFile();

			if (directory != null && directory.isDirectory()) {
			
				// if file already exists: ask to overwrite
				if (((FileTypePanel)JREProperties.getTypePanel()).askToOverwriteFile(new File(directory.getAbsolutePath()+ File.separator + selectedCalendarName + ".ics"), false) == false) {
					return;
				}				
				PCalendar cal =  CalendarControl.getCalendarByName(selectedCalendarName); 
				// select a format for export:
				String[] calendarFormats = { languagesBundle.getString("format_5545_extensions"),
						languagesBundle.getString("format_7986")};
				String selectedCalendarFormat = (String) JOptionPane.showInputDialog(LockFrameCalendar.getFrame(), 
						languagesBundle.getString("select_calendar_format"),
						languagesBundle.getString("calendar_format"),
						JOptionPane.QUESTION_MESSAGE, 
						null, 
						calendarFormats, 
						calendarFormats[0]);
				if (selectedCalendarFormat == null) {
					return;
				}
				boolean newFormat = false;
				if (selectedCalendarFormat.equals(languagesBundle.getString("format_5545_extensions"))) {
					newFormat = false;
				} else {
					newFormat = true;
				}
				String calString = WriteICal.singleICalToString(cal, newFormat);
				boolean success = false;
				success = WriteResources.write(calString.getBytes(CalendarConstants.getCharset()), cal.getName() + ".ics", directory.getAbsolutePath());
				if (success == false) {
					((LockFrameCalendar)PeaControl.getDialog().getLockFrame() ).setMessage(languagesBundle.getString("operation_failed"), true);
					PeaDialog.showMessage(this, languagesBundle.getString("operation_failed") + "\n"
							+  WriteResources.getLastErrorMessage(), languagesBundle.getString("error"), 0);
					return;
				} else {
					((LockFrameCalendar)PeaControl.getDialog().getLockFrame() ).setMessage(CalendarSetting.getResourceBundle().getString("export") + ": " + directory.getAbsolutePath() + File.separator + cal.getName() + ".ics", false);
				}

			} else {
				TestLog.e(getClass(), "No directory selected to export calendar...");
			}
		} else if (command.equals("default-folder")) {
			
			// get existing default file
			String defaultFolder = CalendarSetting.getDefaultFolder();
			String firstText = null;
			if (defaultFolder == null
					|| new File(defaultFolder).exists() == false) {
				firstText = "";
			} else {
				firstText = languagesBundle.getString("change_default_folder") + "\n"
						+ defaultFolder + "\n\n";
			}
			//
			// open file chooser to select a directory to store file
			PeaFileChooser pfc = new PeaFileChooser(this);
			pfc.customizeFileChooser(languagesBundle.getString("save"), firstText + languagesBundle.getString("choose_default_location"), false);
			pfc.setApproveButtonText(languagesBundle.getString("select"));
			pfc.setMultiSelectionEnabled(false);
			pfc.setFileSelectionMode(PeaFileChooser.DIRECTORIES_ONLY);
			pfc.showOpenDialog(this);
			// get selected directory
			File directory = pfc.getSelectedFile();

			if (directory != null) {
				CalendarSetting.setDefaultFolder(directory.getAbsolutePath());
			} else {
				TestLog.e(getClass(), "No directory chosen to store calendars...");
				return;
			}

			((LockFrameCalendar)PeaControl.getDialog().getLockFrame() ).
			setMessage(languagesBundle.getString("default_folder") + ": " + directory.getAbsolutePath(), false);
			//LockFrameCalendar.setDocChangeUnsaved(true);
			if (LockFrameCalendar.getFrame() != null) {
				LockFrameCalendar.getFrame().markUnsavedContentChanges();
			}

		} else if (command.equals("quit")) {
			try {
				((LockFrameCalendar)PeaControl.getDialog().getLockFrame()).close();//checkUnsavedTextChanges();
			} catch (Exception e) {
				int result = PeaDialog.showQuestion(this, e.getLocalizedMessage() + " - " + e.toString()
						+ "\n   " + PeaProperties.getVmBridge().translate("quit") + "?", 
						PeaProperties.getVmBridge().translate("error"), 0);
				if (result == 0) {
					System.exit(0);
				}
			} finally {
				System.exit(0);
			}

			// clear the sessionKeys:
		//	CipherStuff.getInstance().getSessionKeyCrypt().clearKeys();
		//	System.exit(0);

		} else if (command.equals("saveAs")) {			
			
			// choose calendar to save:
			String calendarName = selectCalendar(calLanguagesBundle.getString("select_calendar"));
			if (calendarName == null) {
				return;
			}
			
			// choose a file to store the encrypted calendars:
			PeaFileChooser pfc = new PeaFileChooser(this);
			pfc.customizeFileChooser(languagesBundle.getString("save_as"), languagesBundle.getString("select_calendar_file"), false);
			FileFilter filter = new FileNameExtensionFilter("iCalendar", "ics");
			pfc.setApproveButtonText(languagesBundle.getString("select"));
			//pfc.setSelectedFile(new File(CalendarSetting.getDefaultFileName()));
			String dir = CalendarSetting.getDefaultFolder();
			if (dir != null && new File(dir).exists()) {
				pfc.setCurrentDirectory(new File(dir));
			}
			pfc.setFileFilter(filter);
			pfc.showOpenDialog(this);
			File file = pfc.getSelectedFile();
			// check if file was selected:
			if (file == null) {
				TestLog.e(getClass(), "No file to save calendars selected...");
				return;
			} else {
				
				// rename file if extension is not .ics
				if (! file.getName().endsWith(".ics")) {
					file = new File(file.getAbsolutePath() + ".ics");
				}	
				String fileName = file.getAbsolutePath();
				// if file already exists: ask to overwrite
				if (((FileTypePanel)JREProperties.getTypePanel()).askToOverwriteFile(file, false) == false) {
					return;
				}				
				// find calendar by selected name:
				PCalendar c = CalendarControl.getCalendarByName(calendarName);
				// get the calendar as String:
				String calAsString = WriteICal.iCalToString(c);
				// convert to bytes:
				byte[] plainBytes = calAsString.getBytes(PeaProperties.getCharset());
				// encrypt:
				byte[] cipherBytes = CipherStuff.getInstance().encrypt(plainBytes, 
						CipherStuff.getSessionKeyCrypt().getKey(), // keyMaterial is overwritten
						true, null); // key is stored encrypted							
				// write local:
				boolean success = WriteResources.write(cipherBytes,fileName, null);
				if (success == false) {
					TestLog.e(getClass(), "Save calendar failed: " + calendarName + " in " + fileName);
					PeaDialog.showMessage(this, 
							peaLanguagesBundle.getString("operation_failed") + ":\n"
							+ peaLanguagesBundle.getString("save") + ": " + fileName, 
							peaLanguagesBundle.getString("error"), 0);		
					((LockFrameCalendar)PeaControl.getDialog().getLockFrame() )
						.setMessage(languagesBundle.getString("operation_failed"), true);					
					return;
				} else {
					TestLog.o(getClass(), "Calendar " + calendarName + " saved in " + fileName);					
					// add to path file
//					String[] fileNameToAdd = new String[1];
//					fileNameToAdd[0] = fileName;
//					PathFileManager.addFilesToPathFile(fileNameToAdd);					
					// add to associated file names
					c.addAssociatedFileName(fileName);	
					CalendarControl.addToFileCalendarsMap(fileName, c);
					// to remember in path file:
					PeaControl.addAvailableFileName(fileName);
					// show message:
					((LockFrameCalendar)PeaControl.getDialog().getLockFrame() ).setMessage(languagesBundle.getString("save_as") + ": " 
							+ fileName, false);
				}	
			}

		} else if (command.equals("save")) {
			
			String error = saveAllCalendars();
			if (error == null) {
				// show message:
				((LockFrameCalendar)PeaControl.getDialog().getLockFrame() ).setMessage(peaLanguagesBundle.getString("save") + ": " 
					+ peaLanguagesBundle.getString("success"), false);
				//LockFrameCalendar.setDocChangeUnsaved(false);
				if (LockFrameCalendar.getFrame() != null) {
					LockFrameCalendar.getFrame().markContentIsSaved();
				}
			} else {
				PeaDialog.showMessage(this, 
						peaLanguagesBundle.getString("operation_failed") + ":\n"
						+ peaLanguagesBundle.getString("save"), 
						peaLanguagesBundle.getString("error"), 0);		
				((LockFrameCalendar)PeaControl.getDialog().getLockFrame() )
					.setMessage(languagesBundle.getString("operation_failed"), true);	
			}

/********************************
// TODO
			try {
				cologne.eck.js_cal.JS_Setter.setJsFile(21);
			} catch (Exception e) {
				// 
				StackTraceElement[] ste = Thread.currentThread().getStackTrace();
				String lines = "";
				for (int i=0; i < 5;i++) {
					lines += ste[i].toString() + "\n";
				}
				PeaDialog.showInputDialog(this, "Couldn't set...\n"
						+ e.getLocalizedMessage() +"\n"
						+ e.toString() +"\n"
						+ lines, null, 0);
				e.printStackTrace();
			}
***********************************/

// ======================= Navigation Menu ===========================
			
		} else if (command.startsWith("setMonth")) {
			String month = command.substring(8);
			String[] months = DatePicker.generateMonthStrings();
			int newMonth = 0;
			// check which name was selected
			for (int i = 0;i < 12; i++) {
				if (month.equals(months[i])) {
					newMonth = i;
					break;
				}
			}	
			boolean oldDocStatus = LockFrameCalendar.getFrame().isContentUnsaved();//LockFrameCalendar.isDocChangeUnsaved();
			LockFrameCalendar.getView().setFirstShownDate(LockFrameCalendar.getView().getFirstShownDate().withMonth(newMonth + 1));// +1 because newMonth is index
			CalendarControl.refreshView();
			if (oldDocStatus == true) {
				LockFrameCalendar.getFrame().markUnsavedContentChanges();
			} else {
				LockFrameCalendar.getFrame().markContentIsSaved();
			}
			if (LockFrameCalendar.getView() instanceof WeekPanel) {
				LockFrameCalendar.getFrame().pack();
			}
			
		} else if (command.equals("setYear")) {
			LockFrameCalendar.getView().setFirstShownDate(LockFrameCalendar.getView().getFirstShownDate().withYear((int) yearSpinner.getValue()));
			boolean oldDocStatus = LockFrameCalendar.getFrame().isContentUnsaved();//isDocChangeUnsaved();
			CalendarControl.refreshView();
			if (LockFrameCalendar.getView() instanceof WeekPanel) {
				LockFrameCalendar.getFrame().pack();
			}
			if (oldDocStatus == true) {
				LockFrameCalendar.getFrame().markUnsavedContentChanges();
			} else {
				LockFrameCalendar.getFrame().markContentIsSaved();
			}
			// menu must be closed manually:
			javax.swing.MenuSelectionManager.defaultManager().clearSelectedPath();

		} else if (command.equals("setBeginDate")) {

			LocalDate x = new DatePicker(LocalDate.now())
					.showDatePicker(this, languagesBundle.getString("go_to_date"));
			LockFrameCalendar.getView().setFirstShownDate(x);
			boolean oldDocStatus = LockFrameCalendar.getFrame().isContentUnsaved();//LockFrameCalendar.isDocChangeUnsaved();
			CalendarControl.refreshView();
			if (LockFrameCalendar.getView() instanceof WeekPanel) {
				LockFrameCalendar.getFrame().pack();
			}
			if (oldDocStatus == true) {
				LockFrameCalendar.getFrame().markUnsavedContentChanges();
			} else {
				LockFrameCalendar.getFrame().markContentIsSaved();
			}
			
		} else if (command.equals("search")) {

			TestLog.v(getClass(), "Search...");
			new EventSearcher(this).search();


//========================= EDIT MENU ================================

        	
		} else if (command.equals("rename")) {

			String selectedCalendarName = selectCalendar(languagesBundle.getString("rename_calendar"));
			if (selectedCalendarName == null || selectedCalendarName.equals("")) {
				return;
			}
			String newCalendarName = PeaDialog.showInputDialog(JREProperties.getMainWindow(), languagesBundle.getString("type_name"), 
						languagesBundle.getString("new"), -1);

				if (newCalendarName == null || newCalendarName.equals("") ) {
					return;
				} else {
					if (CalendarControl.getCalendarByName(newCalendarName) == null) {
						CalendarControl.getCalendarByName(selectedCalendarName).setName(newCalendarName);
					} else {
						PeaDialog.showMessage(JREProperties.getMainWindow(), languagesBundle.getString("calendar_exists"), 
								languagesBundle.getString("operation_failed"), 0);
						return;
					}					
				}
			((LockFrameCalendar)PeaControl.getDialog().getLockFrame() ).setMessage(CalendarSetting.getResourceBundle().getString("rename_calendar") + ": "
			+ selectedCalendarName + " -> " + newCalendarName, false);
			//LockFrameCalendar.setDocChangeUnsaved(true);
			LockFrameCalendar.getFrame().markUnsavedContentChanges();
			
		} else if (command.equals("removeCalendar")) {	
			
			String selectedCalendarName = selectCalendar(languagesBundle.getString("remove_calendar"));

			//TestLog.o(getClass(), "Remove " + selectedCalendarName);
			if (selectedCalendarName == null || selectedCalendarName.equals("") ) {
				return;
			}
			PCalendar c = CalendarControl.getCalendarByName(selectedCalendarName);
			String[] cFiles = c.getAssociatedFileNames();
			
			for (int i = 0; i < cFiles.length;i++) {
				// remove file, if there are no other calendars in the file
				if (CalendarControl.getCalendarsOfFileAsSet(cFiles[i]).size() == 1) {
					CalendarControl.removeFileFromFileCalendarsMap(cFiles[i]);
					PeaControl.removeAvailableFileName(cFiles[i]);
					if (cFiles[i].contains(CloudControl.getTmpDir())) { // cloud file
						String prov = CloudControl.getProviderNameFromTmpFile(cFiles[i]);
						String fName = CloudControl.getFileNameFromTmpFile(cFiles[i]);
						if (prov != null) {// 
							CloudPropertyHandler.removeFileOfProvider(prov, fName,
									Converter.bytes2hex(KeyDerivation.getSalt()), 
									Converter.bytes2hex(PeaProperties.getAlgoParamString()), null);
						}
					} else { // local file
						PathFileManager.removeFileName(cFiles[i]);
					}
				}
			}

			CalendarControl.removeCalendar(CalendarControl.getCalendarByName(selectedCalendarName));
			((LockFrameCalendar)PeaControl.getDialog().getLockFrame() ).setMessage(CalendarSetting.getResourceBundle().getString("remove_calendar") + ": "
					+ selectedCalendarName , false);

			CalendarControl.refreshView();
			
			// Ask to delete files (wipe)
			StringBuilder fileListAsString = new StringBuilder();
			for (String f : cFiles) {
				fileListAsString.append(f + "\n");
			}
			int result = PeaDialog.showQuestion(JREProperties.getMainWindow(), CalendarSetting.getResourceBundle().getString("irretrievably_delete_files") 
					+ "\n" + new String(fileListAsString), 
					PeaProperties.getVmBridge().translate("wipe_and_delete"), 2 , -1); // Ok/Cancel and plain message
			if (result == 0) {
				TestLog.o(getClass(), "Securely wipe files: " + new String(fileListAsString));
				for (String f : cFiles) {
					if (f.contains(CloudControl.getTmpDir())) {
						// delete file from cloud
						String providerName = CloudControl.getProviderNameFromTmpFile(f);
						String fileNameToDelete = CloudControl.getFileNameFromTmpFile(f);
						StorageProvider prov = CloudControl.getOpenProvider(providerName);
						String error = prov.deleteFile(fileNameToDelete);
						if (error != null) {
							PeaDialog.showMessage(this, error, PeaProperties.getVmBridge().translate("error"),  0);
						} else {
							TestLog.o(getClass(), "Deleted in cloud: " + fileNameToDelete);
						}
					}
					FileTools.wipeFile(f, true);
				}
			}

		} else if (command.equals("manageCalendars")) {//("removeCalendar")) {
			
			calendarListDialog  = new JDialog(JREProperties.getMainWindow());
			calendarListDialog.setTitle(calLanguagesBundle.getString("hide_show_calendars"));
			JPanel mainPane = (JPanel) calendarListDialog.getContentPane();
			mainPane.setLayout(new BoxLayout(mainPane, BoxLayout.PAGE_AXIS));
			ArrayList<PCalendar> shownList = CalendarControl.getShownCalendarList();
			ArrayList<PCalendar> hiddenList = CalendarControl.getHiddenCalendarList();
			for (PCalendar pc : shownList) {
				if (pc != null ) {//&& pc.getName() != null && ! pc.getName().equals("")) {
					JCheckBox cb = new JCheckBox(pc.getName());
					cb.setName(pc.getCalUid());
					//TestLog.o(getClass(), "Name: " +pc.getName());
					cb.setOpaque(true);
					cb.setBackground(pc.getColor());
					Color fontColor = EventButton.getAppropriateFontColor(pc.getColor());
					cb.setForeground(fontColor); 
					cb.setSelected(true);
					mainPane.add(cb);
				}
			}
			for (PCalendar pc : hiddenList) {
				if (pc != null && pc.getName() != null && ! pc.getName().equals("")) {
					JCheckBox cb = new JCheckBox(pc.getName());
					cb.setName(pc.getCalUid());
					cb.setOpaque(true);
					cb.setBackground(pc.getColor());
					Color fontColor = EventButton.getAppropriateFontColor(pc.getColor());
					cb.setForeground(fontColor);
					cb.setSelected(false);
					mainPane.add(cb);
				}
			}
			mainPane.add(Box.createVerticalStrut(10));
			JButton showHideButton = new JButton(PeaProperties.getVmBridge().translate("ok"));
			showHideButton.addActionListener(this);
			showHideButton.setActionCommand("showHide");
			mainPane.add(showHideButton);
			calendarListDialog.pack();
			calendarListDialog.setMinimumSize(new Dimension(200, 100));
			calendarListDialog.setLocationRelativeTo((Component)this);
			calendarListDialog.setVisible(true);

			
		} else if (command.equals("showHide")) {
			
			ArrayList<PCalendar> shownList = CalendarControl.getShownCalendarList();
//			ArrayList<PCalendar> hiddenList = CalendarControl.getHiddenCalendarList();
//String s = "";	for(PCalendar scal : shownList) {s += " " + scal.getName() + " ";}TestLog.o(getClass(), "shown: " + s);
//String h = "";	for(PCalendar hcal : hiddenList) {h += " " + hcal.getName() + " ";}TestLog.o(getClass(), "hidden: " + h);
			
			if (calendarListDialog == null) {
				new UnexpectedValueException("JDialog", "calendarListDialog","is null").printDescription();
				return;
			}
			if ( ! (calendarListDialog.getContentPane() instanceof JPanel) ) {
				new UnexpectedValueException("Component", "calendarListDialog.getComponent(0)","is not JPanel").printDescription();
				TestLog.e(getClass(), calendarListDialog.getComponent(1).getClass().getName());
				return;
			}
			JPanel mainPane = (JPanel) calendarListDialog.getContentPane();
			Component[] comps = mainPane.getComponents();
			for (Component c : comps) {
				if (c instanceof JCheckBox) {
					JCheckBox cb = (JCheckBox) c;
					// get the calendar by name
					//PCalendar cal = CalendarControl.getCalendarByName(cb.getText());
					// get calendar by UID from name of check box
					PCalendar cal = CalendarControl.getCalendarByUID(cb.getName());
					if (cal == null) {
						new UnexpectedValueException("PCalendar","cal","is null").printDescription();
						calendarListDialog.dispose();
						return;
					}
					if (cb.isSelected() == false) {	
						if (shownList.contains(cal)) {
							CalendarControl.hideShownCalendar(cal);
						}
					} else {
						if ( ! shownList.contains(cal)) {
							CalendarControl.showHiddenCalendar(cal);
						}
					}
				}
			}
			CalendarControl.refreshView();
			calendarListDialog.dispose();			
			
		} else if (command.equals("calendarFiles")) {
			
			String selectedCalendarName = selectCalendar(languagesBundle.getString("calendars_and_files"));
			if (selectedCalendarName == null || selectedCalendarName.equals("")) {
				return;
			}

			// get associated file names:
			PCalendar cal = CalendarControl.getCalendarByName(selectedCalendarName);
			String[] associatedFileNames = cal.getAssociatedFileNames();
			StringBuilder fileNamesAsString = new StringBuilder();
			for (String s : associatedFileNames) {
				if (s.contains(CloudControl.getTmpDir())) {
					String providerName = CloudControl.getProviderNameFromTmpFile(s);
					String fileName = CloudControl.getFileNameFromTmpFile(s);
					fileNamesAsString.append(PeaProperties.getVmBridge().translate("cloud")
							+ " " + providerName + ": " + fileName + "\n");
				} else {
					fileNamesAsString.append(s + "\n");
				}
			}
			PeaDialog.showMessage(JREProperties.getMainWindow(), "   " + cal.getName() + ":\n\n" + fileNamesAsString);

		} else if (command.equals("calendarColor")) {

			String selectedCalendarName = selectCalendar(languagesBundle.getString("change_color"));
			if (selectedCalendarName == null || selectedCalendarName.equals("") ) {
				return;
			}
			//TestLog.o(getClass(), "Change color " + selectedCalendarName);			
			Color color = JColorChooser.showDialog(JREProperties.getMainWindow(),
					languagesBundle.getString("choose_calendar_color"), Color.BLACK);

			if (color != null) {
				//TestLog.o(getClass(), "color: " + color.toString());
				CalendarControl.getCalendarByName(selectedCalendarName).setColor(color);
				CalendarControl.refreshView();
			}
			//LockFrameCalendar.setDocChangeUnsaved(true);
			LockFrameCalendar.getFrame().markUnsavedContentChanges();
			
		} else if (command.equals("increaseFontSize")) {
			
			// change font size
			int oldFontSize = PeaProperties.getFontSize();
			int newFontSize = oldFontSize;
			if (oldFontSize < 18) {
				newFontSize = oldFontSize + 2;
			PeaProperties.setFontSize( newFontSize);
			} 
		    
			// set font size (UIManager)
			FontUIResource f = new FontUIResource(Font.SANS_SERIF, Font.PLAIN, newFontSize);			
			java.util.Enumeration<Object> keys = UIManager.getDefaults().keys();
		    while (keys.hasMoreElements())  {
		        Object key = keys.nextElement();
		        Object value = UIManager.get(key);
		        if (value instanceof javax.swing.plaf.FontUIResource) {
		            UIManager.put(key, f);
		        }
		    }

		    UIManager.getLookAndFeelDefaults()
	        .put("defaultFont", new Font(Font.SANS_SERIF, Font.PLAIN, newFontSize));

			if (SwingUtilities.getWindowAncestor(this) instanceof JFrame){ // LockFrame: plain text mode
				PswDialogView.setUI(false);
			} else if (SwingUtilities.getWindowAncestor(this) instanceof JDialog){ // Password dialog: encrypted mode
				PswDialogView.setUI(true);
			} else {
				new UnexpectedValueException("window ancgestor", "Window",
						"is not JDialog or JFrame: " + SwingUtilities.getWindowAncestor(this).toString()).printDescription();
			}
			Window win = SwingUtilities.windowForComponent(this);
			SwingUtilities.updateComponentTreeUI(win);
			win.pack();
			win.setVisible(true);
		    		    
		} else if (command.equals("decreaseFontSize")) {
			
			// change font size
			int oldFontSize = PeaProperties.getFontSize();
			int newFontSize = oldFontSize;
			if (oldFontSize > 10) {
				newFontSize = oldFontSize - 2;
			PeaProperties.setFontSize( newFontSize);
			} 
			
			// set font size (UIManager)
			FontUIResource f = new FontUIResource(Font.SANS_SERIF, Font.PLAIN, newFontSize);			
			java.util.Enumeration<Object> keys = UIManager.getDefaults().keys();
		    while (keys.hasMoreElements())  {
		        Object key = keys.nextElement();
		        Object value = UIManager.get(key);
		        if (value instanceof javax.swing.plaf.FontUIResource) {
		            UIManager.put(key, f);
		        }
		    }  

		    // resizes currently shown items, labels...
		    UIManager.getLookAndFeelDefaults()
	        .put("defaultFont", new Font(Font.SANS_SERIF, Font.PLAIN, newFontSize));
		    
		    // update (current) view
		    SwingUtilities.updateComponentTreeUI(PswDialogView.getView());		    
//		    updateAllComponents(PswDialogView.getView().getContentPane());		    
			if (SwingUtilities.getWindowAncestor(this) instanceof JFrame){ // LockFrame: plain text mode
				PswDialogView.setUI(false);
			} else if (SwingUtilities.getWindowAncestor(this) instanceof JDialog){ // Password dialog: encrypted mode
				PswDialogView.setUI(true);
			} else {
				new UnexpectedValueException("window ancgestor", "Window",
						"is not JDialog or JFrame: " + SwingUtilities.getWindowAncestor(this).toString()).printDescription();
			}
			Window win = SwingUtilities.windowForComponent(this);
			SwingUtilities.updateComponentTreeUI(win);
			win.pack();
			win.setVisible(true);

		} else if (command.equals("setColor")) {
			int[] colorVals = PeaProperties.getPeaColor();
			Color peaCol = new Color(colorVals[0], colorVals[1], colorVals[2]);
			Color color = JColorChooser.showDialog(this,PeaProperties.getVmBridge().translate("background_color"), peaCol);
			if (color == null) {
				return;
			}
			// check darkness:
			int darkness = ((30 * color.getRed() +
		            59 * color.getGreen() +
		            11 * color.getBlue()) / 100);
			while (darkness < 210) { // make brighter
				color = color.brighter();
				int newDarkness = ((30 * color.getRed() +
		            59 * color.getGreen() +
		            11 * color.getBlue()) / 100);
				if (newDarkness == darkness) { // this occurs if colors are 0
					int r = color.getRed();
					int g = color.getGreen();
					int b = color.getBlue();
					if ( r < 50) {
						r = r + 50;
					}
					if (g < 50) {
						g = g + 50;
					}
					if (b < 50) {
						b = b + 50;
					}
					color = new Color(r,g,b);
					darkness = ((30 * color.getRed() +
				            59 * color.getGreen() +
				            11 * color.getBlue()) / 100);
				} else {
					darkness = newDarkness;
				}
			}
			int[] newCol = {color.getRed(), color.getGreen(), color.getBlue()};
			PeaProperties.setPeaColor(newCol);
			if (SwingUtilities.getWindowAncestor(this) instanceof JFrame){ // LockFrame: plain text mode
				PswDialogView.setUI(false);
			} else if (SwingUtilities.getWindowAncestor(this) instanceof JDialog){ // Password dialog: encrypted mode
				PswDialogView.setUI(true);
			} else {
				new UnexpectedValueException("window ancgestor", "Window",
						"is not JDialog or JFrame: " + SwingUtilities.getWindowAncestor(this).toString()).printDescription();
			}
			
			Window win = SwingUtilities.windowForComponent(this);
			SwingUtilities.updateComponentTreeUI(win);
			win.pack();
			win.setVisible(true);
			PswDialogView.getView().validate();
			PswDialogView.getView().repaint();
			
		} else if (command.equals("changePsw")) {
			
			
			// do not set PeaProperties, this is done later
			// ask to apply changes or make further settings
			// TODO MultiFileSelection
			String[] fileNamesToApply = LockFrameCalendar.getFrame().chooseFileNamesToApply();
			String errorMessage = LockFrameCalendar.getFrame().changePasswordOrApplyChanges(
					LockFrameCalendar.getFrame(), fileNamesToApply, true);
			if (errorMessage != null) {
				PeaDialog.showMessage(LockFrameCalendar.getFrame(), 
						errorMessage, 
						PeaProperties.getVmBridge().translate("error"), 0);
			}
			
			//========================= APPEARANCE MENU ================================

		} else if (command.equals("daysColor")) {

			Color color = JColorChooser.showDialog(this,languagesBundle.getString("background_days"), Color.BLACK);
			if (color != null) {
				TestLog.o(getClass(), "color: " + color.toString());
			}

			CalendarSetting.setDayBackgroundColor(color);


			//LockFrameCalendar.getView().setFirstShownDate(LockFrameCalendar.getView().getFirstShownDate());
			boolean oldDocStatus = LockFrameCalendar.getFrame().isContentUnsaved();//LockFrameCalendar.isDocChangeUnsaved();
			CalendarControl.refreshView();
			if (LockFrameCalendar.getView() instanceof WeekPanel) {
				LockFrameCalendar.getFrame().pack();
			}
			if (oldDocStatus == true) {
				LockFrameCalendar.getFrame().markUnsavedContentChanges();
			} else {
				LockFrameCalendar.getFrame().markContentIsSaved();
			}
			
		} else if (command.equals("startTimeDay")) {
			// TODO hour spinner to set start time in week/day view
			Object[] startHour = {"0", "3", "5", "7", "9", "12", "15"};
		 	String hourString = (String) JOptionPane.showInputDialog(LockFrameCalendar.getFrame(), 
		 			languagesBundle.getString("start_time_day"), languagesBundle.getString("start_time_day"), 
		 			JOptionPane.PLAIN_MESSAGE, null, 
		 			startHour, startHour[3]);
		 	if (hourString != null) {
		 		int hour = Integer.parseInt(hourString);
		 		CalendarSetting.setViewStartHour(hour);
				boolean oldDocStatus = LockFrameCalendar.getFrame().isContentUnsaved();//LockFrameCalendar.isDocChangeUnsaved();
				CalendarControl.refreshView();
				//LockFrameCalendar.setDocChangeUnsaved(oldDocStatus);
				if (LockFrameCalendar.getView() instanceof WeekPanel) {
					LockFrameCalendar.getFrame().pack();
				}
				if (oldDocStatus == true) {
					LockFrameCalendar.getFrame().markUnsavedContentChanges();
				} else {
					LockFrameCalendar.getFrame().markContentIsSaved();
				}
		 	}
			

		} else if (command.equals("eventButtonFont")) {
			Object[] fontSizes = {"8", "10", "12", "14", "16","18", "20"};
		 	String fontString = (String) JOptionPane.showInputDialog(LockFrameCalendar.getFrame(), 
		 			languagesBundle.getString("font_size_event"), languagesBundle.getString("font_size_event"), 
		 			JOptionPane.PLAIN_MESSAGE, null, 
		 			fontSizes, fontSizes[2]);
		 	if (fontString != null) {
		 		int fontSize = Integer.parseInt(fontString);
		 		CalendarSetting.setEventButtonFont(new Font(Font.SANS_SERIF, Font.PLAIN,fontSize));
				boolean oldDocStatus = LockFrameCalendar.getFrame().isContentUnsaved();//LockFrameCalendar.isDocChangeUnsaved();
				CalendarControl.refreshView();
				//LockFrameCalendar.setDocChangeUnsaved(oldDocStatus);
				if (LockFrameCalendar.getView() instanceof WeekPanel) {
					LockFrameCalendar.getFrame().pack();
				}
				if (oldDocStatus == true) {
					LockFrameCalendar.getFrame().markUnsavedContentChanges();
				} else {
					LockFrameCalendar.getFrame().markContentIsSaved();
				}
		 	}

		} else if (command.equals("weekNumber")) {
			Object[] numberOptions = {"1", "3", "5", "7", "9", "11"};
		 	String weekString = (String) JOptionPane.showInputDialog(LockFrameCalendar.getFrame(), 
		 			languagesBundle.getString("number_weeks"), languagesBundle.getString("number_weeks"), 
		 			JOptionPane.PLAIN_MESSAGE, null, 
		 			numberOptions, numberOptions[2]);
		 	if (weekString != null) {
		 		int weekNumber = Integer.parseInt(weekString);
		 		CalendarSetting.setNumberLinesInMonthPanel(weekNumber);
		 		if (LockFrameCalendar.getView() instanceof MonthPanel) {
		 			boolean oldDocStatus = LockFrameCalendar.getFrame().isContentUnsaved();//LockFrameCalendar.isDocChangeUnsaved();
		 			CalendarControl.refreshView();
					if (oldDocStatus == true) {
						LockFrameCalendar.getFrame().markUnsavedContentChanges();
					} else {
						LockFrameCalendar.getFrame().markContentIsSaved();
					}
		 		}
		 	}
		}
	}
	//==================== HELPER FUNCTIONS =========================
	/**
	 * Save all currently shown calendars encrypted in all associated files
	 * 
	 * @return	error message as String or null (no error)
	 */
	public String saveAllCalendars() {

		ResourceBundle languagesBundle = CalendarSetting.getResourceBundle();

		// get all active Calendars as Collection:
		Collection <PCalendar> ncsCol =  CalendarControl.getShownAndHiddenCalendarList();
		// check if collection is null or empty
		if (ncsCol == null || ncsCol.size() == 0) {
			PeaDialog.showMessage(this, languagesBundle.getString("calendar_missing")
					+ "\n → " + languagesBundle.getString("file")
					+ "\n    → " + languagesBundle.getString("new")
					);
			return languagesBundle.getString("calendar_missing");
		}			
		
		
		// create a HashMap of all used file names as key and their content as value (first empty):
		String[] allFileNames = CalendarControl.getAllAssociatedFileNames();
		// create map for cloud file contents
		HashMap<String, HashMap<String, byte[]>> cloudMap = new HashMap<String, HashMap<String, byte[]>>();
		// create map for plain texts: <file name , PCalendar as String>
		HashMap<String, StringBuilder> map = new HashMap<String, StringBuilder>();
		for (String fileOfAllFileName : allFileNames) {
			map.put(fileOfAllFileName, new StringBuilder());
		}


		// get all used calendars:
		PCalendar[] ncs = ncsCol.toArray(new PCalendar[ncsCol.size()]);
		int calLen = ncs.length;
		for (int i = 0; i < calLen; i++) {
			PCalendar c = ncs[i];
			// get the calendar as String
			String calAsString = WriteICal.iCalToString(c);
			if (calAsString == null || calAsString.length() == 0) {
				new UnexpectedValueException("String", "calAsString", "is null or empty").printDescription();
			}
			// get all file names, the calendars were loaded from:
			String[] calFileNames = c.getAssociatedFileNames();		
			//for(String s : calFileNames)TestLog.o(getClass(), "Associated: "+ s);
			// add content (PCalendar) to map
			for (String fileNameOfCal : calFileNames) {
				StringBuilder sb = map.get(fileNameOfCal);
				//TestLog.o(getClass(), "Append "+ calAsString +"\nTO "+ fileNameOfCal);
				// append this calendar to the content
				if (sb == null) {
					sb = new StringBuilder(calAsString);
				} else {
					sb.append(calAsString);
				}
			}
		}
		/*	for(HashMap.Entry<String, StringBuilder> entry : map.entrySet()) {
			TestLog.o(getClass(), "Key: "  + entry.getKey());
			TestLog.o(getClass(), "Value: " + entry.getValue());
		}*/
		// Write the contents to the files:
		for (HashMap.Entry<String, StringBuilder> entry : map.entrySet()) {
		    //TestLog.o(getClass(), entry.getKey() + "/" + entry.getValue());
			if (entry == null) {
				continue;
			}
			if (entry.getKey() == null) {
				continue;
			}
			byte[] plainBytes = new String(entry.getValue()).getBytes(PeaProperties.getCharset());
			if (plainBytes == null || plainBytes.length == 0) {
				new UnexpectedValueException("byte[]","plainBytes","is null: "+ entry.getKey()).printDescription();
			}
			byte[] cipherBytes = CipherStuff.getInstance().encrypt(plainBytes, 
					CipherStuff.getSessionKeyCrypt().getKey(), // keyMaterial is overwritten
					true, null ); // key is stored encrypted			

			if (entry.getKey().contains(CloudControl.getTmpDir())) {
				// Load in cloud:
				// performance: Collect files from provider and upload together
				String cloudProvider = CloudControl.getProviderNameFromTmpFile(entry.getKey());
				if (cloudMap.containsKey(cloudProvider)) {
					cloudMap.get(cloudProvider).put(CloudControl.getFileNameFromTmpFile(entry.getKey()), cipherBytes);
				} else {
					HashMap<String, byte[]> entryMap = new HashMap<String, byte[]>();
					entryMap.put(CloudControl.getFileNameFromTmpFile(entry.getKey()), cipherBytes);
					cloudMap.put(cloudProvider, entryMap);
				}

			} else {
				// write local:
				boolean success = WriteResources.write(cipherBytes,entry.getKey(), null);
				
				if (success == false) {
					TestLog.e(getClass(), "Save calendar failed");
					PeaDialog.showMessage(this, 
							peaLanguagesBundle.getString("operation_failed") + ":\n"
							+ peaLanguagesBundle.getString("save") + ": " + entry.getKey(), 
							peaLanguagesBundle.getString("error"), 0);
					//LockFrameCalendar.setDocChangeUnsaved(true);
					if (LockFrameCalendar.getFrame() != null) {
						LockFrameCalendar.getFrame().markUnsavedContentChanges();
					}
					return languagesBundle.getString("error") + "\n" + CipherStuff.getErrorMessage();
				} else {
					TestLog.o(getClass(), "Calendars saved: " + entry.getKey());
					//LockFrameCalendar.setDocChangeUnsaved(false);
					LockFrameCalendar.getFrame().markContentIsSaved();
					//return null;
				}
			}
		}
		// Upload collected maps to the clouds in threads:
		Iterator<Map.Entry<String, HashMap<String, byte[]>>> providerIterator = cloudMap.entrySet().iterator();
		while (providerIterator.hasNext()) {
		        Map.Entry<String, HashMap<String, byte[]>> pair = (Map.Entry<String, HashMap<String, byte[]>>)providerIterator.next();
			// A file can be uploading by sending a PUT request to the file and sending the raw file contents as the request body.
			// check if content exists 
			String cloudName = pair.getKey();
			HashMap<String, byte[]> providerMap = pair.getValue();
			UploadWorker uw = new UploadWorker((Window)PeaControl.getDialog().getLockFrame(), cloudName, null, 
			null, providerMap);
			uw.execute();
		}
		return null;
	}
	
	/**
	 * Show an option dialog to choose a calendar name
	 * from the currently opened calendars
	 * 
	 * @param message 	the message to show in the dialog 
	 * 					(why select)
	 * 
	 * @return	the selected calendar name or null if no
	 * 			calendar was selected or an error occurred
	 */
	private String selectCalendar(String message) {
		
		// select calendar
		String[] calendarNames = CalendarControl.getShownCalendarNameList();	
		if (calendarNames != null && calendarNames.length > 0) {
			//String[] calNamesPlusNull = new String[calendarNames.length + 1];
			//System.arraycopy(calendarNames, 0, calNamesPlusNull, 1, calendarNames.length);
			//calendarNames = calNamesPlusNull;
			
			String selectedCalendarName = (String) JOptionPane.showInputDialog(JREProperties.getMainWindow(), 
					message,
					calLanguagesBundle.getString("select_calendar"),
					JOptionPane.QUESTION_MESSAGE, 
					null, 
					calendarNames, 
					calendarNames[0]);
			if (selectedCalendarName == null || selectedCalendarName.equals("") ) {
				return null;
			} else {
				return selectedCalendarName;
			}

		} else {
			PeaDialog.showMessage(this, calLanguagesBundle.getString("calendar_missing")
					+ "\n → " + peaLanguagesBundle.getString("file")
					+ "\n    → " + peaLanguagesBundle.getString("new")
					);
			return null;
		}
	}
	
    public AccessibleContext getAccessibleContext() {
        if (accessibleContext == null) {
            accessibleContext = new AccessibleCalendarMenuBar();
        }
        return accessibleContext;
    }

    private class AccessibleCalendarMenuBar extends AccessibleJComponent {
        //Inherit everything, override nothing.
    }

}
