package cologne.eck.calendar_pea.gui;

/*
 * PeaFactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import java.io.IOException;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.ResourceBundle;

import javax.accessibility.AccessibleContext;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import cologne.eck.all_peas.cloud.CloudControl;
import cologne.eck.all_peas.cloud.StorageProvider;
import cologne.eck.all_peas.control.ContentUtilities;
import cologne.eck.all_peas.control.PathFileManager;
import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.files.CloudFilePanel;
import cologne.eck.all_peas.files.FileTypePanel;
import cologne.eck.all_peas.gui.PeaBorderFactory;
import cologne.eck.all_peas.gui.PeaDialog;
import cologne.eck.all_peas.gui.PeaLockFrame;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.all_peas.vm_specific.JREProperties;
import cologne.eck.calendar_pea.control.CalendarControl;
import cologne.eck.calendar_pea.control.CalendarSetting;
import cologne.eck.calendar_pea.data.PCalendar;
import cologne.eck.calendar_pea.gui.calendar_view.CalendarView;
import cologne.eck.calendar_pea.gui.calendar_view.DayPanel;
import cologne.eck.calendar_pea.gui.calendar_view.MonthPanel;
import cologne.eck.calendar_pea.gui.calendar_view.WeekPanel;
import cologne.eck.calendar_pea.util.ReadICal;
import cologne.eck.calendar_pea.util.WriteICal;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.tools.ExceptionHelper;
import cologne.eck.tools.PropertyHandler;
import cologne.eck.tools.ReadResources;
import cologne.eck.tools.TestLog;
import cologne.eck.tools.UnexpectedValueException;


@SuppressWarnings("serial")
public class LockFrameCalendar extends PeaLockFrame 
		implements WindowListener, ActionListener {
	
	private static LockFrameCalendar frame;
	
	private static CalendarView view;// = new WeekPanel(LocalDate.now(), PeaProperties.getFrameDim());
	
	private static CalendarMenuBar menuBar;
	/*
	 * The label for messages and errors
	 */
	private static JLabel messageLabel;
	
	private static JPanel viewPanel;
	
	private static JComboBox<String> viewList;
	
	// if the content of the current document was modified: true
	private boolean docChangeUnsaved = false;
	
//	private static String[] currentUsedFileNames;
	
	private static JButton saveStatusButton = null;
	/**
	 * The Frame that contains MonthPanel or WeekPanel or DayPanel
	 * 
	 * @param location	location to display the frame
	 * @param dim
	 */
	private LockFrameCalendar(Point location, Dimension dim){
		
		frame = this;	
		this.addWindowListener(this);
		this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE); // do not allow to close or to exit:
		// files may remain unencrypted
		
		if (dim == null) {
			int[] dimValues = JREProperties.getFrameDim();
			dim = new Dimension(dimValues[0], dimValues[1]);//PeaProperties.getFrameDim();//CalendarSetting.getViewDimension();
		}
		
		ResourceBundle languagesBundle = CalendarSetting.getResourceBundle();
		
	    this.setIconImage( PswDialogView.getImage() );		
		this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE); // do not allow to close or to exit:
															// files may remain unencrypted
		this.addWindowListener(this);// for windowClosing
		
		JPanel contentPane = new JPanel();
		contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.PAGE_AXIS));
		this.setContentPane(contentPane);		
		//contentPane.setBackground(new Color(237, 237, 222));
		
		JPanel menuPanel = new JPanel();
		menuPanel.setLayout(new BoxLayout(menuPanel, BoxLayout.LINE_AXIS));
		menuBar = new CalendarMenuBar();
		menuPanel.add(menuBar);
		menuPanel.add(Box.createHorizontalGlue());
		menuPanel.setMinimumSize(new Dimension(300, 30));
		menuPanel.setMaximumSize(new Dimension((int)PswDialogView.getScreenwidth(), 40));
//		contentPane.add(Box.createHorizontalGlue());
		contentPane.add(menuPanel);
		
		JPanel statusLabelPanel = new JPanel();
		statusLabelPanel.setLayout(new BoxLayout(statusLabelPanel, BoxLayout.LINE_AXIS));
		saveStatusButton = new JButton(PeaProperties.getVmBridge().translate("save") + "?");
	//	saveStatusButton.setContentAreaFilled(false); 
		saveStatusButton.setBorder(PeaBorderFactory.getButtonBorder());//new LineBorder(Color.GRAY));
	//	saveStatusButton.setMargin(new Insets(2,2,2,2));
		saveStatusButton.setFont(new Font(Font.SERIF, Font.ITALIC, 12));
	//	saveStatusButton.setForeground(Color.GRAY);
	//	saveStatusButton.setBackground(contentPane.getBackground());
		//saveStatusButton.setBorder(new EmptyBorder(2,2,2,2));
		saveStatusButton.addActionListener(this);
		saveStatusButton.setActionCommand("save");
		saveStatusButton.setVisible(false);
		statusLabelPanel.add(Box.createHorizontalGlue());//Strut(50));
		statusLabelPanel.add(saveStatusButton);
		statusLabelPanel.add(Box.createHorizontalGlue());
		statusLabelPanel.setMaximumSize(new Dimension((int) PswDialogView.getScreenwidth(), 50));
		
		//JPanel viewButtonPanel = new JPanel();
		String[] viewStrings = { CalendarSetting.getResourceBundle().getString("day_view"), 
				CalendarSetting.getResourceBundle().getString("week_view"), 
				CalendarSetting.getResourceBundle().getString("month_view") };
		viewList = new JComboBox<String>(viewStrings);
		int selectedIndex = 0;
		if (view instanceof DayPanel) {
			selectedIndex = 0;
		} else if (view instanceof WeekPanel) {
			selectedIndex = 1;
		} else if (view instanceof MonthPanel) {
			selectedIndex = 2;
		} else {
			view = CalendarSetting.getStartView();
		}
		viewList.setSelectedIndex(selectedIndex);
		viewList.addActionListener(this);
		statusLabelPanel.add(viewList);	
		contentPane.add(statusLabelPanel);
		contentPane.add(Box.createVerticalStrut(10));

		// add the view: 
		viewPanel = new JPanel();
		viewPanel.setLayout(new GridLayout(1,1));
		viewPanel.add((Component) view);
		contentPane.add(viewPanel);
		
		JPanel messagePanel= new JPanel();
		messagePanel.setLayout(new BoxLayout(messagePanel, BoxLayout.LINE_AXIS));
		messageLabel = new JLabel();
		messagePanel.add(messageLabel);
		messagePanel.add(Box.createHorizontalGlue());
		this.add(messagePanel);
		this.add(Box.createVerticalStrut(10));
		
		JPanel forwardBackPanel = new JPanel();
		forwardBackPanel.setLayout(new BoxLayout(forwardBackPanel, BoxLayout.LINE_AXIS));
		JButton backButton = new JButton(languagesBundle.getString("back"));
		backButton.addActionListener(this);
		backButton.setActionCommand("back");
		backButton.setBorder(PeaBorderFactory.getButtonBorder());
		forwardBackPanel.add(backButton);
		forwardBackPanel.add(Box.createHorizontalGlue());
		JButton todayButton = new JButton(languagesBundle.getString("today"));
		todayButton.addActionListener(this);
		todayButton.setActionCommand("today");
		todayButton.setBorder(PeaBorderFactory.getButtonBorder());
		forwardBackPanel.add(todayButton);
		forwardBackPanel.add(Box.createHorizontalGlue());
		JButton forwardButton = new JButton(languagesBundle.getString("forward"));
		forwardButton.addActionListener(this);
		forwardButton.setActionCommand("forward");
		forwardButton.setBorder(PeaBorderFactory.getButtonBorder());
		forwardBackPanel.add(forwardButton);		
		this.add(forwardBackPanel);

		
		this.setLocation(location);
		pack();// otherwise vertical scroll bar is not set...

		int screenWidth = (int) PswDialogView.getScreenwidth();
		int screenHeight = (int) PswDialogView.getScreenheight();

		// set the height: consider all components except viewPanel
		dim.height = dim.height + menuPanel.getPreferredSize().height
				+ statusLabelPanel.getPreferredSize().height
				+ 10
				+ messagePanel.getPreferredSize().height
				+ forwardBackPanel.getPreferredSize().width;

		if (dim.width > screenWidth) {
			dim.width = screenWidth;
		}
		if (dim.height > screenHeight) {
			dim.height = screenHeight;
		}

		this.setSize(dim);

		if (view instanceof WeekPanel) {
			this.setMinimumSize( new Dimension (500, 300));//this.getSize() );
		} else {
			this.setMinimumSize( new Dimension (400, 300));//this.getSize() );
		}
	}

	/**
	 * Create the instance of LockFrameCalendar.
	 * There is always only one instance. The get the
	 * existing instance, call getFrame()
	 * 
	 * @param loc	the location to show
	 * @param dim	the dimension to show
	 * 
	 * @return	the instance of LockFrameCalendar, never null
	 */
	public final static LockFrameCalendar getInstance(Point loc, Dimension dim){

		if (frame == null) {
			frame = new LockFrameCalendar(loc, dim);// selectedFileNames);
		}
		return frame;
	}
	/**
	 * Get the instance of LockFrameCalendar. Note: If this
	 * is not created, the functions returns null
	 * 
	 * @return	the current LockFrameCalendar or null
	 */
	public final static LockFrameCalendar getFrame() {
		return frame;
	}
	
	@Override
	public void windowActivated(WindowEvent arg0) {}
	@Override
	public void windowClosed(WindowEvent arg0) {}

	@Override
	public void windowClosing(WindowEvent arg0) {
		close();
	}

	@Override
	public void windowDeactivated(WindowEvent arg0) {}
	@Override
	public void windowDeiconified(WindowEvent arg0) {}
	@Override
	public void windowIconified(WindowEvent arg0) {}
	@Override
	public void windowOpened(WindowEvent arg0) {}

	protected void close() {		
		
		// Check if algorithms or parameters were changed
		// but not yet applied: 
		if (isChangesApplied() == false) {
			changePasswordOrApplyChanges(frame, null, true);
		}

		
		// 1. check if unsaved changes, ask to save
		if (checkUnsavedContentChanges() == true) {
			// do not check twice
			//docChangeUnsaved = false;
			markContentIsSaved();
			//setDocChangeUnsaved(false);
			return;
		}
		
		// 2. check if all files are in path file and ask to store if not
		HashSet<String> availableSet = new HashSet<String>(PeaControl.getAvailableFileNames());
		//TestLog.o(getClass(), "Available\n");for(String s : PeaControl.getAvailableFileNames()) TestLog.o(getClass(), s);
		//TestLog.o(getClass(), "Associated\n");for(String s: CalendarControl.getAllAssociatedFileNames())TestLog.o(getClass(), s);
		if (CalendarControl.getAllAssociatedFileNames() != null) {
			availableSet.addAll( new ArrayList<>(Arrays.asList(CalendarControl.getAllAssociatedFileNames())));
		}
		if (availableSet.size() > 0) {
			PathFileManager.addFileNamesToPathFile(availableSet);
		}

		// 3. save properties
		new PropertyHandler().updateAllPropValues();

		// 4. check if cloud processes are still running and ask to exit
		if (CloudControl.getNumberOfRunningCloudProcesses() > 0) {
			String[] options = {PeaProperties.getVmBridge().translate("cancel"),
					PeaProperties.getVmBridge().translate("ok")};
					
			
			int result = PeaDialog.showOptionYAxis(this, 
					PeaProperties.getVmBridge().translate("cloud_process_warning")
					+ "\n" + PeaProperties.getVmBridge().translate("terminating_damage"), 
					PeaProperties.getVmBridge().translate("warning"), 
					2,  options, 0);

			if (result == 1) {
				TestLog.o(getClass(), "Closing although cloud process is running...");
			} else {
				TestLog.o(getClass(), "Upload process is running.... Break exit...");
				return;
			}
		}	

		// 5. clear key
		CipherStuff.getSessionKeyCrypt().clearKeys();

		// 6. Close all providers
		Collection<StorageProvider> openProviders = CloudControl.getOpenProviders();
		if (openProviders != null &&  ! openProviders.isEmpty()) {
			for (StorageProvider sp : openProviders) {
				String error = sp.closeSession();
				if (error != null) {
					TestLog.e(getClass(), error);
				}
			}
		}

		// 7. delete all temporary backup files of cloud files:
		FileTypePanel ftp = (FileTypePanel)JREProperties.getTypePanel();
		if (ftp != null) {
			CloudFilePanel cfp = ftp.getCloudFilePanel();
			if (cfp != null && cfp.getCloudControl() != null) {
				String[] providers = cfp.getCloudControl().getUsedProviders();

				if (providers != null ) {
					for (String provName : providers) {						
						String[] filesOfProv = cfp.getCloudControl().getUsedFileNamesOfProvider(provName);
						if (filesOfProv != null) {
							for (String fileName : filesOfProv) {
								File fileToDelete = new File(CloudControl.getTmpFileNameForCloudFile(fileName, provName, null));
								boolean deleteResult = fileToDelete.delete();
								if (deleteResult == false) {
									TestLog.e(getClass(), "Could not delete " + fileToDelete);
								} else {
								}
							}
						}
					}
				}
			}
		}
		TestLog.o(getClass(), "Exit");
		System.exit(0);			
	}
	
	/**
	 * Check if the currently shown calendars
	 * were modified and not saved manually and ask 
	 * to save them in this case. 
	 * 
	 * @return	true if save process is initialized, 
	 * 			false if no unsaved changes or unsaved
	 * 			changes should remain
	 */
	@Override
	public boolean checkUnsavedContentChanges() {
		if (docChangeUnsaved == true) {
			int n = JOptionPane.showConfirmDialog(
				    this,
				    PeaProperties.getVmBridge().translate("save_modifications"),//"Save changes?",
				    PeaProperties.getVmBridge().translate("unsaved_modifications"),//"Unsaved Changes",
				    JOptionPane.YES_NO_OPTION);
			if (n == JOptionPane.YES_OPTION) {
				// save calendars in default file
				((CalendarMenuBar)((JPanel)((JPanel)frame.getContentPane()).getComponent(0)).getComponent(0)).saveAllCalendars();
				return true;
			} else {
				return false;
			}
		}
		return false; // no unsaved content remains
	}
	@Override
	public void markUnsavedContentChanges() {
		docChangeUnsaved = true;
		if (saveStatusButton != null) {
			saveStatusButton.setVisible(true);
		}
	}

	@Override
	public void markContentIsSaved() {
		docChangeUnsaved = false;
		if (saveStatusButton != null) {
			saveStatusButton.setVisible(false);
		}
	}
	
	@Override
	public boolean isContentUnsaved() {
		// TODO Auto-generated method stub
		return docChangeUnsaved;
	}

	@Override
	public String saveCurrentContentInCloudTempFile(String providerName, String fileNameToStore) {

		HashSet<PCalendar> calsInFileSet = CalendarControl.getCalendarsOfFileAsSet(fileNameToStore);
		if (calsInFileSet == null) {
			new UnexpectedValueException("HashSet<PCalendar>","calsInFileSet", "is null - file does not contain a calendar: "+ fileNameToStore).printDescription();
			return CalendarSetting.getResourceBundle().getString("no_valid_calendar_found")
					 + fileNameToStore;
		}
		//TestLog.o(getClass(), "<<< File: " + fileNameToStore + " - Calendars in file: ");
		//for(PCalendar c : calsInFileSet) {TestLog.o(getClass(), c.getName()+ " " + c.getCalUid());}

		// get calendars as one String:
		//String plainString = WriteICal.iCalsToString(ncs);
		PCalendar[] calsInFile = calsInFileSet.toArray(new PCalendar[calsInFileSet.size()]);
		String plainString = WriteICal.iCalsToString(calsInFile);
		if (plainString == null || plainString.length() == 0) {
			new UnexpectedValueException("String", "plainString", "is null or empty").printDescription();
			return CalendarSetting.getResourceBundle().getString("no_valid_calendar_found") + " (empty) " 
					 + fileNameToStore;

		}
		String orig = null;
		if ( ! fileNameToStore.contains(CloudControl.getTmpDir())) {
			orig = fileNameToStore;
		}

		String tmpFileName = CloudControl.getTmpFileNameForCloudFile(new File(fileNameToStore).getName(), providerName, orig);
		//System.getProperty("java.io.tmpdir") 
		//+ File.separator + providerName + File.separator + fileNameToStore;
		File file = new File(tmpFileName);
		if (file.exists() == false) {
			try {
				file.getParentFile().mkdirs();
				file.createNewFile();
				TestLog.o(getClass(), "Temp file created for cloud " + providerName + ": " + file.getAbsolutePath());
			} catch (IOException e) {
				TestLog.e(getClass(), fileNameToStore + " - " + new ExceptionHelper(e).getInfos());
				return PeaProperties.getVmBridge().translate("error") + "\nCreate tmp file failed";
			}
		}
		// encrypt the calendars by session key and store in file
		// TODO: if calendar unchanged: copy encrypted content (same IV)
		boolean success = ContentUtilities.saveContent(CipherStuff.getSessionKeyCrypt().getKey(), 
				plainString.getBytes(PeaProperties.getCharset()), null,
				tmpFileName, true);
		if (success == false) {
			TestLog.e(getClass(), fileNameToStore + " - " + "Save calendar in temp file failed");
			return PeaProperties.getVmBridge().translate("error") + "\n" + CipherStuff.getErrorMessage();
		} else {
			//TestLog.o(getClass(), "Calendars saved in temp file " + tmpFileName);
			return null;
		}
	}

	@Override
	public void actionPerformed(ActionEvent ae) {

		if (ae.getSource() instanceof JButton) {

			String command = ae.getActionCommand();
			if (command.equals("save")) {
				// save calendars in default file
				String error = ((CalendarMenuBar)((JPanel)((JPanel)frame.getContentPane()).getComponent(0)).getComponent(0)).saveAllCalendars();
				if (error == null) {
					// show message:
					//((LockFrameCalendar)PeaControl.getDialog().getLockFrame() ).setMessage(PeaProperties.getVmBridge().translate("save") + ": " 
					//	+ PeaProperties.getVmBridge().translate("success"), false);
					markContentIsSaved();
					//LockFrameCalendar.setDocChangeUnsaved(false);
				} else {
					TestLog.ve(getClass(), error, 5);
					/*		PeaDialog.showMessage(this, 
							PeaProperties.getVmBridge().translate("operation_failed") + ":\n"
							+ PeaProperties.getVmBridge().translate("save"), 
							PeaProperties.getVmBridge().translate("error"), 0);		
					((LockFrameCalendar)PeaControl.getDialog().getLockFrame() )
						.setMessage(PeaProperties.getVmBridge().translate("save") + " " + 
								PeaProperties.getVmBridge().translate("failed"), true);	*/
				}					
			} else if (command.equals("back")) {
				LocalDate shownDate = view.getFirstShownDate();
				if (view instanceof DayPanel) {
					shownDate = shownDate.minusDays(1);
				} else if (view instanceof WeekPanel) {
					shownDate = shownDate.minusDays(7);
				} else if (view instanceof MonthPanel) {
					shownDate = shownDate.minusDays(CalendarSetting.getNumberLinesInMonthPanel() * 7);
				}
				view.setFirstShownDate(shownDate);
				view.refreshView();

			} else if (command.equals("today")) {
				LocalDate shownDate = LocalDate.now();
				view.setFirstShownDate(shownDate);
				view.refreshView();

			} else if (command.equals("forward")) {
				LocalDate shownDate = view.getFirstShownDate();
				if (view instanceof DayPanel) {
					shownDate = shownDate.plusDays(1);
				} else if (view instanceof WeekPanel) {
					shownDate = shownDate.plusDays(7);
				} else if (view instanceof MonthPanel) {
					shownDate = shownDate.plusDays(CalendarSetting.getNumberLinesInMonthPanel() * 7);
				}
				view.setFirstShownDate(shownDate);
				view.refreshView();
			}
		} else if (ae.getSource() instanceof JComboBox) {

			@SuppressWarnings("rawtypes")
			int viewIndex = ((JComboBox)ae.getSource()).getSelectedIndex();
			// 0 is day, 1 is week, 2 is month
			changeView(viewIndex);
		}
	}

	/**
	 * Change the current view 
	 * 
	 * @param viewIndex		the index of the  view: 
	 * 						0 for day view, 1 for week view, 2 for month view
	 */
	public void changeView(int viewIndex) {
		
		Dimension dim = ((Component)view).getSize();
		LocalDate date = LockFrameCalendar.view.getFirstShownDate();
		if (viewIndex == 0) {
			if (LockFrameCalendar.view instanceof DayPanel){
				return;
			}		
			viewPanel.remove((Component) view);//MonthPanel.getInstance());
			DayPanel dp = new DayPanel(date, dim, false);
			//LockFrameCalendar.view = dp;
			LockFrameCalendar.setView(dp);
			viewPanel.add(dp);
			viewList.setSelectedIndex(0);// called from date label
		} else if (viewIndex == 1) { // WeekPanel
			if (LockFrameCalendar.view instanceof WeekPanel){
				return;
			}			
			viewPanel.remove((Component) view);
			WeekPanel wp = new WeekPanel(date, dim);//viewSize);
			viewPanel.add(wp);
			//LockFrameCalendar.view = wp;
			LockFrameCalendar.setView(wp);
			viewList.setSelectedIndex(1);// called from date label
		} else if (viewIndex == 2) {
			if (LockFrameCalendar.view instanceof MonthPanel){
				return;
			}
			viewPanel.remove((Component)view);
			MonthPanel monthPanel = new MonthPanel(date, dim);//viewSize);
			//LockFrameCalendar.view.setFirstShownDate(date);
			viewPanel.add(monthPanel);
			//LockFrameCalendar.view = monthPanel;
			LockFrameCalendar.setView(monthPanel);
			viewList.setSelectedIndex(2);// called from date label
		}
		viewPanel.revalidate();// refresh may cause NullPointer otherwise
		viewPanel.repaint();
		view.refreshView();// maybe an event was added
		
	}
	/**
	 * Display a message: error in red, otherwise black
	 * 
	 * @param message	the message to show
	 * @param isError	true if message is an error
	 * 					(message font is red)
	 */	
	@Override	
	public void setMessage(String message, boolean isError) {
		if(isError == true) {
			messageLabel.setForeground(Color.red);
			messageLabel.setText(message);			
		} else {
			messageLabel.setForeground(Color.black);
			messageLabel.setText(message);
		}
	}

	/**
	 * Get the current view: day, week, month
	 * 
	 * @return	the current view
	 */
	public static CalendarView getView() {
		return view;
	}

	/**
	 * Set the view: day,week or month
	 * 
	 * @param _view
	 */
	public static synchronized void setView(CalendarView _view) {
		LockFrameCalendar.view = _view;
	}
	
    public AccessibleContext getAccessibleContext() {
        if (accessibleContext == null) {
            accessibleContext = new AccessibleLockFrameCalendar();
        }
        return accessibleContext;
    }

    private class AccessibleLockFrameCalendar extends AccessibleJFrame {
        //Inherit everything, override nothing.
    }

	@Override
	public String[] getCurrentFileNames() {
		/*if (CalendarSetting.getDefaultFileAbsolutePath() != null) {
			// default file was opened too
			return CalendarSetting.getDefaultFileAbsolutePath();
		} else {
			return currentUsedFileNames;
		}*/
		return CalendarControl.getAllAssociatedFileNames();
		//return currentUsedFileNames;
	}
	@Override
	public void setCurrentFileNames(String[] fileNamesToSet) {
		new UnexpectedValueException("function", "setCurrentFileNames", "not implemented").printDescription();
	}

	@Override
	public void updateModelWithCloudFiles(File[] files, String providerName) {
		
	//	String modelError = "";
		for (int i = 0; i < files.length;i++) {
			HashSet<PCalendar> cals = CalendarControl.getFileCalendarsMap().get(files[i].getAbsolutePath());
			if (cals != null && cals.size() > 0) {
				Iterator<PCalendar> calIterator = cals.iterator();
				while(calIterator.hasNext()) {
					for (int j = 0; j < cals.size(); j++) {
						PCalendar cal = calIterator.next();
						// get the backup file 
						String cloudFileTmp = CloudControl.getTmpFileNameForCloudFile(files[i].getName(), providerName, null);
						// add associated file name to calendar
						cal.addAssociatedFileName(cloudFileTmp);
						// add to file-calendar map:
						CalendarControl.addToFileCalendarsMap(cloudFileTmp, cal);
					}
				}		
			} else {
				// new uploaded: Get file-calendar map (was set in CloudMenu)
				if (((FileTypePanel)JREProperties.getTypePanel()) != null 
						&& ((FileTypePanel)JREProperties.getTypePanel()).getCloudFilePanel() != null) {

					String origin = ((FileTypePanel)JREProperties.getTypePanel()).getCloudFilePanel().getCloudControl().getCloudFileModel().getLocalOrigin(files[i].getAbsolutePath());
					if (origin == null) { // there is no origin, only the cloud file
						TestLog.e(getClass(), "Can't find the original local file that is uploaded to the cloud: " + files[i].getAbsolutePath());
						//new UnexpectedValueException("String", "origin", "is null").printDescription();
						//return;
						origin = files[i].getAbsolutePath();
					}
					HashSet<PCalendar> originCals = CalendarControl.getCalendarsOfFileAsSet(origin);
					if (originCals == null || originCals.size() == 0) {
						// cloud only files: no origin
						//				new UnexpectedValueException("HashSet<PCalendar>", "originCals", "is null from file: "+ files[i].getAbsolutePath()).printDescription();
						//modelError += PeaProperties.getOsBridge().translate("upload_failed") + ": " + files[i].getAbsolutePath()
						//		+ "\n";
					} else {
						Iterator<PCalendar> originCalsIterator = originCals.iterator();
						while (originCalsIterator.hasNext()) {
							PCalendar cal = originCalsIterator.next();
							// get the backup file 
							String cloudFileTmp = CloudControl.getTmpFileNameForCloudFile(files[i].getName(), providerName, null);
							cal.addAssociatedFileName(cloudFileTmp);
							//TestLog.o(getClass(), "added: " + cloudFileTmp + " in " + cal.getName());
							// add to file-calendar map:
							CalendarControl.addToFileCalendarsMap(cloudFileTmp, cal);
						}
					}
				} else { // Initialization and save in cloud
					TestLog.ve(getClass(), "Missing cloud file panel to set calendar map...");
				}
			}
		}
	/*	if (modelError != null && modelError.length() > 0) {
			return modelError;
		} else {
			return null;
		} */
		
	}

	@Override
	public String decryptAndLoadFileToView(String fileName) {
		byte[] cipherText = ReadResources.readExternFile(fileName);
		if (cipherText == null) {
			new UnexpectedValueException("byte[]", "cipherText", "is null").printDescription();
			return fileName + " - " + PeaProperties.getVmBridge().translate("no_access");
		}
		// check password identifier
		byte[] keyMaterial = CipherStuff.getSessionKeyCrypt().getKey();
		String error = PeaControl.checkPassword(cipherText, keyMaterial);
		if (error != null) {
			new UnexpectedValueException("String", "error", "is not null").printDescription();
			return fileName + " - " + error;
		} 
		// decrypt
		byte[] plainText = CipherStuff.getInstance().decrypt(
				cipherText, keyMaterial, true);
		if (plainText == null) {
			new UnexpectedValueException("byte[]", "plainText", "is null").printDescription();
			TestLog.e(getClass(), CipherStuff.getErrorMessage());
			return fileName + " - " + PeaProperties.getVmBridge().translate("failed") + " (decryption)";
		}
		PCalendar[] cals = new ReadICal().readCalendarsFromBytes(plainText, fileName, true);
		if (cals == null) {
			new UnexpectedValueException("PCalendar[]", "cals", "is null").printDescription();
			return fileName + " - " + CalendarSetting.getResourceBundle().getString("no_valid_calendar_found");
		}
		for (int i = 0; i < cals.length; i++) {
			CalendarControl.showCalendar(cals[i],fileName);
		}
		return null;// no error occurred
	}

	@Override
	public String saveCurrentContent() {
		// TODO TEST
		// CalendarMenuBar:  saveAllCalendars
		String error = menuBar.saveAllCalendars();
		if (error == null) {
		} else {
			PeaDialog.showMessage(this, 
							PeaProperties.getVmBridge().translate("operation_failed") + ":\n"
					+ PeaProperties.getVmBridge().translate("save"), 
					PeaProperties.getVmBridge().translate("error"), 0);		
			((LockFrameCalendar)PeaControl.getDialog().getLockFrame() )
				.setMessage(PeaProperties.getVmBridge().translate("operation_failed"), true);	
		}
		return null;
	}

	@Override
	public String[] chooseFileNamesToApply() {
		// TODO ??? already chosen ???
		return null;
	}
}
