package cologne.eck.calendar_pea.gui.calendar_view;

/*
 * PeaFactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */
//import static org.junit.jupiter.api.Assertions.*;

import java.awt.Color;
import java.awt.Dimension;
import java.io.File;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.HashSet;

import javax.swing.JFrame;

import org.junit.Before;

import cologne.eck.calendar_pea.control.CalendarControl;
import cologne.eck.calendar_pea.data.PCalendar;
import cologne.eck.calendar_pea.data.PEvent;
import cologne.eck.calendar_pea.gui.LockFrameCalendar;

class CalendarLayoutTest {
	
	private static JFrame frame = new JFrame();
	private static DayPanel dp = new DayPanel(LocalDate.now(), new Dimension(600,500), false);

	@Before
	static final void initialize() {
		HashSet<String> list = new HashSet<String>();
		list.add(System.getProperty("user.dir") + File.separator + "XX");
		PCalendar c = new PCalendar("XX", "YY", new Color(185,75,76,127), list);
		
		PEvent eSingle = new PEvent(c);
		eSingle.setDateStart(LocalDateTime.now().withHour(1).withMinute(0));
		eSingle.setDateEnd(LocalDateTime.now().withHour(4).withMinute(0));
		eSingle.setSummary("1-4");
		eSingle.setDescription("1 von 3");
		c.addEvent(eSingle);
		
		
		PEvent e = new PEvent(c);
		e.setDateStart(LocalDateTime.now().withHour(7).withMinute(0));
		e.setDateEnd(LocalDateTime.now().withHour(8).withMinute(0));
		e.setSummary("7-8");
		e.setDescription("2 von 3");
		e.setPriority(2);
		c.addEvent(e);		
		//CalendarControl.showCalendar(c);
		

		PEvent e1 = new PEvent(c);
		e1.setDateStart(LocalDateTime.now().withHour(7).withMinute(0));
		e1.setDateEnd(LocalDateTime.now().withHour(12).withMinute(0));
		e1.setSummary("7-12");
		e1.setDescription("3 von 3");
		c.addEvent(e1);
	//	CalendarControl.setCurrentView("day");

		PEvent e1a = new PEvent(c);
		e1a.setDateStart(LocalDate.now().atStartOfDay());
		e1a.setAllDay(true);
		//e6.setDateEnd(LocalDateTime.now().withHour(21).withMinute(30));
		e1a.setSummary("all-day 1 von 3");
		c.addEvent(e1a);
		CalendarControl.addShownCalendar(c,null);
		
		list = new HashSet<String>();
		list.add(System.getProperty("user.dir") + File.separator + "XXX");
		PCalendar c2 = new PCalendar("XXX", "YY", new Color(255,0,255,127), list);
		PEvent e2 = new PEvent(c2);
		e2.setDateStart(LocalDateTime.now().withHour(9).withMinute(0));
		e2.setDateEnd(LocalDateTime.now().withHour(15).withMinute(30));
		e2.setSummary("9-20.30");
		e2.setDescription("1 von 5");
		c2.addEvent(e2);
		//CalendarControl.showCalendar(c2);
		
		PEvent e2a = new PEvent(c2);
		e2a.setDateStart(LocalDateTime.now().withHour(13).withMinute(15));
		e2a.setDateEnd(LocalDateTime.now().withHour(16).withMinute(15));
		e2a.setSummary("13.15-16.15");
		e2a.setDescription("2 von 5");
		e2a.setPriority(1);
		c2.addEvent(e2a);

		PEvent e3 = new PEvent(c2);
		e3.setDateStart(LocalDateTime.now().withHour(17).withMinute(0));
		e3.setDateEnd(LocalDateTime.now().withHour(19).withMinute(30));
		e3.setSummary("17-19.30");
		e3.setDescription("3 von 5");
		c2.addEvent(e3);
		
		PEvent e4 = new PEvent(c2);
		e4.setDateStart(LocalDateTime.now().withHour(18).withMinute(0));
		e4.setDateEnd(LocalDateTime.now().withHour(19).withMinute(30));
		e4.setSummary("18-19.30");
		e4.setDescription("4 von 5  		PEvent e4 = new PEvent(c2);\n" + 
				"		e4.setDateStart(LocalDateTime.now().withHour(18).withMinute(0));\n" + 
				"		e4.setDateEnd(LocalDateTime.now().withHour(19).withMinute(30))");
		c2.addEvent(e4);
		
		PEvent e5 = new PEvent(c2);
		e5.setDateStart(LocalDateTime.now().withHour(20).withMinute(0));
		e5.setDateEnd(LocalDateTime.now().withHour(21).withMinute(30));
		e5.setSummary("20-21.30");
		e5.setDescription("5 von 5");
		c2.addEvent(e5);
				
		PEvent e6 = new PEvent(c2);
		e6.setDateStart(LocalDate.now().atStartOfDay());
		e6.setAllDay(true);
		//e6.setDateEnd(LocalDateTime.now().withHour(21).withMinute(30));
		e6.setSummary("all-day 2 von 3");
		c2.addEvent(e6);
		
		PEvent e7 = new PEvent(c2);
		e7.setDateStart(LocalDate.now().atStartOfDay());
		e7.setAllDay(true);
		//e6.setDateEnd(LocalDateTime.now().withHour(21).withMinute(30));
		e7.setSummary("all-day 3 von 3");
		c2.addEvent(e7);
		
		CalendarControl.addShownCalendar(c2, null);
		
	//	dp.setPreferredSize(new Dimension(500,500));
		LockFrameCalendar.setView(dp);
		dp.revalidate();

		
		frame.setSize(new Dimension(600,600));
		frame.add(dp);
		frame.setVisible(true);
		frame.pack();
	}
	
	public static void main(String[] args) {
		initialize();
	/*	
		try {
			Thread.sleep(2000);
		} catch (InterruptedException e6) {
			e6.printStackTrace();
		}
		
		frame.setSize(new Dimension (100,100));
		
		try {
			Thread.sleep(1000);
		} catch (InterruptedException e6) {
			e6.printStackTrace();
		}
		
		frame.setSize(new Dimension (800,800));
		dp.setMessage("Test ok... Test ok... Test ok... Test ok... Test ok... ", false);
		*/
	}

}
