package cologne.eck.calendar_pea.gui.calendar_view;

/*
 * PeaFactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;

import java.time.LocalDate;
import java.time.temporal.WeekFields;
import java.util.ArrayList;
import java.util.Locale;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.border.MatteBorder;

import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.all_peas.vm_specific.JREProperties;
import cologne.eck.calendar_pea.control.CalendarControl;
import cologne.eck.calendar_pea.control.CalendarSetting;
import cologne.eck.calendar_pea.gui.event_display.EventButton;
import cologne.eck.tools.UnexpectedValueException;


@SuppressWarnings("serial")
public class WeekPanel extends JPanel 
	implements CalendarView {

	/*
	 * The date of this DayPanel
	 */
	private LocalDate firstShownDate;	
	/*
	 * Label to display the currently shown date
	 */
	private JLabel weekNumberLabel;	

	/*
	 * panel for events except all-day events
	 */
	private JPanel dayPanelHolder;

	/*
	 * Scroll bar to set the view port depending on setting start time
	 */
	private JScrollBar verticalBar;	
	
	private ArrayList<ArrayList<ArrayList<EventButton>>> eventButtonLists;
	private int dayIndex = 0;
	

	/**
	 * panel to show event of a day: 
	 * extra panel for all-day events
	 * 
	 * @param _date	the date of this panel
	 * @param dim 	the dimension of this panel
	 */
	public WeekPanel(LocalDate _date, Dimension dim) {
		
		this.firstShownDate = _date;
		this.setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
		if (dim == null) {
			int[] dimValues = JREProperties.getFrameDim();
			dim = new Dimension(dimValues[0], dimValues[1]);
			JREProperties.getFrameDim();//CalendarSetting.getViewDimension();//new Dimension(125, 500);
			new UnexpectedValueException("dim", "Dimension", "is null").printStackTraceLines();
		} 

		WeekFields weekFields = WeekFields.of(Locale.getDefault()); 
		int weekNumber = firstShownDate.get(weekFields.weekOfWeekBasedYear());
		String week = CalendarSetting.getResourceBundle().getString("week") + ": " + weekNumber;
		if (firstShownDate.getDayOfWeek().getValue() != 1) { //not monday
			week = week + " - " + (weekNumber +1);
		}
		weekNumberLabel = new JLabel(week);//CalendarConstants.getFullDateFormatter().format(firstShownDate));
		this.add(weekNumberLabel);
		this.add(Box.createVerticalStrut(10));

		
		dayPanelHolder = new JPanel(); // does not hold all-day events and date label
		dayPanelHolder.setName("weekView");
		// set the view of the view port:
		dayPanelHolder.setPreferredSize(new Dimension(dim.width - (dim.width / 4), dim.height + (dim.height / 2)));
		dayPanelHolder.setLayout(new GridLayout(1,7));
		
		// scroll pane for vertical scroll only (horizontal is handled in LockFrameCalendar)
		// this scroll includes only the DayPanels without date label and all-day events
		JScrollPane timeEventsVerticalScroll = new JScrollPane(dayPanelHolder);
		timeEventsVerticalScroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		// date label and all-day events are not part of the scroll pane...
		timeEventsVerticalScroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);	
		timeEventsVerticalScroll.setPreferredSize(new Dimension( dim.width, dim.height));
		// the vertical scroll bar is set in CalendarLayout
		verticalBar = timeEventsVerticalScroll.getVerticalScrollBar();
		verticalBar.setValue((verticalBar.getMaximum() / 24) * CalendarSetting.getViewStartHour());

		// dimension for DayPanels
		Dimension dayDim = new Dimension((dim.width / 7), dim.height);		
		
		// panel for date labels (not scrolled vertically)
		JPanel dateLabelHolder = new JPanel(); 
		dateLabelHolder.setMaximumSize(new Dimension((int)PswDialogView.getScreenwidth(), 30)); 
		// consider scroll bar width:
		dateLabelHolder.setBorder(new MatteBorder(0,0,0,verticalBar.getPreferredSize().width, dateLabelHolder.getBackground()));
		dateLabelHolder.setLayout(new GridLayout(1,7));		
		
		// panel for all-day events (not scrolled vertically)
		JPanel allDayPanelHolder = new JPanel();
		allDayPanelHolder.setMaximumSize(new Dimension((int)PswDialogView.getScreenwidth(), 30)); 
		// consider scroll bar width:
		allDayPanelHolder.setBorder(new MatteBorder(0,0,0,verticalBar.getPreferredSize().width, allDayPanelHolder.getBackground()));
		allDayPanelHolder.setLayout(new GridLayout(1,7));
		
		// set the events
//		eventButtonLists = CalendarControl.getSortedEventButtonList(firstShownDate, 7);
//TestLog.o(getClass(), "list " +eventButtonLists.size());// = CalendarControl.getSortedEventButtonList(firstShownDate, 7);
		
		// add dateLabel, allDayPanel, dayPanel
		MatteBorder rightBorder = new MatteBorder(0, 0, 0, 1, Color.LIGHT_GRAY);
		for (int i = 0; i < 7; i++) {
			DayPanel dp = new DayPanel(firstShownDate.plusDays(i), dayDim, true);
			JLabel dateLabel = dp.getDateLabel();
			JPanel allDayPanel = dp.getAllDayPanel();
			if (i < 6) {
				dp.setBorder(rightBorder);
				dateLabel.setBorder(rightBorder);
				allDayPanel.setBorder(rightBorder);
			}
			dayPanelHolder.add(dp);
			dateLabelHolder.add(dateLabel);
			allDayPanelHolder.add(allDayPanel);
		}
		// add one by one, to scroll only time events
		this.add(dateLabelHolder);
		this.add(allDayPanelHolder);
		this.add(timeEventsVerticalScroll);
	}
	
	
	@Override
	public void refreshView() {
		
		eventButtonLists = CalendarControl.getSortedShownEventButtonList(this.firstShownDate, 7);
//TestLog.o(getClass(), "eventButtonLists size: " + eventButtonLists.size());
		
		WeekFields weekFields = WeekFields.of(Locale.getDefault()); 
		int weekNumber = this.firstShownDate.get(weekFields.weekOfWeekBasedYear());
		String week = CalendarSetting.getResourceBundle().getString("week") + ": " + weekNumber;
		if (this.firstShownDate.getDayOfWeek().getValue() != 1) { //not Monday
			week = week + " - " + (weekNumber +1);
		}
		weekNumberLabel.setText(week);
		int size = 7;//dayPanelHolder.getComponentCount();
		for (int i = 0; i < size; i++) {
			DayPanel viewPanel = (DayPanel) dayPanelHolder.getComponent(i);//).getViewPanel();
			viewPanel.setFirstShownDate(this.firstShownDate.plusDays(i));
			// for background reset
			if (viewPanel.getFirstShownDate().equals(LocalDate.now())) {
				viewPanel.setBackground(CalendarSetting.getTodayColor());
			} else {
				viewPanel.setBackground(CalendarSetting.getDayBackgroundColor());
			}
			//TestLog.o(getClass(), "Week set for day: " + viewPanel.getFirstShownDate() + ", week: " + this.firstShownDate);
			viewPanel.refreshView();
			//viewPanel.getLayout().layoutContainer(viewPanel);
		}
		this.revalidate();
		this.repaint();
	}
	/**
	 * Get an ArrayList of EventButtons for the next day. 
	 * This list contains two lists: at index 0 all-day events, 
	 * at index 1 events with time
	 * 
	 * @return 	a list of two lists of EventButtons: at index 0 all-day events, 
	 * 			at index 1 events with time
	 */
	public ArrayList<ArrayList<EventButton>> getNextDayList(){
		
		if (eventButtonLists == null) {
			ArrayList<ArrayList<EventButton>> emptyLists = new ArrayList<ArrayList<EventButton>>();
			emptyLists.add(new ArrayList<EventButton>());
			emptyLists.add(new ArrayList<EventButton>());
			return emptyLists;
		} else {
			if (dayIndex > 6) {
				dayIndex = 0; //reset
			}
			return eventButtonLists.get(dayIndex++);
		}
	}
	
	/**
	 * Set the vertical scroll bar of the view port
	 * of the viewPanel. This will set the first shown hour.
	 * 
	 * @param value	the first shown hour
	 */
	public void setVerticalScrollBar(int value) {
		//verticalBar.get
		verticalBar.setValue(value);
	}


	@Override
	public LocalDate getFirstShownDate() {
		return firstShownDate;
	}
	/**
	 * Set the date for this day view.
	 * The view needs to be refreshed. 
	 * 
	 * @param newDate	the date for this day view
	 */
	public void setFirstShownDate(LocalDate newDate) {
		//weekNumberLabel.setText(CalendarConstants.getFullDateFormatter().format(firstShownDate));
		this.firstShownDate = newDate;
		WeekFields weekFields = WeekFields.of(Locale.getDefault()); 
		int weekNumber = firstShownDate.get(weekFields.weekOfWeekBasedYear());
		String week = CalendarSetting.getResourceBundle().getString("week") + ": " + weekNumber;
		if (firstShownDate.getDayOfWeek().getValue() != 1) { //not monday
			week = week + " - " + (weekNumber +1);
		}
		weekNumberLabel.setText(week);
	}


	@Override
	public JPanel getPanel() {
		return this;
	}

}
