package cologne.eck.calendar_pea.gui.event_display;

/*
 * PeaFactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GraphicsConfiguration;
//import java.awt.GraphicsDevice;
import java.awt.Point;

import java.util.Arrays;
import java.util.ResourceBundle;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JTextArea;
import javax.swing.JWindow;

import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.calendar_pea.control.CalendarConstants;
import cologne.eck.calendar_pea.control.CalendarSetting;
import cologne.eck.calendar_pea.data.PEvent;
import cologne.eck.calendar_pea.data.RecurringPEvent;
import cologne.eck.tools.TestLog;

@SuppressWarnings("serial")
public class EventInfo extends JWindow {

	// static instance
	private static EventInfo info = null;
	
	// the event button that starts this info
	private EventButton eb = null;

	// default font to use for label and text areas
	private static Font infoFont = new Font(Font.SANS_SERIF, Font.PLAIN, 12);

	private static int fixedInfoWidth = 350;// width of this
	private static int minHeight = 120; // minimum height
	private static int maxHeight = 500; // maximum height

	
	private ResourceBundle languageBundle;
	
	
	/**
	 * Tool tip like info window to show infos about an event, 
	 * when a mouse enters an EventButton
	 * 
	 * @param e		the event to show
	 * @param eb	the affected EventButton
	 */
	private EventInfo(PEvent e, EventButton _eb) {

		info = this;	
		this.eb = _eb;
		if (_eb == null) {
			TestLog.ve(getClass(), "Missing event", 5);
			return;
		}

		// background color depends an calendar color
		Color backgroundColor = e.getCalendar().getColor();
		backgroundColor = new Color(backgroundColor.getRed(),backgroundColor.getGreen(), backgroundColor.getBlue(), 127);
		// font color depends on brightness of background (black or white)
		Color fontColor = EventButton.getAppropriateFontColor(backgroundColor);

		// language bundle to use
		languageBundle = CalendarSetting.getResourceBundle();
		
		if (e.getStatus() != null) {
			if (e.getStatus().equals("CANCELLED")) {
				CancelledEventPanel cep = new CancelledEventPanel();
				this.setContentPane(cep);
			} else if (e.getStatus().equals("TENTATIVE")) {
				TentativeEventPanel tep = new TentativeEventPanel();
				this.setContentPane(tep);
			}
		}
		JPanel contentPane = (JPanel) this.getContentPane();
		
		
		contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.PAGE_AXIS));		
		
		// getPreferredSize does not work because of line wrap in text areas
		// -> collect heights of components
		double prefHeight = 0;

		// panel for date: one-day events: one date, several: two separated dates
		JPanel dateLabelPanel = new JPanel();
		dateLabelPanel.setLayout(new BoxLayout(dateLabelPanel, BoxLayout.LINE_AXIS));
		if (e.isSeveralDays()) { // show start at begin and end at end
			String dateString = CalendarConstants.getShortDateFormatter().format(e.getDateStart());
			JLabel dateStartLabel = new JLabel(dateString);
			dateStartLabel.setFont(infoFont);		
			dateLabelPanel.add(dateStartLabel);
			dateLabelPanel.add(Box.createHorizontalGlue());
			if (e.getDateEnd() != null) {
				JLabel dateEndLabel = new JLabel(CalendarConstants.getShortDateFormatter().format(e.getDateEnd()));	
				dateEndLabel.setFont(infoFont);
				dateLabelPanel.add(dateEndLabel);
			}
		} else { // one day event			
			JLabel dateLabel = new JLabel(CalendarConstants.getFullDateFormatter().format(e.getDateStart()));
			dateLabel.setFont(infoFont);		
			dateLabelPanel.add(dateLabel);
			dateLabelPanel.add(Box.createHorizontalGlue());
			prefHeight += dateLabelPanel.getPreferredSize().getHeight();
		}

		// time: all-day: no time, one-day: start and end time in one label
		// several days: start and end separated
		String endTime = null;
		if (e.getDateEnd() != null) {
			endTime = CalendarConstants.getTimeFormatter().format(e.getDateEnd());
		} else {
			// maybe recurrence rule sets date end
			//endTime = "unknown";
		}
		JPanel timeLabelPanel = null;
		if (! e.isAllDay() ) {
			timeLabelPanel = new JPanel();
			timeLabelPanel.setLayout(new BoxLayout(timeLabelPanel, BoxLayout.LINE_AXIS));

			if (e.isSeveralDays()) { // show start at begin and end at end
				JLabel timeLabel = new JLabel(CalendarConstants.getTimeFormatter().format(e.getDateStart()));
				timeLabel.setFont(infoFont);
				timeLabelPanel.add(Box.createHorizontalStrut(10));
				timeLabelPanel.add(timeLabel);
				timeLabelPanel.add(Box.createHorizontalGlue());
				if (e.getDateEnd() != null) {
					JLabel timeEndLabel = new JLabel(endTime);
					timeEndLabel.setFont(infoFont);
					timeLabelPanel.add(timeEndLabel);
				}
			} else {
				String startString = CalendarConstants.getTimeFormatter().format(e.getDateStart());
				if (endTime != null) {
					startString += " - " + endTime;
				}
				JLabel timeLabel = new JLabel(startString);
				timeLabel.setFont(infoFont);
				timeLabelPanel.add(Box.createHorizontalStrut(10));
				timeLabelPanel.add(timeLabel);
				timeLabelPanel.add(Box.createHorizontalGlue());
			}
			prefHeight += timeLabelPanel.getPreferredSize().getHeight();
		}
		// only for recurring events: show frequency
		JPanel dateTimeInfoPanel = null;
		if (e.isRecurring()) {
			dateTimeInfoPanel = new JPanel();
			dateTimeInfoPanel.setLayout(new BoxLayout(dateTimeInfoPanel, BoxLayout.LINE_AXIS));
			
			JLabel dateTimeInfoLabel = null;
			RecurringPEvent re = (RecurringPEvent) e;
			String dtInfo = null;
			if (re.getFrequencyString() != null) {
				dtInfo = languageBundle.getString(re.getFrequencyString().toString());
			} else {
				dtInfo = languageBundle.getString("recurring_event");
			}
			if (re.getCount() > 0) {
				dtInfo += " * " + (re.getCount() );
			} else if (re.getUntil() != null) {
				dtInfo += " " + languageBundle.getString("until_date") + " " + CalendarConstants.getShortDateFormatter().format(re.getUntil());
			}
			if (re.getInterval() > 1) {
				dtInfo += ", " + languageBundle.getString("interval") + ": " + re.getInterval();
			}
			dateTimeInfoLabel = new JLabel(dtInfo);
			dateTimeInfoLabel.setFont(infoFont);
			dateTimeInfoPanel.add(dateTimeInfoLabel);
			prefHeight += dateTimeInfoPanel.getPreferredSize().getHeight();
		}
		// show summary always, event if null
		String summary = null;
		if (e.getSummary() == null) {
			summary = " ";
		} else {
			summary = e.getSummary();
		}
		JTextArea summaryArea = new JTextArea();
		summaryArea.setBackground(backgroundColor);
		summaryArea.setForeground(fontColor);
		summaryArea.setText(summary);
		summaryArea.setFont(new Font(infoFont.getFontName(), Font.BOLD, infoFont.getSize() + 2));
		summaryArea.setLineWrap(true);
		summaryArea.setWrapStyleWord(true);
		// estimate the number of lines
		int stringWidth = summaryArea.getFontMetrics(summaryArea.getFont()).stringWidth(summary);
		if (stringWidth > fixedInfoWidth) {
			// quick and dirty...
			double lines = stringWidth / fixedInfoWidth;// + 1;
			if (lines > 4) {
				//lines = lines +1;
			}
			prefHeight += summaryArea.getPreferredSize().getHeight() * lines;
		} else {
			prefHeight += summaryArea.getPreferredSize().getHeight();
		}
		// show description if exists
		JTextArea descriptionArea = null;		
		if (e.getDescription() != null && ( ! e.getDescription().equals(""))) {
			descriptionArea = new JTextArea();
			descriptionArea.setFont(infoFont);
			descriptionArea.setBackground(backgroundColor);
			descriptionArea.setForeground(fontColor);
			descriptionArea.setLineWrap(true);
			descriptionArea.setWrapStyleWord(true);
			descriptionArea.setText(e.getDescription());
			stringWidth = descriptionArea.getFontMetrics(infoFont).stringWidth(e.getDescription());
			if (stringWidth > fixedInfoWidth) {
				// quick and dirty...
				double lines = stringWidth / fixedInfoWidth;// + 1;
				prefHeight += descriptionArea.getPreferredSize().getHeight() * lines;
			} else {
				prefHeight += descriptionArea.getPreferredSize().getHeight();
			}
		}
		// show location if exists
		JTextArea locationArea = null;		
		if (e.getLocation() != null && ( ! e.getLocation().equals(""))) {
			locationArea = new JTextArea();
			locationArea.setFont(infoFont);
			locationArea.setBackground(backgroundColor);
			locationArea.setForeground(fontColor);
			locationArea.setLineWrap(true);
			locationArea.setWrapStyleWord(true);
			locationArea.setText(e.getLocation());
			stringWidth = locationArea.getFontMetrics(infoFont).stringWidth(e.getLocation());
			if (stringWidth > fixedInfoWidth) {
				// quick and dirty...
				double lines = stringWidth / fixedInfoWidth;// + 1;
				prefHeight += locationArea.getPreferredSize().getHeight() * lines;
			} else {
				prefHeight += locationArea.getPreferredSize().getHeight();
			}
		}
		JPanel statusAndPriorityPanel = new JPanel();
		statusAndPriorityPanel.setLayout(new BoxLayout(statusAndPriorityPanel, BoxLayout.LINE_AXIS));
		JLabel priorityLabel = null;
		int priority = e.getPriority();
		if (priority > 0) {
			char[] prio = new char[10- priority];
			Arrays.fill(prio, '*');
			priorityLabel = new JLabel(languageBundle.getString("priority") 
					+ " " + String.valueOf(prio));
			priorityLabel.setForeground(Color.RED);
			prefHeight += priorityLabel.getPreferredSize().getHeight();
			statusAndPriorityPanel.add(priorityLabel);
			statusAndPriorityPanel.add(Box.createHorizontalGlue());
		}
		String status = e.getStatus();
		if (status != null) {
			if (status.equals("TENTATIVE")) {
				statusAndPriorityPanel.add(new JLabel(languageBundle.getString("status")  + languageBundle.getString("tentative")));
			} else if (status.equals("CANCELLED")) {
				statusAndPriorityPanel.add(new JLabel(languageBundle.getString("status")  + languageBundle.getString("cancelled")));
			} else if (status.equals("CONFIRMED")) {
				statusAndPriorityPanel.add(new JLabel(languageBundle.getString("status")  + languageBundle.getString("confirmed")));
			}  
		}

		contentPane.add(dateLabelPanel);
		if (timeLabelPanel != null) {
			contentPane.add(timeLabelPanel);
		}
		if (dateTimeInfoPanel != null) {
			contentPane.add(dateTimeInfoPanel);
		}
		contentPane.add(new JSeparator(JSeparator.HORIZONTAL));
		contentPane.add(summaryArea);
		if (descriptionArea != null) {
			contentPane.add(new JSeparator(JSeparator.HORIZONTAL));
			contentPane.add(descriptionArea);
		}
		if (locationArea != null) {
			contentPane.add(new JSeparator(JSeparator.HORIZONTAL));
			contentPane.add(locationArea);
		}
		if (priority > 0 || status != null) {
			contentPane.add(statusAndPriorityPanel);
		}

		// set the height: minimum and maximum
		double infoHeight = prefHeight + 20;//some extra space for margins...

		if (infoHeight < minHeight) {
			infoHeight = minHeight;
		}
		if (infoHeight > maxHeight) {
			infoHeight = maxHeight;
		}
		this.setPreferredSize(new Dimension(fixedInfoWidth, (int)infoHeight));				
		this.pack();		
		//this.setLocationRelativeTo(eb);// unsuitable: covers the EventButton
		
		// set location manually
		Point ebLoc = eb.getLocationOnScreen(); // top-left on screen

		if (PswDialogView.getNumberOfScreens() < 2 || PeaControl.isWindowsOS() == true) {
			// Linux displays the EventInfo on wrong position if app is on the second screen
			double  screenHeight = PswDialogView.getScreenheight();
			double screenWidth = PswDialogView.getScreenwidth();
			int xLoc = ebLoc.x + (eb.getWidth() );// end of EventButton
			// get x
			if ((xLoc + fixedInfoWidth) > screenWidth) {
				//xLoc = ebLoc.x - ebWidth;
				xLoc = (int) (screenWidth - fixedInfoWidth);// move left to fit in screen
			}
			// get y
			int yLoc = ebLoc.y + (eb.getHeight() / 2); // middle of EventButton
			if ( (yLoc + infoHeight) > screenHeight) {
				yLoc = (int) (screenHeight - infoHeight);// set over screen end
				if ( ( ebLoc.x - fixedInfoWidth) > 0) { // set x on left side of EventButton if this is inside the screen
					xLoc = ebLoc.x - fixedInfoWidth;
				}
			}
			//TestLog.v(getClass(), "LOCATION: x: "+ xLoc + ", y: " + yLoc);
			this.setLocation(new Point(xLoc, yLoc));// in parent coordinates!
		} else {
			// multiple screens: that meen problem especially under Linux... 
			// This is a workaround:
			this.setLocationRelativeNoCover(eb, fixedInfoWidth, (int) infoHeight);
		}		
		this.setVisible(true);
	}
	 /**
	  * Set EventInfo relative to EventButton, 
	  * but never cover the EventButton.
	  * If possible right beside and same y, 
	  * if not left beside / over EventButton. 
	  * This should also work for multiple screens
	  * on Linux (workaround)
	  * 
	  * @param c				Component to be relative to
	  * @param fixedInfoWidth	EventInfo width
	  * @param infoHeight		EventInfo height
	  */
	 private void setLocationRelativeNoCover(Component c, int fixedInfoWidth, int infoHeight) {
		 
		 int priorY = c.getLocationOnScreen().y; 
		 super.setLocationRelativeTo(c);
		 this.setVisible(true);
		 
		 // now shift:
		 int x = this.getLocationOnScreen().x;
		 int y = this.getLocationOnScreen().y;
		 int xLoc = 0;
		 int yLoc = 0;
		 
		 int compWidth = c.getWidth();//.getPreferredSize().width;
		 int compHeight = c.getPreferredSize().height;
		 
		 GraphicsConfiguration config = c.getGraphicsConfiguration();
//		 GraphicsDevice myScreen = config.getDevice();
		 
		 //TestLog.v(getClass(), "current screen ID: " + myScreen.getIDstring());
		// TestLog.v(getClass(), "bounds: " + config.getBounds());
		 //TestLog.v(getClass(), "x: " + x + ", y: " + y);
		 
		 xLoc = x + ((fixedInfoWidth + compWidth) / 2);// right beside
		 if (xLoc + fixedInfoWidth > (config.getBounds().width + config.getBounds().x) ) {
			 xLoc = x - ((fixedInfoWidth + compWidth) / 2);// left beside
		 }
		 
		 
		 yLoc = priorY ;// same
		 if (yLoc + infoHeight > (config.getBounds().height + config.getBounds().y) ) {
			 yLoc = y - ((infoHeight + compHeight) / 2);// over
		 }		 
		 this.setLocation(xLoc, yLoc);
		 
	 }

	

	// for test only
//	public static void main(String[] args) {
	//	PCalendar c = new PCalendar("xx", "xxxx", Color.BLUE);
	/*	PEvent e = new PEvent(c);
		e.setSummary("event summary...");
		e.setDescription("event description...\n");//...\n...\n...\n...\n...\n...\n...end");
		e.setDateStart(LocalDateTime.now());
		e.setDateEnd(LocalDateTime.now().plusHours(2));*/
		
/*		RecurringPEvent re = new RecurringPEvent(c);
		re.setSummary("event summary...event summary.event summary...event summary...e");
		re.setDescription("event description... escription...  event description.event description... escription...  event description...event description... escription...  event description... event description... escription...  event description...  event description... event description... event description... event description... event description... event description... event description... ENDE ");//...\n...\n...\n...\n...\n...\n...end");
		re.setDateStart(LocalDateTime.now());
		re.setDateEnd(LocalDateTime.now().plusHours(2));
		re.setFrequency(ChronoUnit.WEEKS);
		re.setPriority(2);
		
		EventInfo ei = EventInfo.getInstance(re, null);
		ei.setLocation(100,100);
		ei.setVisible(true); */
//	}

	/**
	 * Get the EventInfo for an event, shown as tooltip like info for an EventButton. 
	 * There is always only one instance. 
	 * 
	 * @param _e	event to show
	 * @param _eb	EventButton with mouse enter event
	 * 
	 * @return	the EventInfo
	 */
	protected static final EventInfo getInstance(PEvent _e, EventButton _eb) {
		if (info != null && info.isVisible()) { // close current info
			info.setVisible(false);
			info.dispose();
		}
		return new EventInfo(_e, _eb);
	}

	/**
	 * Set the font for the EventInfo
	 * 
	 * @param infoFont	the font to set
	 */
/*	public static void setInfoFont(Font infoFont) {

		EventInfo.infoFont = infoFont;
	}*/

	/**
	 * Set the minimum height of the EventInfo. 
	 * 
	 * @param newMinHeight	the minimum height of the EventInfo
	 */
/*	public static void setInfoMinHeight(int newMinHeight) {

		EventInfo.minHeight = newMinHeight;
	}*/

	/**
	 * Set the maximum height of the EventInfo. Note: There is no
	 * scroll pane, longer infos will be cut.
	 * 
	 * @param newMaxHeight	the maximum height of the EventInfo
	 */
/*	public static void setInfoMaxHeight(int newMaxHeight) {

		EventInfo.maxHeight = newMaxHeight;
	} */
	/**
	 * Set the fixed width of the EventInfo.
	 * 
	 * @param newWidth	the width to set as int
	 */
/*	public static void setInfoWidth(int newWidth) {

		EventInfo.fixedInfoWidth = newWidth;
	}*/
	/**
	 * Dispose the current EventInfo
	 */
	public static void disposeInfo() {
		if (info != null && info.isVisible()) {
			info.setVisible(false);
			info.dispose();
		}
	}
}
