package cologne.eck.calendar_pea.gui.event_setting;

/*
 * PeaFactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.KeyboardFocusManager;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneId;
import java.time.temporal.ChronoUnit;
import java.util.Calendar;
import java.util.Date;
import java.util.ResourceBundle;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JSpinner.DateEditor;//setStartDateSpinner
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SpinnerDateModel;
import javax.swing.Timer;
//import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.Document;

import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.gui.IconManager;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.calendar_pea.control.CalendarControl;
import cologne.eck.calendar_pea.control.CalendarSetting;
import cologne.eck.calendar_pea.data.PCalendar;
import cologne.eck.calendar_pea.data.PEvent;
import cologne.eck.calendar_pea.data.RecurringPEvent;
import cologne.eck.calendar_pea.gui.CalendarComboBox;
import cologne.eck.calendar_pea.gui.LockFrameCalendar;
import cologne.eck.tools.TestLog;
import cologne.eck.tools.UnexpectedValueException;


@SuppressWarnings("serial")
public class EventSetter extends JDialog implements ActionListener, ChangeListener, DocumentListener {
	
	private PEvent existingEvent = null;

	private boolean newEvent = false;
	
	
	private JTabbedPane tabbedPane;
	private JPanel contentPaneMain;

	private JTextField summaryField;
	private JTextArea descriptionArea;
	private JTextField locationField;
	
	private CalendarComboBox calendarComboBox;
	private JSpinner startDateSpinner;
	private TimePicker startTimePicker;
	private JSpinner endDateSpinner;
	private TimePicker endTimePicker;
	
	private JLabel mainMessageLabel;
	private JLabel moreMessageLabel;
	private JLabel recurrenceMessageLabel;

	private JScrollPane scroll;
	
	/*
	 * Panels to remove and add time setting for
	 * all-day events
	 */
	private JPanel endDatePanel;
	private JPanel startTimeAndDatePanel;	
	private JPanel endTimeAndDatePanel;	

	private boolean allDay = false;
	private boolean isRecurring = false;
	
	
	//private JCheckBox recurringCheckX;
	// Panel that holds the check box and if selected the RecurrencePanel at index 1
	private JPanel recPanel;
	
	private JComboBox <String> priorityComboBox;
	private JComboBox <String> statusComboBox;		
	//private JTextField categoriesField;
	private JTextField urlField;
	private JButton urlButton;
/*	private JTextField commentField;	
	private JTextField locationField; */
	//private JTextField resourcesField = null;
//	private RecurrencePanel recurrencePanel;
	/*
	private JSpinner durationWeeksSpinner;
	private JSpinner durationDaysSpinner;
	private JSpinner durationHoursSpinner;
	private JSpinner durationMinutesSpinner;
	*/
	

	/**
	 * Show an EventSetter to set date, time, summary and description...
	 * 
	 * @param owner			the owner component
	 * @param _existingEvent		the event to set or null for new event
	 * @param _newEvent		true, if the Event is not yet listed
	 * @param selectedDate	the date and time to start the EventSetters date spinner or null
	 */
	public EventSetter(Component owner, //Point location,
			PEvent _existingEvent, boolean _newEvent, 
			LocalDateTime selectedDate) { 
		

		ResourceBundle languagesBundle = CalendarSetting.getResourceBundle();
		// reset last message:
		((LockFrameCalendar)PeaControl.getDialog().getLockFrame() ).setMessage(null, false);
		mainMessageLabel = new JLabel();
		
		// initialize start and end:
		LocalDateTime defaultStartDateTime = null;
		LocalDateTime defaultEndDateTime = null;
		// add new Event or modify existing Event
		this.newEvent =	_newEvent; 
		if (newEvent == false) { // EventSetter was started by click on eventButton
			TestLog.v(getClass(), "Modify existing event...");
			existingEvent = _existingEvent;
			if (existingEvent == null) {
				TestLog.e(getClass(), "Event to add is null!");
				new UnexpectedValueException("existingEvent", "PEvent", "is null").printDescription();
				return;
			}
			if (existingEvent.getCalendar() == null) {
				new UnexpectedValueException("existingEvent.getCalendar()", "PCalendar", "for existing event is null: " + existingEvent.getSummary()).printDescription();
				return;
			}
			if (existingEvent.isRecurring()) {
				setRecurring(true);
			} else {
				setRecurring(false);
			}

			// set date start and end, even if selectedDate != null, 
			// because of recurring and several-days events:
			defaultStartDateTime = existingEvent.getDateStart();
			if (existingEvent.getDateEnd() != null) {
				if (existingEvent.isAllDay() == true 
						// default for one day event: start date = end date
						&& ! existingEvent.getDateEnd().equals(existingEvent.getDateStart())) {
					// decrement one day in spinner
					defaultEndDateTime = existingEvent.getDateEnd().minusDays(1);
				} else {
					defaultEndDateTime = existingEvent.getDateEnd();
				}
			} else {
				if (existingEvent.isAllDay() == true) { // one day event
					defaultEndDateTime = defaultStartDateTime;
				} else {
					// set one hour later
					defaultEndDateTime = existingEvent.getDateStart().plusHours(1);
					mainMessageLabel.setText("Missing date end: is set one hour after date start");
					this.pack();
				}
			}

		} else { // EventSetter was started by click on dayPanel, monthPanel...:			
			if (selectedDate != null) {
				defaultStartDateTime = selectedDate;//, LocalTime.of(12, 0));
				defaultEndDateTime = defaultStartDateTime.plusHours(1);
			} else {
				// set date to now and time to default time 12:00
				defaultStartDateTime = LocalDateTime.now().withHour(12).withMinute(0).withSecond(0);
				// set one hour later:
				defaultEndDateTime = defaultStartDateTime.plusHours(1);
			}
		}

		this.setTitle("Calendar Lock PEA - " + languagesBundle.getString("event_setting"));
		this.setModal(false);
		this.setAlwaysOnTop(true);			

		contentPaneMain = new JPanel();//(JPanel) this.getContentPane();
		this.add(contentPaneMain);
		contentPaneMain.setLayout(new BoxLayout(contentPaneMain, BoxLayout.PAGE_AXIS));
		contentPaneMain.setBorder(new EmptyBorder(10, 10, 10, 10));
		String okButtonText = languagesBundle.getString("set_event");
		if (newEvent == false) {
			okButtonText = languagesBundle.getString("change_event");
		}

		// ComboBox to select the PCalendar for this Event
		JPanel calendarPanel = new JPanel();
		calendarPanel.setLayout(new BoxLayout(calendarPanel, BoxLayout.X_AXIS));
		JLabel calendarLabel = new JLabel(languagesBundle.getString("calendar"));
		calendarPanel.add(calendarLabel);
		calendarPanel.add(Box.createHorizontalStrut(20));

		calendarComboBox = new CalendarComboBox();
		if (calendarComboBox.getItemCount() <= 0) {
			// no calendars
			return;
		}
		if (newEvent == false) { // set the calendar
			PCalendar c = existingEvent.getCalendar();
			String cName = c.getName();
			calendarComboBox.setSelectedCalendar(cName);
		}
		// to check if calendar was changed
		calendarComboBox.addActionListener(this);
		calendarComboBox.setActionCommand("calendarChange");
		calendarPanel.add(calendarComboBox);
		calendarPanel.add(Box.createHorizontalGlue());
		//contentPaneMain.add(calendarPanel);
		contentPaneMain.add(calendarPanel);

		// set the background color
		if (existingEvent != null) {
			if (existingEvent.getCalendar() != null &&
					existingEvent.getCalendar().getColor() != null) {
				this.getContentPane().setBackground(existingEvent.getCalendar().getColor());
			} else {
				TestLog.ve(getClass(), "Cant set color...", 5);
			}
		} else {
			if (calendarComboBox.getSelectedItem() instanceof String) {
				String cal = (String) calendarComboBox.getSelectedItem();				
				PCalendar c = CalendarControl.getCalendarByName(cal);
				if (c != null) {
					Color color = c.getColor();
					this.getContentPane().setBackground(color);
				} else {
					TestLog.ve(getClass(), "Missing color", 5);	
				}
			} else {
				TestLog.ve(getClass(), "Invalid item", 5);
			}
		}

		JPanel allDayCheckPanel = new JPanel();
		allDayCheckPanel.setLayout(new BoxLayout(allDayCheckPanel, BoxLayout.LINE_AXIS));
		JCheckBox allDayCheck = new JCheckBox(languagesBundle.getString("all_day_event"));
		allDayCheckPanel.add(Box.createHorizontalGlue());
		allDayCheckPanel.add(allDayCheck);
		if (_newEvent == false && existingEvent != null) {
			if (existingEvent.isAllDay()) {
				allDayCheck.setSelected(true);
				allDay = true;
			} 
		}
		allDayCheck.addActionListener(this);
		allDayCheck.setActionCommand("all-day");

		//contentPaneMain.add(allDayCheckPanel);
		contentPaneMain.add(allDayCheckPanel);

		/* Panel with Spinners to choose start and end time and date */
		JPanel timeAndDatePanel = new JPanel();
		timeAndDatePanel.setLayout(new BoxLayout(timeAndDatePanel, BoxLayout.PAGE_AXIS));

		JPanel startLabelPanel = new JPanel();
		startLabelPanel.setLayout(new BoxLayout(startLabelPanel, BoxLayout.LINE_AXIS));
		JLabel startLabel = new JLabel(languagesBundle.getString("start"));
		startLabel.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 12));
		startLabelPanel.add(startLabel);
		startLabelPanel.add(Box.createHorizontalGlue());
		timeAndDatePanel.add(startLabelPanel);

		/* Panel for start date and time */
		startTimeAndDatePanel = new JPanel();
		startTimeAndDatePanel.setLayout(new BoxLayout(startTimeAndDatePanel, BoxLayout.LINE_AXIS));		
		startTimeAndDatePanel.add(Box.createHorizontalStrut(10));		
		/* start date */
		startTimeAndDatePanel.add(new JLabel(languagesBundle.getString("date")));
		startTimeAndDatePanel.add(Box.createHorizontalStrut(5));
		// Spinner for start date: 
		SimpleDateFormat dateModel = new SimpleDateFormat("dd.MM.yy");

		Date defaultStartDate = java.util.Date
				.from(defaultStartDateTime.atZone(ZoneId.systemDefault())
						.toInstant());

		SpinnerDateModel startDateModel = new SpinnerDateModel(defaultStartDate,
					null, null, Calendar.DATE);
		startDateSpinner = new JSpinner(startDateModel);
		startDateSpinner.setName("startDateSpinner");
		startDateSpinner.setEditor(new JSpinner.DateEditor(startDateSpinner, dateModel.toPattern()));
		startDateSpinner.setMaximumSize(new Dimension(150, 25));
		startDateSpinner.setMinimumSize(new Dimension(150, 25));
		startDateSpinner.addChangeListener(this);
		
		// add a date-picker button
		JButton startDatePickerButton = new JButton(IconManager.loadScaledIcon("date-picker.png", 
				"Date Picker to select a date", 30, 25,"Date Picker", "Date Picker to select a date"));
		startDatePickerButton.setBorder(BorderFactory.createEmptyBorder());
		startDatePickerButton.addActionListener(this);
		startDatePickerButton.setActionCommand("datePickerStart");
		
		startTimeAndDatePanel.add(startDateSpinner);
		this.add(Box.createHorizontalStrut(5));
		startTimeAndDatePanel.add(startDatePickerButton);

		/* start time */
		startTimePicker = new TimePicker(defaultStartDateTime.toLocalTime());
		startTimePicker.getHourSpinner().setName("startHourSpinner");
		startTimePicker.getHourSpinner().addChangeListener(this);

		startTimeAndDatePanel.add(Box.createHorizontalGlue());
		if (allDay == false) {
			startTimeAndDatePanel.add(startTimePicker);
		}
				
		timeAndDatePanel.add(startTimeAndDatePanel);
		timeAndDatePanel.add(Box.createVerticalStrut(5));

		/* end of event */
		JPanel endLabelPanel = new JPanel();
		endLabelPanel.setLayout(new BoxLayout(endLabelPanel, BoxLayout.LINE_AXIS));
		JLabel endLabel = new JLabel(languagesBundle.getString("end"));
		endLabel.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 12));
		endLabelPanel.add(endLabel);
		endLabelPanel.add(Box.createHorizontalGlue());
		timeAndDatePanel.add(endLabelPanel);		

		/* Panel for end date and time */
		endTimeAndDatePanel = new JPanel();
		endTimeAndDatePanel.setLayout(new BoxLayout(endTimeAndDatePanel, BoxLayout.LINE_AXIS));
		endTimeAndDatePanel.add(Box.createHorizontalStrut(10));
		
		endDatePanel = new JPanel();
		endDatePanel.setLayout(new BoxLayout(endDatePanel, BoxLayout.LINE_AXIS));
		
		endDatePanel.add(new JLabel(languagesBundle.getString("date")));
		endDatePanel.add(Box.createHorizontalStrut(5));
		// date end spinner
		SpinnerDateModel dateModelEnd = new SpinnerDateModel(
				java.util.Date.from( defaultEndDateTime
						.atZone(ZoneId.systemDefault()).toInstant()),	 
				null, null, Calendar.DATE);
		
		endDateSpinner = new JSpinner(dateModelEnd);			
		endDateSpinner.setEditor(new JSpinner.DateEditor(endDateSpinner, dateModel.toPattern()));
		endDateSpinner.setMaximumSize(new Dimension(150, 25));
		endDateSpinner.setMinimumSize(new Dimension(150, 25));
		
		// add a date-picker button
		JButton endDatePickerButton = new JButton(IconManager.loadScaledIcon("date-picker.png", 
				"Date Picker to select a date", 30, 25,"Date Picker", "Date Picker to select a date"));
		endDatePickerButton.setBorder(BorderFactory.createEmptyBorder());
		endDatePickerButton.addActionListener(this);
		endDatePickerButton.setActionCommand("datePickerEnd");

		endDatePanel.add(endDateSpinner);
		endDatePanel.add(endDatePickerButton);


		endTimeAndDatePanel.add(endDatePanel);
		if (existingEvent != null && existingEvent.isRecurring() == true) {
			endDatePanel.setVisible(false);
		}
		endTimeAndDatePanel.add(Box.createHorizontalGlue());

		// time end spinner		
		endTimePicker = new TimePicker( defaultEndDateTime.toLocalTime());
		if (allDay == false) {
			endTimeAndDatePanel.add(endTimePicker);
		}

		timeAndDatePanel.add(endTimeAndDatePanel);
		timeAndDatePanel.add(Box.createVerticalStrut(10));
		
/*		} else if (event.getDuration() != null) {
			
			JPanel durationLabelPanel = new JPanel();
			durationLabelPanel.setLayout(new BoxLayout(durationLabelPanel, BoxLayout.X_AXIS));
			JLabel endLabel = new JLabel("duration: " );
			durationLabelPanel.add(endLabel);
			durationLabelPanel.add(Box.createHorizontalGlue());
			timeAndDatePanel.add(durationLabelPanel);
			
			JPanel durationPanel = new JPanel();
			durationPanel.setLayout(new BoxLayout(durationPanel, BoxLayout.X_AXIS));
			durationPanel.add(Box.createHorizontalStrut(10));
			
			Duration duration = event.getDuration().getValue();
			int minutes = 0;
			int hours = 0;
			int days = 0;
			int weeks = 0;
			if (duration.getSeconds() != null) {
				minutes = duration.getMinutes();
			}
			if (duration.getSeconds() != null) {
				hours = duration.getHours();
			}
			if (duration.getSeconds() != null) {
				days = duration.getDays();
			}
			if (duration.getSeconds() != null) {
				weeks = duration.getWeeks();
			}
			durationPanel.add(new JLabel("weeks: "));
			durationPanel.add(Box.createHorizontalStrut(5));			
		    durationWeeksSpinner = new JSpinner();
		    durationWeeksSpinner.setModel(new SpinnerNumberModel(weeks, 0, 52,1)); // value, min, max, step
		    durationWeeksSpinner.setMaximumSize(new Dimension(60, 25));
		    durationWeeksSpinner.setMinimumSize(new Dimension(60, 25));			    
		    durationPanel.add(durationWeeksSpinner);
		    durationPanel.add(Box.createHorizontalStrut(10));		    
			durationPanel.add(new JLabel("days: "));
			durationPanel.add(Box.createHorizontalStrut(5));			
		    durationDaysSpinner = new JSpinner();
		    durationDaysSpinner.setModel(new SpinnerNumberModel(days, 0, 31,1)); // value, min, max, step
		    durationDaysSpinner.setMaximumSize(new Dimension(60, 25));
		    durationDaysSpinner.setMinimumSize(new Dimension(60, 25));			    
		    durationPanel.add(durationDaysSpinner);
		    durationPanel.add(Box.createHorizontalStrut(10));			    
			durationPanel.add(new JLabel("hours: "));
			durationPanel.add(Box.createHorizontalStrut(5));			
		    durationHoursSpinner = new JSpinner();
		    durationHoursSpinner.setModel(new SpinnerNumberModel(hours, 0, 24,1)); // value, min, max, step
		    durationHoursSpinner.setMaximumSize(new Dimension(60, 25));
		    durationHoursSpinner.setMinimumSize(new Dimension(60, 25));				    
		    durationPanel.add(durationHoursSpinner);
		    durationPanel.add(Box.createHorizontalStrut(10));				    
			durationPanel.add(new JLabel("minutes: "));
			durationPanel.add(Box.createHorizontalStrut(5));			
		    durationMinutesSpinner = new JSpinner();
		    durationMinutesSpinner.setModel(new SpinnerNumberModel(minutes, 0, 60,1)); // value, min, max, step
		    durationMinutesSpinner.setMaximumSize(new Dimension(60, 25));
		    durationMinutesSpinner.setMinimumSize(new Dimension(60, 25));			    
		    durationPanel.add(durationMinutesSpinner);
		    durationPanel.add(Box.createHorizontalStrut(10));						
			durationPanel.add(Box.createHorizontalGlue());
			timeAndDatePanel.add(durationPanel);
*/					
		contentPaneMain.add(timeAndDatePanel);

		
		//===============
		// SUMMARY ======
		JPanel summaryLabelPanel = new JPanel();		
		summaryLabelPanel.setLayout(new BoxLayout(summaryLabelPanel, BoxLayout.LINE_AXIS));
		JLabel summaryLabel = new JLabel(languagesBundle.getString("summary"));
		//summaryLabel.setMaximumSize(new Dimension(800, 30));
		summaryLabelPanel.add(summaryLabel);
		summaryLabelPanel.add(Box.createHorizontalGlue());
		contentPaneMain.add(summaryLabelPanel);		
		
		summaryField = new JTextField();//50);
		if (existingEvent != null && existingEvent.getSummary() != null) {
			summaryField.setText(existingEvent.getSummary());
		}
	//summaryField.setMaximumSize(new Dimension(500, (int)summaryField.getMaximumSize().getHeight()));
		summaryField.setPreferredSize(new Dimension(200, (int)summaryField.getPreferredSize().getHeight()));
		summaryField.setMaximumSize(new Dimension(800, 50));
		//summaryField.setMinimumSize(new Dimension((int)summaryField.getPreferredSize().getWidth(), 30));
		contentPaneMain.add(summaryField);		
		contentPaneMain.add(Box.createVerticalStrut(10));		
		
		//===================
		// DESCRIPTION ======
		JPanel descriptionLabelPanel = new JPanel();
		descriptionLabelPanel.setLayout(new BoxLayout(descriptionLabelPanel, BoxLayout.LINE_AXIS));
		JLabel descriptionLabel = new JLabel(languagesBundle.getString("description"));
		descriptionLabelPanel.add(descriptionLabel);
		descriptionLabelPanel.add(Box.createHorizontalGlue());
		contentPaneMain.add(descriptionLabelPanel);
		
		descriptionArea = new JTextArea();
		if (existingEvent != null && existingEvent.getDescription() != null) {
			descriptionArea.setText(existingEvent.getDescription());
		}
		descriptionArea.setLineWrap(true);
		descriptionArea.setWrapStyleWord(true);
		// change behavior of tab key: from TAB to focus for accessibility
		descriptionArea.setFocusTraversalKeys(KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS, null);
		descriptionArea.setFocusTraversalKeys(KeyboardFocusManager.BACKWARD_TRAVERSAL_KEYS, null);
		JScrollPane descriptionScroll = new JScrollPane(descriptionArea);
		descriptionScroll.setPreferredSize(new Dimension(400, 60));
		
		contentPaneMain.add(descriptionScroll);		
		contentPaneMain.add(Box.createVerticalStrut(10));
				
		JPanel messageLabelPanel = new JPanel();
		messageLabelPanel.setLayout(new BoxLayout(messageLabelPanel, BoxLayout.LINE_AXIS));
		messageLabelPanel.add(mainMessageLabel);
		contentPaneMain.add(messageLabelPanel);
		
		JPanel okButtonPanel = new JPanel();
		okButtonPanel.setLayout(new BoxLayout(okButtonPanel, BoxLayout.LINE_AXIS));		
		okButtonPanel.setMaximumSize(new Dimension(800, 40));
		if (newEvent == false) {
			JButton removeButton = new JButton(languagesBundle.getString("remove_event"));
			removeButton.addActionListener(this);
			removeButton.setActionCommand("remove");
			okButtonPanel.add(removeButton);
		}
		okButtonPanel.add(Box.createHorizontalGlue());

		JPanel setButtonPanel = new JPanel();
		JButton setButton = new JButton(okButtonText);
		setButton.addActionListener(this);
		setButton.setActionCommand("set");	
		setButtonPanel.add(setButton);
		//Color c = setButtonPanel.getBackground();		
		//setButtonPanel.setBorder(new LineBorder(c.brighter(), 5));
		setButton.setBackground(new Color(179, 255, 179));
		okButtonPanel.add(setButtonPanel);			

		contentPaneMain.add(okButtonPanel);
		
		
		// ===================== RecurrencePanel =======================================
		
		recPanel = new JPanel();
		recPanel.setLayout(new BoxLayout(recPanel, BoxLayout.PAGE_AXIS));
		recPanel.setBorder(new EmptyBorder(10, 10, 10, 10));

		// display the panel with recurrence rule:
		RecurrencePanel rp = new RecurrencePanel(existingEvent, defaultStartDateTime.toLocalDate(), this);
		recPanel.add(rp);
		recPanel.add(Box.createVerticalStrut(50));
		
		JPanel recurrenceLabelPanel = new JPanel();
		recurrenceLabelPanel.setLayout(new BoxLayout(recurrenceLabelPanel, BoxLayout.LINE_AXIS));
		recurrenceMessageLabel = new JLabel();
		recurrenceLabelPanel.add(recurrenceMessageLabel);
		recPanel.add(recurrenceLabelPanel);
		
		JPanel okButtonPanel2 = new JPanel();
		okButtonPanel2.setLayout(new BoxLayout(okButtonPanel2, BoxLayout.LINE_AXIS));		
		okButtonPanel2.setMaximumSize(new Dimension(800, 40));
		if (newEvent == false) {
			JButton removeButton2 = new JButton(languagesBundle.getString("remove_event"));
			removeButton2.addActionListener(this);
			removeButton2.setActionCommand("remove");
			okButtonPanel2.add(removeButton2);
		}
		okButtonPanel2.add(Box.createHorizontalGlue());		
		JPanel setButtonPanel2 = new JPanel();
		JButton setButton2 = new JButton(okButtonText);
		setButton2.addActionListener(this);
		setButton2.setActionCommand("set");	
		setButtonPanel2.add(setButton2);
		//Color c = setButtonPanel.getBackground();		
		//setButtonPanel.setBorder(new LineBorder(c.brighter(), 5));
		setButton2.setBackground(new Color(179, 255, 179));
		okButtonPanel2.add(setButtonPanel2);			
		recPanel.add(okButtonPanel2);

		scroll = new JScrollPane(contentPaneMain);	
		JScrollPane scrollRecurrence = new JScrollPane(recPanel);

		
		//============================= MORE OPTIONS TAB

		JPanel morePanel = new JPanel();
		morePanel.setLayout(new BoxLayout(morePanel, BoxLayout.PAGE_AXIS));
		morePanel.setBorder(new EmptyBorder(10, 10, 10, 10));
		
		JPanel moreLabelPanel = new JPanel();
		moreLabelPanel.setLayout(new BoxLayout(moreLabelPanel, BoxLayout.LINE_AXIS));
		JLabel moreLabel = new JLabel(languagesBundle.getString("more_options"));
		Font f = moreLabel.getFont();
		moreLabel.setFont(new Font(f.getFamily(), Font.BOLD, f.getSize()));
		moreLabelPanel.add(moreLabel);
		moreLabelPanel.add(Box.createHorizontalGlue());
		morePanel.add(Box.createVerticalStrut(10));
		morePanel.add(moreLabelPanel);
		morePanel.add(Box.createVerticalStrut(30));
		
		// LOCATION ======
		JPanel locationLabelPanel = new JPanel();		
		locationLabelPanel.setLayout(new BoxLayout(locationLabelPanel, BoxLayout.LINE_AXIS));
		JLabel locationLabel = new JLabel(languagesBundle.getString("location"));
		//summaryLabel.setMaximumSize(new Dimension(800, 30));
		locationLabelPanel.add(locationLabel);
		locationLabelPanel.add(Box.createHorizontalGlue());
		morePanel.add(locationLabelPanel);		
		
		locationField = new JTextField();//50);
		if (existingEvent != null && existingEvent.getLocation() != null) {
			locationField.setText(existingEvent.getLocation());
		}
	//summaryField.setMaximumSize(new Dimension(500, (int)summaryField.getMaximumSize().getHeight()));
		locationField.setPreferredSize(new Dimension(200, (int)locationField.getPreferredSize().getHeight()));
		locationField.setMaximumSize(new Dimension(800, 50));
		//summaryField.setMinimumSize(new Dimension((int)summaryField.getPreferredSize().getWidth(), 30));
		morePanel.add(locationField);		
		morePanel.add(Box.createVerticalStrut(20));		
		
/*		JPanel categoriesLabelPanel = new JPanel();
		categoriesLabelPanel.setLayout(new BoxLayout(categoriesLabelPanel, BoxLayout.X_AXIS));
		JLabel categoriesLabel = new JLabel("categories: "); // TODO language
		categoriesLabelPanel.add(categoriesLabel);
		categoriesLabelPanel.add(Box.createHorizontalGlue());
		morePanel.add(categoriesLabelPanel);
		
		categoriesField = new JTextField();
		if (existingEvent != null && existingEvent.getCategories() != null) {*/
		/*	String[] categories =   existingEvent.getCategories().split(",");
			int len = categories.length;
			StringBuilder builder = new StringBuilder();
			for (int i = 0; i < len; i++) {
				List<String> catList = categories[i].getValues();
				int listLen = catList.size();
				for (int j = 0; j < listLen; j++) {					
					builder.append(catList.get(j));
					builder.append(", ");
				}
			}
			// delete last ,
			builder.delete(builder.length() - 2, builder.length());
			categoriesField.setText(new String(builder));*/
/*			categoriesField.setText(existingEvent.getCategories()); // TODO separate
		}
		categoriesField.setPreferredSize(new Dimension(200, (int)categoriesField.getPreferredSize().getHeight()));
		categoriesField.setMaximumSize(new Dimension(800, 50));
		morePanel.add(categoriesField);		
		morePanel.add(Box.createVerticalStrut(10));		*/
		
		
		JPanel urlLabelPanel = new JPanel();
		urlLabelPanel.setLayout(new BoxLayout(urlLabelPanel, BoxLayout.X_AXIS));
		JLabel urlLabel = new JLabel(PeaProperties.getVmBridge().translate("web") + ": ");
		urlLabelPanel.add(urlLabel);
		urlLabelPanel.add(Box.createHorizontalGlue());
		urlButton = new JButton(PeaProperties.getVmBridge().translate("go_to"));
		urlButton.addActionListener(this);
		urlButton.setActionCommand("openUrl");
		urlLabelPanel.add(urlButton);
		morePanel.add(urlLabelPanel);
		
		urlField = new JTextField();
		urlField.getDocument().addDocumentListener(this); // check valid URL
		if (existingEvent != null && existingEvent.getUrl() != null) {
			urlField.setText(existingEvent.getUrl().toString());
		}
		urlField.setPreferredSize(new Dimension(200, (int)urlField.getPreferredSize().getHeight()));
		urlField.setMaximumSize(new Dimension(800, 50));
		morePanel.add(urlField);		
		morePanel.add(Box.createVerticalStrut(10));	
		
		//================
		// PRIORITY ======
		JPanel priorityPanel = new JPanel();
		priorityPanel.setLayout(new BoxLayout(priorityPanel, BoxLayout.X_AXIS));
		JLabel priorityLabel = new JLabel(languagesBundle.getString("priority"));
		priorityPanel.add(Box.createHorizontalStrut(10));
		priorityPanel.add(priorityLabel);
		priorityPanel.add(Box.createHorizontalStrut(20));
		
		priorityComboBox = new JComboBox <String>();
		priorityComboBox.addItem("no");
		priorityComboBox.addItem("1");
		priorityComboBox.addItem("2");
		priorityComboBox.addItem("3");
		priorityComboBox.addItem("4");
		priorityComboBox.addItem("5");
		priorityComboBox.addItem("6");
		priorityComboBox.addItem("7");		
		priorityComboBox.addItem("8");
		priorityComboBox.addItem("9");
		priorityComboBox.setMaximumSize(new Dimension(50, 25));
		if (existingEvent != null && existingEvent.getPriority() > 0 && existingEvent.getPriority() < 10) {
			priorityComboBox.setSelectedIndex(existingEvent.getPriority());
		}
		priorityPanel.add(priorityComboBox);
		priorityPanel.add(Box.createHorizontalGlue());
		morePanel.add(priorityPanel);
		morePanel.add(Box.createVerticalStrut(10));
		
		
		// STATUS ======
		JPanel statusPanel = new JPanel();
		statusPanel.setLayout(new BoxLayout(statusPanel, BoxLayout.X_AXIS));
		JLabel statusLabel = new JLabel("status");
		statusPanel.add(Box.createHorizontalStrut(10));
		statusPanel.add(statusLabel);
		statusPanel.add(Box.createHorizontalStrut(20));
		
		statusComboBox = new JComboBox <String>();
		statusComboBox.addItem("no");
		statusComboBox.addItem(languagesBundle.getString("tentative"));
		statusComboBox.addItem(languagesBundle.getString("confirmed"));
		statusComboBox.addItem(languagesBundle.getString("cancelled"));
		statusComboBox.setMaximumSize(new Dimension(100, 25)); 
		if (existingEvent != null && existingEvent.getStatus() != null) {
			String currentStatus = existingEvent.getStatus();
			int index = 0;
			if (currentStatus.equals("TENTATIVE")){
				index = 1;
			} else if (currentStatus.equals("CONFIRMED")){
				index = 2;
			} else if (currentStatus.equals("CANCELLED")){
				index = 3;
			}  else {
				index = 0;
				//new UnexpectedValueException("currentStatus", "String", " is unknown").printStackTraceLines();
			}
			statusComboBox.setSelectedIndex(index);
		} 
		statusPanel.add(statusComboBox);
		statusPanel.add(Box.createHorizontalGlue());
		morePanel.add(statusPanel);
		
		
/*		JPanel commentLabelPanel = new JPanel();
		commentLabelPanel.setLayout(new BoxLayout(commentLabelPanel, BoxLayout.X_AXIS));
		JLabel commentLabel = new JLabel("comment");
		commentLabelPanel.add(commentLabel);
		commentLabelPanel.add(Box.createHorizontalGlue());
		contentPane.add(commentLabelPanel);
		
		commentField = new JTextField();
		if (event.getComments().isEmpty() == false) {
			String[] comments = (String[]) event.getComments().toArray();
			int len = comments.length;
			StringBuilder builder = new StringBuilder();
			for (int i = 0; i < len; i++) {
				builder.append(comments[i]);
				builder.append(", ");
			}
			// delete last ,
			builder.delete(builder.length() - 2, builder.length());
			commentField.setText(new String(builder));
		}
		contentPane.add(commentField);		
		contentPane.add(Box.createVerticalStrut(10));		*/
		
/*		JPanel resourcesLabelPanel = new JPanel();
		resourcesLabelPanel.setLayout(new BoxLayout(resourcesLabelPanel, BoxLayout.X_AXIS));
		JLabel resourcesLabel = new JLabel("resources");
		resourcesLabelPanel.add(resourcesLabel);
		resourcesLabelPanel.add(Box.createHorizontalGlue());
		contentPane.add(resourcesLabelPanel);
		
		resourcesField = new JTextField();
		if (event.getResources() != null) {
			Iterator <Resources> iterator = event.getResources().iterator();
			StringBuilder builder = new StringBuilder();
			while (iterator.hasNext()) {
				builder.append(iterator.next().toString());
				builder.append(", ");
			}
			// delete last ,
//			builder.delete(builder.length() - 2, builder.length());
			resourcesField.setText(new String(builder));
		}
		contentPane.add(resourcesField);		
		contentPane.add(Box.createVerticalStrut(10));*/		
		
		morePanel.add(Box.createVerticalStrut(20));	

		moreMessageLabel = new JLabel();
		moreMessageLabel.setForeground(Color.RED);
		JPanel moreMessageLabelPanel = new JPanel();
		moreMessageLabelPanel.setLayout(new BoxLayout(moreMessageLabelPanel, BoxLayout.LINE_AXIS));
		moreMessageLabelPanel.add(moreMessageLabel);
		morePanel.add(moreMessageLabelPanel);

		JPanel okButtonPanel3 = new JPanel();
		okButtonPanel3.setLayout(new BoxLayout(okButtonPanel3, BoxLayout.LINE_AXIS));	
		okButtonPanel3.setMaximumSize(new Dimension(800, 40));
		if (newEvent == false) {
			JButton removeButton3 = new JButton(languagesBundle.getString("remove_event"));
			removeButton3.addActionListener(this);
			removeButton3.setActionCommand("remove");
			okButtonPanel3.add(removeButton3);
		}
		okButtonPanel3.add(Box.createHorizontalGlue());		
		JPanel setButtonPanel3 = new JPanel();
		JButton setButton3 = new JButton(okButtonText);
		setButton3.addActionListener(this);
		setButton3.setActionCommand("set");	
		setButtonPanel3.add(setButton3);
		//Color c = setButtonPanel.getBackground();		
		//setButtonPanel.setBorder(new LineBorder(c.brighter(), 5));
		setButton3.setBackground(new Color(179, 255, 179));
		okButtonPanel3.add(setButtonPanel3);			
		morePanel.add(okButtonPanel3);
		
		JScrollPane scrollMore = new JScrollPane(morePanel);
		

		//====================== TABS: 
		
		tabbedPane = new JTabbedPane();
		tabbedPane.addTab(languagesBundle.getString("main"), null, scroll,
				languagesBundle.getString("main_settings"));
		tabbedPane.addTab(languagesBundle.getString("more"), null, scrollMore,
				languagesBundle.getString("more_options"));
		tabbedPane.addTab(languagesBundle.getString("repeat"), null, scrollRecurrence,
                languagesBundle.getString("recurrence"));//"Recurrence settings"); 
		this.getContentPane().add(tabbedPane);
		


		this.pack();
		//if (location == null) {
		this.setLocationRelativeTo(owner);
	/*	} else {
			this.setLocation(location);
		}*/

		//this.setMinimumSize(new Dimension (200, 100));
		//this.setMaximumSize(new Dimension(500, 500));
		this.setVisible(true);
	}

	@Override
	public void pack() {
		super.pack();
		
		// set scroll pane if larger than screen:
		Dimension prefSize = this.getPreferredSize();
		if (prefSize.width > PswDialogView.getScreenwidth()){
			scroll.setPreferredSize(new Dimension ((int)PswDialogView.getScreenwidth(), (int) prefSize.getHeight()));
		}
		if (prefSize.height > PswDialogView.getScreenheight()) {
			scroll.setPreferredSize(new Dimension ((int) prefSize.getWidth(), (int)PswDialogView.getScreenheight()));
		}		
	}


	@Override
	public void actionPerformed(ActionEvent ap) {
		String command = ap.getActionCommand();
		
		// reset messageLabel:
		if (mainMessageLabel != null) {
			mainMessageLabel.setText("");
		} else {
			TestLog.ve(getClass(), "Missing message label...", 5);
		}
		if (moreMessageLabel != null) {		
			moreMessageLabel.setText("");
		} else {
			TestLog.ve(getClass(), "Missing message label...", 5);
		}
		if (recurrenceMessageLabel != null) {
			recurrenceMessageLabel.setText("");
		} else {
			TestLog.ve(getClass(), "Missing message label...", 5);
		}
		
		// the event to modify or to create
		PEvent event = null;
		// the calendar, this event belongs to
		PCalendar modifiedCalendar = null;
		if (newEvent == false) {
			//TestLog.o(getClass(), "EventSetter newEvent FALSE, cal listLen: "+existingEvent.getCalendar().getEventList().size());
			if (existingEvent.getCalendar().getEventList().size() == 0) {
				new UnexpectedValueException("int", "size of eventList", "is 0:"+ 
						existingEvent.getCalendar().getName() + " " + existingEvent.getCalendar().getCalUid()).printDescription();
			}
			event = existingEvent;
			modifiedCalendar = existingEvent.getCalendar();
			
			if ( calendarComboBox != null) {
				String currentCal = (String) calendarComboBox.getSelectedItem();
				if ( ! modifiedCalendar.getName().equals(currentCal)) {
					// calendar was changed: event must be removed from original calendar
					// and set to new calendar
					TestLog.v(getClass(), "Calendar of event was changed, from " + modifiedCalendar.getName() 
					+ " to " + currentCal);
					if (currentCal != null) {
						PCalendar newCal = CalendarControl.getCalendarByName(currentCal);
						if (newCal != null) {
							// remove event
							modifiedCalendar.removeEvent(existingEvent);
							// set new calendar for event
							existingEvent.setCalendar(newCal);
							// change modified calendar:
							modifiedCalendar = newCal;
						} else {
							TestLog.e(getClass(), "No calendar was found for " + currentCal);
							TestLog.ve(getClass(), "No calendar was found for " + currentCal, 5);
							setErrorMessageToAllTabs(CalendarSetting.getResourceBundle().getString("unexpected_error")
									+ " - there is no calendar " + currentCal);
							return;
						}
					} else {
						TestLog.e(getClass(), "No calendar was selected");
						TestLog.ve(getClass(), "No calendar was selected", 5);
						setErrorMessageToAllTabs(CalendarSetting.getResourceBundle().getString("unexpected_error")
								+ " - No calendar selected ");
						return;
					}
				}
			} else {
				TestLog.ve(getClass(), "No calendar combo box", 5);
				setErrorMessageToAllTabs(CalendarSetting.getResourceBundle().getString("unexpected_error"));
				return;
			}
			
			
		} else {
			//TestLog.o(getClass(), "EventSetter newEvent TRUE");
			String calendarName = (String) calendarComboBox.getSelectedItem();
			if (calendarName == null) {
				JOptionPane.showMessageDialog(this, "You must choose a calendar for this event");
				setErrorMessageToAllTabs(CalendarSetting.getResourceBundle().getString("must_select_calendar"));
				return;
			}			
			modifiedCalendar = CalendarControl.getCalendarByName(calendarName);
			event = new PEvent(modifiedCalendar);
		}
		
		if (modifiedCalendar == null) {
			new UnexpectedValueException("modifiedCalendar", "PCalendar", "for event is null: " + event.getSummary()).printDescription();
			setErrorMessageToAllTabs(CalendarSetting.getResourceBundle().getString("must_select_calendar"));
			return;
		}
		
		if (command.equals("remove")) {
			if (newEvent == false) {
				CalendarControl.removeEvent(event, modifiedCalendar);
				CalendarControl.refreshView();
			}
			this.dispose();
			
		} else if (command.equals("removeMoreMessage")) {
			moreMessageLabel.setText(null);

		} else if (command.equals("openUrl")) {
			String url = urlField.getText();
			try {         
				java.awt.Desktop.getDesktop().browse(java.net.URI.create(url));
			} catch (Exception e) {
				moreMessageLabel.setText(e.getLocalizedMessage());
				Timer timer1 = new Timer(3000, this);
				timer1.setActionCommand("removeMoreMessage");
				timer1.setRepeats(false);
				timer1.start();
			}

		} else if (command.startsWith("datePicker")) {
						
			if (command.equals("datePickerStart")) {
				if (startDateSpinner.isEnabled() == false) {
					return;
				}
				// get current set date and convert to LocalDate
				LocalDate startDate = ((java.util.Date) startDateSpinner.getValue())
						.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
				//TestLog.o(getClass(), "startDate: " + startDate);
				
				// show the DatePicker with local date to start
				LocalDate date = new DatePicker(startDate).showDatePicker(this, null);
				// set new until date from DatePicker (convert to Date)
				startDateSpinner.setValue(java.util.Date.from(date.atStartOfDay(ZoneId.systemDefault()).toInstant()));			

			} else if (command.equals("datePickerEnd")) {
				// get current set date and convert to LocalDate
				LocalDate endDate = ((java.util.Date) endDateSpinner.getValue())
						.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
				//TestLog.o(getClass(), "startDate: " + startDate);
				
				// show the DatePicker with local date to start
				LocalDate date = new DatePicker(endDate).showDatePicker(this, null);
				// set new until date from DatePicker (convert to Date)
				endDateSpinner.setValue(java.util.Date.from(date.atStartOfDay(ZoneId.systemDefault()).toInstant()));							
			}
			
		} else if (command.equals("all-day")) {
			// add or remove time spinners
			JCheckBox check = (JCheckBox) ap.getSource();
			if (check.isSelected()) {
				allDay = true;
				event.setAllDay(true);
				startTimeAndDatePanel.remove(startTimePicker);
				endTimeAndDatePanel.remove(endTimePicker);	
				this.pack();
			} else {
				allDay = false;
				event.setAllDay(false);
				startTimeAndDatePanel.add(startTimePicker);
				endTimeAndDatePanel.add(endTimePicker);		
				this.pack();
			}
			
			// calendarComboBox.setActionCommand("calendarChange");
		} else if (command.equals("calendarChange")) {
			String calName = (String) calendarComboBox.getSelectedItem();
			if (calName != null) {
				PCalendar c = CalendarControl.getCalendarByName(calName);
				if (c != null) {
					Color color = c.getColor();
					if (color != null) {
						this.getContentPane().setBackground(color);
					}
				}
			}			
			
		} else if (command.equals("set")) {
					
			//=========================================================
			// 1. get the properties for the event:			
			
			LocalDateTime startDateTime = null;			
			LocalDate startDate = null;
			LocalDate endDate = null;
			if (allDay == true ) { // all-day event
				event.setAllDay(true);				
				startDate = ((SpinnerDateModel)startDateSpinner.getModel()).getDate().toInstant()
					      .atZone(ZoneId.systemDefault())
					      .toLocalDate();
				startDateTime = startDate.atStartOfDay();// hours, minutes 00:00
				event.setDateStart(startDateTime);
				
				if (endDateSpinner.isVisible()) {
					endDate = ((SpinnerDateModel) endDateSpinner.getModel()).getDate().toInstant()
							.atZone(ZoneId.systemDefault())
							.toLocalDate();
				} else {
					endDate = startDate;
				}
				// for value=date always one date later!!
				// date is treated as time 000000, dtend is treated as exclusive
				LocalDateTime endDateTime = endDate.atStartOfDay().plusDays(1); // hours, minutes 00:00
				event.setDateEnd(endDateTime);
				if (endDateTime.isBefore(startDateTime)) {
					setErrorMessageToAllTabs(CalendarSetting.getResourceBundle().getString("end") + 
							CalendarSetting.getResourceBundle().getString("is_before_start_date"));
					this.pack();
					return;
				}

			} else {
				event.setAllDay(false);
			
				startDate = ((SpinnerDateModel)startDateSpinner.getModel()).getDate().toInstant()
				      .atZone(ZoneId.systemDefault())
				      .toLocalDate();
				LocalTime startTime = startTimePicker.getTime();
				startDateTime = LocalDateTime.of(startDate, startTime);
				event.setDateStart(startDateTime);
				
				endDate = ((SpinnerDateModel) endDateSpinner.getModel()).getDate().toInstant()
					      .atZone(ZoneId.systemDefault())
					      .toLocalDate();
				LocalTime endTime = endTimePicker.getTime();
				if (isRecurring == true) { //recurringCheck.isSelected()) { // several days recurring is not allowed yet
					endDate = startDate;
				}
				LocalDateTime endDateTime = LocalDateTime.of(endDate, endTime);
				if (endDateTime.isBefore(startDateTime)) {
					setErrorMessageToAllTabs(CalendarSetting.getResourceBundle().getString("end") + 
							CalendarSetting.getResourceBundle().getString("is_before_start_date"));
					//mainMessageLabel.setText( CalendarSetting.getResourceBundle().getString("end") + 
					//		CalendarSetting.getResourceBundle().getString("is_before_start_date"));				
							//"event end is before event start");
					//mainMessageLabel.setForeground(Color.RED);
					this.pack();
					return;
				}

				event.setDateEnd(endDateTime);
			
/*			if (durationWeeksSpinner != null && durationWeeksSpinner.isVisible()) {
	
				int durationWeeks = (Integer) durationWeeksSpinner.getModel().getValue();
				int durationDays = (Integer) durationDaysSpinner.getModel().getValue();
				int durationHours = (Integer) durationHoursSpinner.getModel().getValue();
				int durationMinutes = (Integer) durationMinutesSpinner.getModel().getValue();
				
				Duration duration = Duration.builder().
						weeks(durationWeeks).
						days(durationDays).
						hours(durationHours).
						minutes(durationMinutes).build();

				event.setDuration(duration);
			}  */
			
			
			} // end not all-day event
			// set if this event extends to more than one day or not
			
			if (endDate.isAfter(event.getDateStart().toLocalDate())) {
				event.setSeveralDays(true);					
			} else {
				event.setSeveralDays(false);	
			}			
			RecurrencePanel rp = (RecurrencePanel) recPanel.getComponent(0);
			// An event is recurring if frequency and single exceptions are not null:
			if ( rp.getFrequency() != null || rp.getRDates() != null){// event.isRecurring()) {//.isSelected() ) {
				// recurring event:				
				RecurringPEvent rEvent = RecurringPEvent.castToRecurringPEvent(event);				
				
				// set RDATE independent of rule
				if (rp.getRDates() != null) {
					LocalDate[] rDates = rp.getRDates();
					int rDatesLen = rDates.length;
					//LocalDate startDate = rEvent.getDateStart().toLocalDate();
					// check if a date is before start date:
					for (int i=0; i < rDatesLen; i++) {
						if (rDates[i].isBefore(startDate)) {
							setErrorMessageToAllTabs( CalendarSetting.getResourceBundle().getString("single_repetitions") + 
									CalendarSetting.getResourceBundle().getString("is_before_start_date") + ": " + rDates[i]);
							this.pack();
							return;
						}
					}
					rEvent.setRecurrenceDates(rp.getRDates());
				}

				ChronoUnit frequency = rp.getFrequency();
				if (frequency == null && rEvent.getRecurrenceDates() == null) {
					setErrorMessageToAllTabs("Couldn't set recurrence: frequency missed");
					this.pack();
					event.setRecurring(false);
				} else {
					rEvent.setFrequency(frequency);
					rEvent.setInterval(rp.getInterval());
					rEvent.setByMonth(rp.getByMonth());
					rEvent.setByDay(rp.getByDayStrings());					
					int count = rp.getCount();
					LocalDateTime until = rp.getUntil();
					if (until != null && ! until.isAfter(rEvent.getDateStart())) {
						setErrorMessageToAllTabs( CalendarSetting.getResourceBundle().getString("until_date") + " - " + 
								CalendarSetting.getResourceBundle().getString("is_before_start_date") + ": " + until.toLocalDate());						
						this.pack();
						return;
					}
					rEvent.setCount(count);	
					rEvent.setUntil(until);		
					rEvent.setExceptionDates(rp.getExceptions());
					event = rEvent;
				}
			} else {				
				if (event instanceof RecurringPEvent) {
					event = RecurringPEvent.resetRecurrenceProperties((RecurringPEvent) event);
				}
				event.setRecurring(false);
			}
			
			if (summaryField.getText() != null) {
				event.setSummary(summaryField.getText());
			}
			if (descriptionArea.getText() != null) {
				event.setDescription(descriptionArea.getText());
			}
			if (locationField.getText() != null) {
				event.setLocation(locationField.getText());
			}
			if (urlField.getText() != null && urlField.getText().length() > 0) {
				try {
					URL url = new URL(urlField.getText());
					event.setUrl(url);
				} catch( Exception e) {
					TestLog.e(EventSetter.class, "Invalid URL: " + urlField.getText());
				}
			}
/*			if (categoriesField.getText() != null) {
				event.addCategories(categoriesField.getText());
			}
			if (commentField.getText() != null) {
				event.addComment(commentField.getText());
			}
			if (locationField.getText() != null) {
				event.setLocation(locationField.getText());
			} */
/*			if (resourcesField.getText() != null) {
	//			event.setRessource(resourcesField.getText());
			} */			
			int priority = priorityComboBox.getSelectedIndex();
			event.setPriority(priority);

			// set time stamp:
			event.setTimeStamp(Instant.now());
			if (newEvent == true) {
				CalendarControl.addEvent(event, modifiedCalendar);
/*				if (event.isRecurring() == false && event.isSeveralDays() == false) {
					// refresh one day
					//if (MonthPanel.getInstance(null).isVisible() == true) {
					//	CalendarControl.refreshOneDayView(startDateTime.toLocalDate());						
						CalendarControl.refreshView();
					//}
				} else {
				//	if (MonthPanel.getInstance(null).isVisible() == true) {
						CalendarControl.refreshView();
				//	}
				}
			} else { // event was probably several days or recurring before...
				//if (MonthPanel.getInstance(null).isVisible() == true) {
					CalendarControl.refreshView();
				//}*/
			}
			CalendarControl.refreshView();
			((LockFrameCalendar)PeaControl.getDialog().getLockFrame() ).setMessage(null, false);
			LockFrameCalendar.getFrame().markUnsavedContentChanges();//.setDocChangeUnsaved(true);
			this.dispose();
			int status = statusComboBox.getSelectedIndex();
			if (status >= 0) {
				if (status == 0) {
					event.setStatus(null);
				} else if (status == 1) {
					event.setStatus("TENTATIVE");
				} else if (status == 2) {
					event.setStatus("CONFIRMED");
				} else if (status == 3) {
					event.setStatus("CANCELLED");
				}
			}						
		}		
	}
	
	private void setErrorMessageToAllTabs(String message) {
		if (mainMessageLabel != null) {
			mainMessageLabel.setText(message);
			mainMessageLabel.setForeground(Color.RED);
		} else {
			TestLog.ve(getClass(), "Missing message label...", 5);
		}
		if (moreMessageLabel != null) {
			moreMessageLabel.setText(message);
			moreMessageLabel.setForeground(Color.RED);
		} else {
			TestLog.ve(getClass(), "Missing message label...", 5);
		}
		if (recurrenceMessageLabel != null) {
			recurrenceMessageLabel.setText(message);
			recurrenceMessageLabel.setForeground(Color.RED);
		} else {
			TestLog.ve(getClass(), "Missing message label...", 5);
		}
	}



	@Override
	public void stateChanged(ChangeEvent e) {
		
		JSpinner spinner = (JSpinner) e.getSource();
		if (spinner.getName().equals("startTimeSpinner")) {
			// set end time only if start date is equal or less than end date
			if ((((java.util.Date)startDateSpinner.getValue()).getTime() + (1000*60*60)) < (((java.util.Date)endDateSpinner.getValue()).getTime() )){
				return;
			} else {
				endTimePicker.setTime(startTimePicker.getTime().plusHours(1));
			}
		} else if (spinner.getName().equals("startDateSpinner")) {

			if (isRecurring == true) { //recurringCheck.isSelected() == true) {
				
				// step 1 a: set date of DTEND to DTSTART if required:
				if (! ((java.util.Date)startDateSpinner.getValue()).equals((java.util.Date)endDateSpinner.getValue())) {
					endDateSpinner.setValue(startDateSpinner.getValue());
				}
				// set 1 b: set time if end is after start:
				if ((((java.util.Date)startDateSpinner.getValue()).getTime() + (1000*60*60)) < (((java.util.Date)endDateSpinner.getValue()).getTime() )){
					// ok
				} else {
					endTimePicker.setTime(startTimePicker.getTime().plusHours(1));
				}
				
				// step 2: set & reset BYDAY values:
				if (recPanel != null && recPanel.getComponent(0) instanceof RecurrencePanel) {
					RecurrencePanel rp = (RecurrencePanel) recPanel.getComponent(0);
					// TODO		
					rp.updateStartDate( ( (java.util.Date) startDateSpinner.getValue())
							.toInstant().atZone(ZoneId.systemDefault()).toLocalDate());
				}
				
			} else {
				// reset date end if required
				if ( ! ((java.util.Date)startDateSpinner.getValue()).before((java.util.Date)endDateSpinner.getValue())) {
					endDateSpinner.setValue(startDateSpinner.getValue());
					// check if end time is after start time
					if ((((java.util.Date)startDateSpinner.getValue()).getTime() + (1000*60*60)) < (((java.util.Date)endDateSpinner.getValue()).getTime() )){
						// ok
					} else {
						// set one hour later
						endTimePicker.setTime(startTimePicker.getTime().plusHours(1));
					}
					
				}
			}
		}
	}
	/**
	 * Get the current start date of the event to set
	 * 
	 * @return	the current start date of the event
	 */
	public LocalDate getStartDate() {
		return ((java.util.Date) startDateSpinner.getValue())
			.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
	}
	/**
	 * Disable or enable the startDateSpinner. The value
	 * can't be changed if the spinner is disabled. 
	 * This is enabled, if a BY_DAY rule is defined, 
	 * because the start day must be included in the rule
	 * 
	 * @param enable	false: start date is not modifiable
	 * 					true: start date can be changed
	 */
	public void setStartDateSpinner(boolean enable) {
		startDateSpinner.setEnabled(enable);
		if (enable == false) {
			JSpinner.DateEditor editor = (DateEditor) startDateSpinner.getEditor();
			editor.getTextField().setDisabledTextColor(Color.BLACK);
		} 
	}
	/**
	 * Set if the current event is recurring.
	 * This also changes the display.
	 * 
	 * @param _recurring
	 */
	protected void setRecurring(boolean _recurring) {
		boolean originalRecurring = this.isRecurring;
		this.isRecurring = _recurring;
		if (_recurring == true) {
			if (existingEvent != null) {
				existingEvent.setRecurring(true);
			}
			if (endDatePanel != null) {
				endDatePanel.setVisible(false);
			}
		} else {
			if (existingEvent != null) {
				existingEvent.setRecurring(false);
			}
			if (endDatePanel != null) {
				endDatePanel.setVisible(true);
			}
		}
		if (originalRecurring != _recurring) {
			this.pack();
		}
	}
	/**
	 * Check if the current event is recurring
	 * 
	 * @return	true if recurring
	 */
/*	protected boolean isRecurring() {
		return this.isRecurring;
	}*/

	@Override
	public void changedUpdate(DocumentEvent de) {}
	@Override
	public void insertUpdate(DocumentEvent iu) {
		//TestLog.o(EventSetter.class, "insert");
		Document doc = iu.getDocument();
		if (urlField.getDocument().equals(doc)) {			
			//URL validUrl = null;
			try {
				String url = doc.getText(0, doc.getLength());
				new URL(url);
				urlField.setBorder(new LineBorder(Color.DARK_GRAY, 1));
			} catch (Exception e) {
				urlField.setBorder(new LineBorder(Color.RED, 1));
			}
		}
		}
	@Override
	public void removeUpdate(DocumentEvent ru) {
		//TestLog.o(EventSetter.class, "remove");
		Document doc = ru.getDocument();
		if (urlField.getDocument().equals(doc)) {			
			//URL validUrl = null;
			try {
				String url = doc.getText(0, doc.getLength());
				new URL(url);
				urlField.setBorder(new LineBorder(Color.DARK_GRAY, 1));
			} catch (Exception e) {
				urlField.setBorder(new LineBorder(Color.RED, 1));
			}
		}
	}
}
