package cologne.eck.peafactory.crypto;
/*
 * PeaFactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */


import static org.junit.jupiter.api.Assertions.*;

import java.util.Arrays;

import org.bouncycastle.crypto.engines.AESEngine;
import org.bouncycastle.crypto.engines.ThreefishEngine;
import org.junit.Assert;
import org.junit.Before;
import org.junit.jupiter.api.Test;

import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.tools.Converter;
import cologne.eck.tools.EntropyPool;

class CipherStuffExtraTest {

	
	@Before
	final void initialize() {
		PeaControl.initializeVariables();
		EntropyPool.getInstance().updateThread(System.nanoTime());
	}
	
	
	@Test
	final void testProcessCTR_AESDecrypt() {
		// There are official test vectors only for AES CTR
		CipherStuff.setCipherAlgo(new AESEngine());
		// NIST SP 800-38-A: CTR AES256 Decrypt Block #1
		//Key 603deb1015ca71be2b73aef0857d77811f352c073b6108d72d9810a30914dff4
		// iv f0f1f2f3f4f5f6f7f8f9fafbfcfdfeff
		//Plaintext 6bc1bee22e409f96e93d7e117393172a
		//Ciphertext 601ec313775789a5b7a7f504bbf3d228
		byte[] key = Converter.hex2bytes("603deb1015ca71be2b73aef0857d77811f352c073b6108d72d9810a30914dff4");
		// iv + cipher text:
		byte[] cipherText = Converter.hex2bytes("f0f1f2f3f4f5f6f7f8f9fafbfcfdfeff601ec313775789a5b7a7f504bbf3d228");
		// decrypt and get the plain text
		byte[] decryptedText = CipherStuff.processCTR(false, cipherText, key, true);
		assertEquals(Converter.bytes2hex(decryptedText), "6bc1bee22e409f96e93d7e117393172a".toUpperCase());      
	}
	@Test
	final void testProcessCTRZeroizeKey() {

		CipherStuff.setCipherAlgo(new ThreefishEngine(512));
		byte[] key = new byte[64];
		Arrays.fill(key, (byte) 0xFF);
		// iv + cipher text:
		byte[] cipherText = new byte[CipherStuff.getCipherAlgo().getBlockSize()];
		// decrypt and zeroize key
		CipherStuff.processCTR(false, cipherText, key, true);
		byte[] test = new byte[key.length ];
		Arrays.fill(test, (byte) 0);
		// key should be zeroized:
		Assert.assertTrue(Arrays.equals(key, test));
	}
	@Test
	final void testProcessCTRNotZeroizeKey() {

		CipherStuff.setCipherAlgo(new ThreefishEngine(512));

		byte[] key = new byte[64];
		Arrays.fill(key, (byte) 0xFF);
		// iv + cipher text:
		byte[] cipherText = new byte[CipherStuff.getCipherAlgo().getBlockSize()];
		// decrypt and zeroize key
		CipherStuff.processCTR(false, cipherText, key, false);
		byte[] test = new byte[key.length ];
		Arrays.fill(test, (byte) 0xFF);
		// key should not be zeroized, but original key
		Assert.assertTrue(Arrays.equals(key, test));
	}
/*
	@Test
	final void testEncrypt() { // not possible: IV is always generated
		fail("Not yet implemented"); // TODO
	}

	@Test
	final void testEncryptKeyFromKeyMaterial() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	final void testDecrypt() { // use a test file
		fail("Not yet implemented"); // TODO
	}

	@Test
	final void testDetectKey() {
		fail("Not yet implemented"); // TODO
	}

	@Test
	final void testHandleKey() {
		fail("Not yet implemented"); // TODO
	} */

}
