package cologne.eck.tools;

import java.awt.Dimension;
import java.awt.Toolkit;

/*
 * PeaFactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */


import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Properties;

import javax.swing.JFrame;

import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.vm_specific.JREProperties;
import cologne.eck.peafactory.crypto.AlgoParamHandler;
import cologne.eck.peafactory.crypto.kdf.KeyDerivation;


public class PropertyHandler {

	// only file name, path inside jar is always /, not File.separator
	private static final String propFileName = "pea.properties";//"resources" + File.separator + 
	
	//private 	String result = "";
	private InputStream inputStream;
	private OutputStream outputStream;
	
	/**
	 * Add some properties to the property file
	 * 
	 * @param props	the properties to set 
	 * 
	 * @return	an error message or null
	 */
/*	public String addProperties(Properties props) {
		String errorMessage = null;
		if (props == null) {
			System.err.println("No properties to add...");
			return null;
		}
		int len = props.size();
		if (len == 0) {
			System.err.println("No properties to add...");
			return null;
		}
		File file = new File(System.getProperty("user.dir") + File.separator 
				+ "resources" + File.separator + propFileName);
		try {
		if (! file.exists()) {
			// create directory
			file.getParentFile().mkdirs();
			// create file
			file.createNewFile();
		} 
		outputStream = new FileOutputStream(file);
		@SuppressWarnings("unchecked")
		Enumeration<String> enums = (Enumeration<String>) props.propertyNames();
	    while (enums.hasMoreElements()) {
	      String key = enums.nextElement();
	      String value = props.getProperty(key);
	      props.setProperty(key, value);
	      System.out.println(key + " : " + value);
	    }
	    if (outputStream != null) {
	    	outputStream.close();
	    }
		
		} catch (IOException e){
			System.err.println("IOException: " + e.getMessage());
			errorMessage += e + ", " + e.getLocalizedMessage();
		}
		return errorMessage;
	}*/
	
	/**
	 * Update all values in properties file
	 * that are not PEA specific. 
	 * This method is called once when a PEA is closed. 
	 * This should be called in EDT.
	 * 
	 * @return	error message or null
	 */
	public String updateAllPropValues() {

		String errorMessage = null;
		try {
			//==================================================
			// set properties for all PEAs
			Properties prop = new Properties();
			File file = new File(System.getProperty("user.dir") + File.separator 
					+ "resources" + File.separator + propFileName);
			if (! file.exists()) {
				// create directory
				file.getParentFile().mkdirs();
				// create file
				file.createNewFile();
			} 
			outputStream = new FileOutputStream(file);

			// Color ===============================
			int[] peaColor = PeaProperties.getPeaColor();
			prop.setProperty("pea_color_r", "" + peaColor[0]);
			prop.setProperty("pea_color_g", "" + peaColor[1]);
			prop.setProperty("pea_color_b", "" + peaColor[2]);
			// Check updates =======================
			int checkUpdates = PeaProperties.getCheckUpdates();
			prop.setProperty("check_updates", "" + checkUpdates);			
			// Font size ================================
			prop.setProperty("font_size", "" + PeaProperties.getFontSize());
			// dropbox access token:
			String token = PeaProperties.getDropboxAccessToken();
			if (token != null) {
				prop.setProperty("dropbox_access_token", token);
			}
			//String existingDefaultSalt = prop.getProperty("default_salt");
			String currentlyUsedSalt = Converter.bytes2hex(KeyDerivation.getSalt());
			prop.setProperty("last_salt", currentlyUsedSalt);

	/*		String algoParam = Converter.bytes2hex(PeaProperties.getAlgoParamString());
			prop.setProperty("algo_param", algoParam); */
			
			String defaultAlgoParam = PeaProperties.getDefaultAlgoParamString();
			prop.setProperty("default_algo_param", defaultAlgoParam);
			
			String defaultKeyFileName = PeaProperties.getDefaultKeyFileName();
			if (defaultKeyFileName != null) {
				prop.setProperty("key_file", defaultKeyFileName);
			}

/*			// main Dropbox API server host name
			String api = PeaProperties.getDropboxApi();
			if (api != null) {
				prop.setProperty("dropbox_api", api);
			}
			// Dropbox API content server host name
			String content = PeaProperties.getDropboxContent();
			if (content != null) {
				prop.setProperty("dropbox_content", content);
			}
			// Dropbox web server host name
			String web = PeaProperties.getDropboxWeb();
			if (web != null) {
				prop.setProperty("dropbox_web", web);
			}
			// Dropbox notification server host name
			String notify = PeaProperties.getDropboxNotify();
			if (notify != null) {
				prop.setProperty("dropbox_notify", notify);
			}*/
			
			// language =================================
//			prop.setProperty("lang", PeaProperties.getLanguage());
			// dimension and location ========================
			JFrame frame = (JFrame) PeaControl.getDialog().getLockFrame();
			if (frame == null) {
				new UnexpectedValueException("frame", "JFrame", "is null").printDescription();
			} else {
				prop.setProperty("frame_x", "" + frame.getLocationOnScreen().getX());
				prop.setProperty("frame_y", "" + frame.getLocationOnScreen().getY());
				prop.setProperty("frame_width", "" + frame.getSize().getWidth());
				prop.setProperty("frame_height", "" + frame.getSize().getHeight());
			}
			if (outputStream != null) {
				prop.store(outputStream, "Properties for all PEAs");

			} else {
				//throw new FileNotFoundException("property file '" + propFileName + "' not found in the classpath");
				new UnexpectedValueException("outputStream", "OutputStream", "file not found: " + "resources" + File.separator + propFileName).printDescription();
				errorMessage = "file not found: " + "resources" + File.separator + propFileName;
			}
			try {
				if (outputStream != null) {
				outputStream.close();
				}
			} catch (IOException e) {
				System.err.println("PropertyHandler IOException: " + e.getMessage());
				errorMessage += e + ", " + e.getLocalizedMessage();;
			}
			//==========================================================
			// set PEA specific properties: 
			PeaControl peaControl = PeaControl.getDialog();
			
			// There are no specific properties for Image Lock PEA and File Lock PEA:
			String fileType = PeaProperties.getFileType();
			if (fileType.equals("file")) { // File Lock PEA
				return errorMessage;
			} else if (fileType.equals("image")) { // Image Lock PEA
				return  errorMessage;			
			}
			
			prop = new Properties();
			file = new File(System.getProperty("user.dir") + File.separator 
					+ "resources" + File.separator + PeaControl.getPeaPropFileName());
			if (! file.exists()) {
				// create directory
				file.getParentFile().mkdirs();
				// create file
				file.createNewFile();
			}
			outputStream = new FileOutputStream(file);
			peaControl.updatePeaPropValues(prop);//, outputStream);
			if (outputStream != null) {
				peaControl.updatePeaPropValues(prop);
				prop.store(outputStream, "Properties for specific PEA");
			} else {
				//throw new FileNotFoundException("property file '" + propFileName + "' not found in the classpath");
				new UnexpectedValueException("outputStream", "OutputStream", "file not found: " 
						+ "resources" + File.separator + propFileName).printDescription();
				errorMessage = "file not found: " + "resources" + File.separator + propFileName;
			}
/*			if (fileType.equals("text file")) { // Notebook PEA
				
			} else if (fileType.equals("file")) { // Notebook PEA
				
			} else if (fileType.equals("image")) { // Notebook PEA
				
			} else if (fileType.equals("calendar")) { // Notebook PEA
				CalendarSetting.setDefaultFileName(prop.getProperty("default_file_name"));
			} else {
				new UnexpectedValueException("fileType", "String", "does not match expected values: " + fileType).printDescription();
				errorMessage += "\nFileType does not match expected values: " + fileType;
			} */

		} catch (IOException e) {
			System.err.println("Exception when storig property values: " 
					+ new ExceptionHelper(e).getInfos());
			errorMessage += e + ", " + e.getLocalizedMessage();
		} catch (Exception e) {
			System.err.println("Exception when storig property values: " 
					+ new ExceptionHelper(e).getInfos());
			errorMessage += e + ", " + e.getLocalizedMessage();			
		} finally {
			try {
				if (outputStream != null) {
					outputStream.close();
				}
			} catch (IOException e) {
				System.err.println("PropertyHandler IOException: " + e.getMessage());
				errorMessage += e + ", " + e.getLocalizedMessage();;
			}
		}
		return errorMessage;
	}
	/**
	 * Set one property in pea.properties
	 * 
	 * @param key		the key to set
	 * @param value		the value to set
	 * 
	 * @return	null for success, an error message otherwise
	 */
	public String setOneProperty(String key, String value) {
		String errorMessage = null;
		try {
			//==================================================
			// set properties for all PEAs
			Properties prop = new Properties();
			File file = new File(System.getProperty("user.dir") + File.separator 
					+ "resources" + File.separator + propFileName);
			if (! file.exists()) {
				return "property file does not exist: " + file.getAbsolutePath();
			} else {
				inputStream = new FileInputStream(file);
						//getClass().getResourceAsStream( "/resources/" +  propFileName);
			}
			if (inputStream != null) {
				prop.load(inputStream);
			} else {// should not happen... 
				//throw new FileNotFoundException("property file '" + propFileName + "' not found in the classpath");
				new UnexpectedValueException("inputStream", "InputStream", "file not found: " + "/resources/" + propFileName).printDescription();				
				errorMessage = "file not found: " + propFileName;		
				return errorMessage;
			}
			outputStream = new FileOutputStream(file);
			
			prop.setProperty(key, value);

			if (outputStream != null) {
				prop.store(outputStream, "Properties for all PEAs");
			} else {
				//throw new FileNotFoundException("property file '" + propFileName + "' not found in the classpath");
				new UnexpectedValueException("outputStream", "OutputStream", "file not found: " + "resources" + File.separator + propFileName).printDescription();
				errorMessage = "file not found: " + "resources" + File.separator + propFileName;
			}
			try {
				if (outputStream != null) {
					outputStream.close();
				}
			} catch (IOException e) {
				System.err.println("PropertyHandler IOException: " + e.getMessage());
				errorMessage += e + ", " + e.getLocalizedMessage();;
			}

		} catch (IOException e) {
			System.err.println("Exception when storig property values: " 
					+ new ExceptionHelper(e).getInfos());
			errorMessage += e + ", " + e.getLocalizedMessage();
		} catch (Exception e) {
			System.err.println("Exception when storig property values: " 
					+ new ExceptionHelper(e).getInfos());
			errorMessage += e + ", " + e.getLocalizedMessage();			
		} finally {
			try {
				if (outputStream != null) {
					outputStream.close();
				}
			} catch (IOException e) {
				System.err.println("PropertyHandler IOException: " + e.getMessage());
				errorMessage += e + ", " + e.getLocalizedMessage();;
			}
		}
		return errorMessage;
	}
	
	/**
	 * Read properties from the property file and set the
	 * values in PeaProperties. 
	 * This method is called once when a PEA was started. 
	 * 
	 * @return	an error message or null
	 */
	public String readAndSetAllPropValues() {// throws IOException {
		String errorMessage = null;
		try {
			//==================================================
			// set properties for all PEAs
			Properties prop = new Properties();
 
			File file = new File(System.getProperty("user.dir") + File.separator + "resources" + File.separator + propFileName);
			if (file.exists()) {
				// external file in resources
				inputStream = new FileInputStream(file);
			} else {
				// inside jar: default
				System.out.println("Load default properties...");
				inputStream = getClass().getResourceAsStream( "/resources/" +  propFileName);
			} 
			if (inputStream != null) {
				prop.load(inputStream);
			} else {// should not happen... 
				//throw new FileNotFoundException("property file '" + propFileName + "' not found in the classpath");
				new UnexpectedValueException("inputStream", "InputStream", "file not found: " + "/resources/" + propFileName).printDescription();				
				errorMessage = "file not found: " + propFileName;		
				return errorMessage;
			}
			
			// should not happen:
			if (prop.getProperty("pea_color_r") == null
					&& prop.getProperty("font_size") == null
					&& prop.getProperty("frame_x") == null) {
				System.err.println("Can't read properties from file " + "resources" + File.separator + propFileName);
				new UnexpectedValueException("Property","color, font size, x", "is missing").printDescription();
				return errorMessage + ("Can't read properties from file " + "resources" + File.separator + propFileName);
			}
			// Color ===============================
			int r, g, b;
			String rString = prop.getProperty("pea_color_r");
			try {
				r = Integer.parseInt(rString);
			} catch (Exception e) {
				r = PeaProperties.getPeaColor()[0];
				new UnexpectedValueException("rString", "String", "is not digit: " + rString).printDescription();
			}
			String gString = prop.getProperty("pea_color_g");
			try {
				g = Integer.parseInt(gString);
			} catch (Exception e) {
				g = PeaProperties.getPeaColor()[1];
				new UnexpectedValueException("gString", "String", "is not digit: " + gString).printDescription();
			}
			String bString = prop.getProperty("pea_color_b");
			try {
				b = Integer.parseInt(bString);
			} catch (Exception e) {
				b = PeaProperties.getPeaColor()[2];
				new UnexpectedValueException("bString", "String", "is not digit: " + bString).printDescription();
			}
			int[] color = {r,g,b};
			PeaProperties.setPeaColor(color);
			// Dropbox access token
			String token = prop.getProperty("dropbox_access_token");
			if (token != null) {
				PeaProperties.setDropboxAccessToken(token);
			}
			
			String checkUpdates = prop.getProperty("check_updates");			
			if (checkUpdates != null ) {//&& checkUpdates.toLowerCase().equals("true")) {
				int check = 0;
				try {
					check = Integer.parseInt(checkUpdates);
					if (check == 0 || check == -1 || check == 1) {
						PeaProperties.setCheckUpdates(check);
					}
				} catch (Exception e){
					TestLog.ve(PropertyHandler.class, e.getLocalizedMessage());
				}
			}

			// main Dropbox API server host name
	/*		String api = prop.getProperty("dropbox_api");
			if (api != null) {
				PeaProperties.setDropboxApi(api);
			}
			// Dropbox API content server host name
			String content = prop.getProperty("dropbox_content");
			if (content != null) {
				PeaProperties.setDropboxContent(content);
			}
			// Dropbox web server host name
			String web = prop.getProperty("dropbox_web");
			if (web != null) {
				PeaProperties.setDropboxWeb(web);
			}
			// Dropbox notification server host name
			String notify = prop.getProperty("dropbox_notify");
			if (notify != null) {
				PeaProperties.setDropboxNotify(notify);
			}*/
			// Font size ================================
			String fontSize = prop.getProperty("font_size");
			try {
				PeaProperties.setFontSize(Integer.parseInt(fontSize));
			} catch (Exception e) {
				new UnexpectedValueException("fontSize", "String", "is not digit: " + fontSize).printDescription();
			}
			// language =================================
/*			String lang = prop.getProperty("lang");
			if (lang != null) {
				PeaProperties.setLanguagesBundle(lang);
			} else {
				PeaProperties.setLanguagesBundle(Locale.getDefault().getLanguage());
			} */
			// dimension and location ========================
			// avoid showing outside the monitor in Windows
			double screenWidth = 0;
			double screenHeight = 0;
	//		if (System.getProperty("os.name").contains("Windows")) {
				Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize(); 
				screenWidth = screenSize.getWidth();
				screenHeight = screenSize.getHeight();
	//		}
			int w,h, x, y;			
			String frameX = prop.getProperty("frame_x");
			try {
				double xD = Double.parseDouble(frameX); 
				if (xD > screenWidth) {
					xD = 50.0;
				}
				x = (int) xD;
			} catch (Exception e) {
				new UnexpectedValueException("frameX", "String", "is not digit: " + frameX).printDescription();
				x = JREProperties.getFrameLocation()[0];
			}			
			String frameY = prop.getProperty("frame_y");
			try {
				double yD = Double.parseDouble(frameY);
				if (yD > screenHeight) {
					yD = 50.0;
				}
				y = (int) yD;
			} catch (Exception e) {
				new UnexpectedValueException("frameY", "String", "is not digit: " + frameY).printDescription();
				y = JREProperties.getFrameLocation()[1];
			}		
			String frameWidth = prop.getProperty("frame_width");
			if (frameWidth !=  null) {
				try {
					w = (int) Double.parseDouble(frameWidth);
				} catch (Exception e) {
					new UnexpectedValueException("frameWidth", "String", "is not digit: " + frameWidth).printDescription();
					w = JREProperties.getFrameDim()[0];
				}
			} else {
				w = JREProperties.getFrameDim()[0];
			}
			String frameHeight = prop.getProperty("frame_height");
			if (frameHeight != null) {
				try {
					h = (int) Double.parseDouble(frameHeight);
				} catch (Exception e) {
					new UnexpectedValueException("frameHeight", "String", "is not digit: " + frameHeight).printDescription();
					h = JREProperties.getFrameDim()[1];
				}			
			} else {
				h = JREProperties.getFrameDim()[1];
			}
			int[] dimVals = {w,h};
			int[] locVals = {x,y};
			JREProperties.setFrameDim(dimVals);
			JREProperties.setFrameLocation(locVals);
			
			
			String lastSaltProp = prop.getProperty("last_salt");
			if (lastSaltProp != null) { // if no valid file found...
				PeaProperties.setLastSalt(Converter.hex2bytes(lastSaltProp));
			}
		/*	String algoParam = prop.getProperty("algo_param");
			if (algoParam != null) {
				PeaProperties.setAlgoParamString(Converter.hex2bytes(algoParam));
			} */
			String keyFile = prop.getProperty("key_file");
			if (keyFile != null) {
				TestLog.v(PropertyHandler.class, "Default key file: " + keyFile);
				PeaProperties.setDefaultKeyFileName(keyFile);
				//AlgoParamHandler.readString(keyFile, true);
				PeaProperties.setDefaultKeyFileName(keyFile);
			} else {
				TestLog.ve(PropertyHandler.class, "No default key file");
			}
			
			String defaultAlgoParam = prop.getProperty("default_algo_param");
			if (defaultAlgoParam != null) {
				TestLog.v(PropertyHandler.class, "Default algo string: " + defaultAlgoParam);
				PeaProperties.setDefaultAlgoParamString(defaultAlgoParam);
				AlgoParamHandler.readString(defaultAlgoParam, true);
				PeaProperties.setDefaultAlgoParamString(defaultAlgoParam);
			} else {
				TestLog.ve(PropertyHandler.class, "No default algo string");
			}
			//prop.setProperty("default_algo_param", defaultAlgoParam);
			
		} catch (IOException e) {
			System.err.println("PropertyHandler IOException: " + e.getMessage());
			errorMessage += e + ", " + e.getLocalizedMessage();
		} catch (Exception e) {
			System.err.println("PropertyHandler Exception: " + e.getMessage());
			errorMessage += e + ", " + e.getLocalizedMessage();			
		} finally {
			try {
				if (inputStream != null) {
					inputStream.close();
				}
			} catch (IOException e) {
				System.err.println("PropertyHandler IOException: " + e.getMessage());
				errorMessage += e + ", " + e.getLocalizedMessage();;
			}
		}
		return errorMessage;
	}
	
	/**
	 * Get one property value for a specific PEA from
	 * the property file of this PEA
	 * 
	 * @param peaType	the type of the PEA (image, file, calendar, text file)
	 * @param key		the key of this value
	 * 
	 * @return	the value of the key in the specified property file
	 */
/*	public String getSpecificPropValue(String peaType, String key) {
		String errorMessage = null;
		// There are no specific properties for Image Lock PEA and File Lock PEA:
		String fileType = PeaProperties.getFileType();
		if (fileType.equals("file")) { // File Lock PEA
			return null;
		} else if (fileType.equals("image")) { // Image Lock PEA
			return  null;			
		}
		try { 		
			
			//==========================================================
			// get PEA specific properties: 

			Properties prop = new Properties();
			File file = new File(System.getProperty("user.dir") + File.separator + 
					"resources" + File.separator + 
					PeaControl.getPeaPropFileName());
			if (file.exists()) {
				// external file in resources
				inputStream = new FileInputStream(file);
			} else {
				// inside jar: default
				System.out.println("Load default pea specific properties...");
				inputStream = getClass().getResourceAsStream("/resources/" + PeaControl.getPeaPropFileName());
			}
			if (inputStream != null) {
				prop.load(inputStream);
				return prop.getProperty(key);
			} else {
				new UnexpectedValueException("inputStream", "InputStream", "file not found: " + PeaControl.getPeaPropFileName()).printDescription();
				//errorMessage = "file not found: " + PeaControl.getPeaPropFileName();
			}
			//peaControl.setPeaPropValues(prop, inputStream);

		} catch (IOException e) {
			System.err.println("IOException: " + e.getMessage());
			errorMessage += e + ", " + e.getLocalizedMessage();
		} catch (Exception e) {
			System.err.println("Exception: " + e.getMessage());
			errorMessage += e + ", " + e.getLocalizedMessage();			
		} finally {
			try {
				inputStream.close();
			} catch (Exception e) {
				System.err.println("Exception: " + e.getMessage());
				errorMessage += e + ", " + e.getLocalizedMessage();;
			}
		}
		System.err.println(errorMessage);
		return null;

	}*/
	/**
	 * Set one property for PEA-specific property file
	 * (e.g. in calendar.properties)
	 * 
	 * @return null for success or an error message if operation failed
	 */
	public String setSpecificPropValue() {
		String errorMessage = null;
		// There are no specific properties for Image Lock PEA and File Lock PEA:
		String fileType = PeaProperties.getFileType();
		if (fileType.equals("file")) { // File Lock PEA
			return null;
		} else if (fileType.equals("image")) { // Image Lock PEA
			return  null;			
		}
		try { 		
			
			//==========================================================
			// set PEA specific properties: 
			PeaControl peaControl = PeaControl.getDialog();
			
			Properties prop = new Properties();
			File file = new File(System.getProperty("user.dir") + File.separator + 
					"resources" + File.separator + 
					PeaControl.getPeaPropFileName());
			if (file.exists()) {
				// external file in resources
				inputStream = new FileInputStream(file);
			} else {
				// inside jar: default
				System.out.println("Load default pea specific properties...");
				inputStream = getClass().getResourceAsStream("/resources/" + PeaControl.getPeaPropFileName());
			}
			if (inputStream != null) {
				prop.load(inputStream);
			} else {
				new UnexpectedValueException("inputStream", "InputStream", "file not found: " + PeaControl.getPeaPropFileName()).printDescription();
				errorMessage = "file not found: " + PeaControl.getPeaPropFileName();
			}
			peaControl.setPeaPropValues(prop, inputStream);

/*			if (fileType.equals("text file")) { // Notebook PEA
				
			} else if (fileType.equals("file")) { // Notebook PEA
				
			} else if (fileType.equals("image")) { // Notebook PEA
				
			} else if (fileType.equals("calendar")) { // Notebook PEA
				CalendarSetting.setDefaultFileName(prop.getProperty("default_file_name"));
			} else {
				new UnexpectedValueException("fileType", "String", "does not match expected values: " + fileType).printDescription();
				errorMessage += "\nFileType does not match expected values: " + fileType;
			} */

		} catch (IOException e) {
			System.err.println("PropertyHandler IOException: " + e.getMessage());
			errorMessage += e + ", " + e.getLocalizedMessage();
		} catch (Exception e) {
			System.err.println("PropertyHandler Exception: " + e.getMessage());
			errorMessage += e + ", " + e.getLocalizedMessage();			
		} finally {
			try {
				inputStream.close();
			} catch (Exception e) {
				System.err.println("PropertyHandler Exception: " + e.getMessage());
				errorMessage += e + ", " + e.getLocalizedMessage();;
			}
		}
		return errorMessage;
	}
	
	/**
	 * Set one property in the pea.property file, which
	 * is not PEA specific. 
	 * The key must not exist already.
	 * 
	 * @param propKey	the key of the property as String
	 * @param propValue	the value of the property as String
	 * 
	 * @return an error message or null
	 */
/*	public String setSingleAllProperty(String propKey, String propValue) {
		String errorMessage = null;
		try {
			//==================================================
			// set properties for all PEAs
			Properties prop = new Properties();
			File file = new File(propFileName);

			if (! file.exists()) {
				// create directory
				file.getParentFile().mkdirs();
				// create file
				file.createNewFile();
			} 
			inputStream = new FileInputStream(file);
			prop.load(inputStream);
			inputStream.close();
			outputStream = new FileOutputStream(file);
			

			Object oldProp = prop.setProperty(propKey, propValue);
			if (oldProp == null) {
				System.out.println("Created new property: " + propKey);
			} else {
				System.out.println("Property updated: " + propKey);
			}

			prop.store(outputStream,  "Properties for all PEAs");
			outputStream.close();

		} catch (IOException e) {
			errorMessage += e + ", " + e.getLocalizedMessage();	
			System.err.println(new ExceptionHelper(e).getInfos() );
		} catch (Exception e) {
			System.err.println(new ExceptionHelper(e).getInfos() );
			errorMessage += e + ", " + e.getLocalizedMessage();			
		} finally {
			try {
				if (outputStream != null) {
					outputStream.close();
				}
			} catch (IOException e) {
				System.err.println(new ExceptionHelper(e).getInfos() );
				errorMessage += e + ", " + e.getLocalizedMessage();;
			}
			try {
				if (inputStream != null) {
					inputStream.close();
				}
			} catch (IOException e) {
				System.err.println(new ExceptionHelper(e).getInfos() );
				errorMessage += e + ", " + e.getLocalizedMessage();;
			}
		}
		return errorMessage;
	}*/


/*	public static String getPropfilenameXXX() {
		return propFileName;
	}*/
	public String deleteLastSaltProperty() {
		String errorMessage = null;
		 
		try {
			//==================================================
			// set properties for all PEAs
			Properties prop = new Properties();
 
			File file = new File(System.getProperty("user.dir") + File.separator + "resources" + File.separator + propFileName);
			if (file.exists()) {
				// external file in resources
				inputStream = new FileInputStream(file);
			} else {
				// inside jar: default
				System.out.println("Load default properties...");
				inputStream = getClass().getResourceAsStream( "/resources/" +  propFileName);
			} 
			if (inputStream != null) {
				prop.load(inputStream);
			} else {// should not happen... 
				//throw new FileNotFoundException("property file '" + propFileName + "' not found in the classpath");
				new UnexpectedValueException("inputStream", "InputStream", "file not found: " + "/resources/" + propFileName).printDescription();				
				errorMessage = "file not found: " + propFileName;		
				return errorMessage;
			}
			
			// remove lastSalt
			prop.remove("last_salt");
			
			outputStream = new FileOutputStream(file);
			
			if (outputStream != null) {
				prop.store(outputStream, "Properties for all PEAs");

			} else {
				//throw new FileNotFoundException("property file '" + propFileName + "' not found in the classpath");
				new UnexpectedValueException("outputStream", "OutputStream", "file not found: " + "resources" + File.separator + propFileName).printDescription();
				errorMessage = "file not found: " + "resources" + File.separator + propFileName;
			}

		} catch (IOException e) {
			System.err.println("PropertyHandler IOException: " + e.getMessage());
			errorMessage += e + ", " + e.getLocalizedMessage();
		} catch (Exception e) {
			System.err.println("PropertyHandler Exception: " + e.getMessage());
			errorMessage += e + ", " + e.getLocalizedMessage();			
		} finally {
			try {
				if (inputStream != null) {
					inputStream.close();
				}
			} catch (IOException e) {
				System.err.println("PropertyHandler IOException: " + e.getMessage());
				errorMessage += e + ", " + e.getLocalizedMessage();;
			}
			try {
				if (outputStream != null) {
					outputStream.close();
				}
			} catch (IOException e) {
				System.err.println("PropertyHandler IOException: " + e.getMessage());
				errorMessage += e + ", " + e.getLocalizedMessage();;
			}
		}
		return errorMessage;
	}

}
